#include <View.h>
#include <malloc.h>
#include <string.h>
#include "BitmapUtil.h"
#include "Colors.h"

BBitmap* ConvertBitmap	(BBitmap*		src,
						 BRect			dst,
						 color_space	color,
						 bool			preserve_aspect_ratio)
{
	if (preserve_aspect_ratio)
	{
		BRect	bounds(src->Bounds());

		if (bounds.Width() > bounds.Height())
			dst.bottom = bounds.Height() / bounds.Width() * dst.Height();
		else
			dst.right = bounds.Width() / bounds.Height() * dst.Width();
	}

	BBitmap*	tmp = new BBitmap(dst, color, true);
	BBitmap*	final = new BBitmap(dst, color);
	BView*		view = new BView(dst, "", B_FOLLOW_ALL, B_WILL_DRAW);

	tmp->AddChild(view);
	tmp->Lock();
	view->SetHighColor((rgb_color){222, 222, 222, 255});
	view->FillRect(dst);

	view->DrawBitmap(src, src->Bounds(), dst);
	tmp->Unlock();
	memcpy(final->Bits(), tmp->Bits(), tmp->BitsLength());
	delete tmp;
	return final;
}

#define Uint32	unsigned int
#define Uint8	unsigned char

inline void uncombine_rgb( Uint32 color, Uint8 &r, Uint8 &g, Uint8 &b, Uint8 &a )
{
	r = (color >> 24) & 0xff;
	g = (color >> 16) & 0xff;
	b = (color >> 8) & 0xff;
	a = color & 0xff;
}

inline Uint32 combine_rgb( Uint8 r, Uint8 g, Uint8 b, Uint8 a )
{
	return r << 24 | g << 16 | b << 8 | a;
}

BBitmap* ConvertBitmap2	(BBitmap*		src,
						 BRect			dst,
						 color_space	color,
						 bool			preserve_aspect_ratio)
{
	BRect	bounds(src->Bounds());

	if (preserve_aspect_ratio)
	{
		if (bounds.Width() > bounds.Height())
			dst.bottom = bounds.Height() / bounds.Width() * dst.Height();
		else
			dst.right = bounds.Width() / bounds.Height() * dst.Width();
	}

	BBitmap*	final = new BBitmap(dst, color);

	if ( bounds.Width() == dst.Width() && bounds.Height() == dst.Height() )
	{
		memcpy( final->Bits(), src->Bits(), src->BitsLength() );
		return final;
	}

	float s_h = bounds.Height();
	float s_w = bounds.Width();
	int d_sx = (int)dst.left;
	int d_sy = (int)dst.top;
	int xl = final->BytesPerRow() / 4;
	int xs = src->BytesPerRow() / 4;
	float xlop = s_w / dst.Width();
	float ylop = s_h / dst.Height();
	float x, y;
	int ix, iy;
	Uint32 *srcp = (Uint32*)src->Bits();
	Uint32 *dstp = (Uint32*)final->Bits();

	for ( iy = 0, y = 0; y < s_h; iy ++, y += ylop )
	{
		for ( ix = 0, x = 0; x < s_w; ix ++, x += xlop )
		{
			int slin = (int)y * xs + (int)x;
			int lin = ((iy + d_sy) * xl + ix + d_sx);
			int ct = 1;
			register Uint8 r, g, b, a;
			uncombine_rgb( srcp[slin], r, g, b, a);
			Uint32 red = r, green = g, blue = b, alpha = a;

			if ( !(y == 0 || x == 0 || y >= ( s_h - ylop) || x >= (s_w - xlop)) )
			{		
				if ( y > 0 )
				{
					ct ++;
					uncombine_rgb( srcp[slin-xs], r, g, b, a);
					red += r;
					green += g;
					blue += b;
					alpha += a;
				}
				if ( y < ( s_h - ylop ) )
				{
					ct ++;
					uncombine_rgb( srcp[slin+xs], r, g, b, a);
					red += r;
					green += g;
					blue += b;
					alpha += a;
				}
				if ( x > 0 )
				{
					ct ++;
					uncombine_rgb( srcp[slin-1], r, g, b, a);
					red += r;
					green += g;
					blue += b;
					alpha += a;
				}
				if ( x < (s_w - xlop ) )
				{
					ct ++;
					uncombine_rgb( srcp[slin+1], r, g, b, a);
					red += r;
					green += g;
					blue += b;
					alpha += a;
				}
				if ( x > 0 && y > 0)
				{
					ct ++;
					uncombine_rgb( srcp[slin-xs-1], r, g, b, a);
					red += r;
					green += g;
					blue += b;
					alpha += a;
				}
				if ( x < (s_w - xlop) && y > 0 )
				{
					ct ++;
					uncombine_rgb( srcp[slin-xs+1], r, g, b, a);
					red += r;
					green += g;
					blue += b;
					alpha += a;
				}
				if ( x > 0 && y < ( s_h - ylop ))
				{
					ct ++;
					uncombine_rgb( srcp[slin+xs-1], r, g, b, a);
					red += r;
					green += g;
					blue += b;
					alpha += a;
				}
				if ( x < ( s_w - xlop ) && y < (s_h - ylop) )
				{
					ct ++;
					uncombine_rgb( srcp[slin+xs+1], r, g, b, a);
					red += r;
					green += g;
					blue += b;
					alpha += a;
				}
			}
			red /= ct;
			green /= ct;
			blue /= ct;
			alpha /= ct;
			dstp[lin] = combine_rgb( red, green, blue, alpha );
		}
	}

	return final;
}