#include <fs_attr.h>
#include <string.h>
#include <stdio.h>
#include <Node.h>
#include <Volume.h>
#include <Entry.h>
#include "TumbnailCache.h"
#include "basBitmapWorker.h"

basTumbnailCache *TumbnailCache;

basTumbnailCache::basTumbnailCache()
{
	TumbnailCache = this;
	caching_enabled = true;
}

basTumbnailCache::~basTumbnailCache()
{
}

void basTumbnailCache::Enable(bool b)
{
	caching_enabled = b;
}

BBitmap *basTumbnailCache::GetTumbnail( const char *file, int *x, int *y, int min )
{
	if ( caching_enabled )
	{
		entry_ref Ref;
		get_ref_for_path(file, &Ref);
		BVolume volume(Ref.device);

		if ( volume.IsPersistent() && volume.KnowsAttr() )
		{
			BNode file_node(file);

			if (file_node.InitCheck() == B_OK )
			{
				attr_info anfo;
				if ( file_node.GetAttrInfo( "meta:tumbnail:off_t", &anfo ) == B_OK )
				{
					off_t size, should_size;
					file_node.ReadAttr( "meta:tumbnail:off_t", anfo.type, 0, (void*)&size, anfo.size );
					BEntry entry(file);
					entry.GetSize( &should_size );
					if ( should_size == size )
					{
						if ( file_node.GetAttrInfo( "meta:tumbnail:real_dim", &anfo ) == B_OK )
						{
							char real_dim[32];
							file_node.ReadAttr( "meta:tumbnail:real_dim", anfo.type, 0, real_dim, anfo.size );
							sscanf( real_dim, "%ix%i", x, y );
							if ( file_node.GetAttrInfo( "meta:tumbnail:dim", &anfo ) == B_OK )
							{
								char dim[32];
								int dim_x, dim_y;
								file_node.ReadAttr( "meta:tumbnail:dim", anfo.type, 0, dim, anfo.size );
								sscanf( dim, "%ix%i", &dim_x, &dim_y );
								int dim_xx = min-1, dim_yy = min-1;
								if ( *x > *y )
									dim_yy = (int)((float)*y / (float)*x * (float)min);
								else
									dim_xx = (int)((float)*x / (float)*y * (float)min);
								if ( dim_x == dim_xx || dim_y == dim_yy )
								{
									if ( file_node.GetAttrInfo( "meta:tumbnail:data", &anfo ) == B_OK )
									{
										BBitmap *tumb = new BBitmap( BRect( 0, 0, dim_x, dim_y ), B_RGBA32 );
										file_node.ReadAttr( "meta:tumbnail:data", anfo.type, 0, tumb->Bits(), tumb->BitsLength() );
										return tumb;
									}
								}
							}
						}
					}
				}
			}
		}
	}

	BBitmap *bb = _BitmapWorker->LoadBitmap(file);
	if ( bb )
	{
		*x = bb->Bounds().IntegerWidth()+1;
		*y = bb->Bounds().IntegerHeight()+1;
		if ( *x > min || *y > min )
		{
			BBitmap *rb = _BitmapWorker->CreateTumbnail( bb, min, min, true, true, true );
			delete bb;

			if ( caching_enabled )
			{
				BNode file_node(file);

				off_t size;
				BEntry entry(file);
				entry.GetSize( &size );
				if ( file_node.WriteAttr( "meta:tumbnail:off_t", 'RAWT', 0, &size, sizeof(off_t)) == sizeof(off_t))
				{
					char real_dim[32];
					sprintf(real_dim, "%ix%i", *x, *y );
					if ( file_node.WriteAttr ("meta:tumbnail:real_dim", 'CSTR', 0, real_dim, strlen(real_dim)+1 ) == (int)(strlen(real_dim)+1))
					{
						char dim[32];
						sprintf(dim, "%ix%i", (int)rb->Bounds().IntegerWidth(), (int)rb->Bounds().IntegerHeight() );
						if ( file_node.WriteAttr ("meta:tumbnail:dim", 'CSTR', 0, dim, strlen(dim)+1 ) == (int)(strlen(dim)+1))
						{
							file_node.WriteAttr("meta:tumbnail:data", 'RAWT', 0, rb->Bits(), rb->BitsLength());
						}
					}
				}
			}

			return rb;
		}
		else
			return bb;
	}

	return NULL;
}