#include <stdio.h>
#include <Path.h>
#include <Node.h>
#include <NodeMonitor.h>
#include <string.h>
#include <ctype.h>
#include "basApplication.h"
#include "Colors.h"
#include "basWindow.h"
#include "basLocale.h"
#include "basTreeView.h"

int32 rgb_to( rgb_color &co );

int main()
{
	new basApplication();

	be_app->Run();

	delete be_app;

	return 0;
}

#define base_key	1235

bool check_if_key_is_valid( char *inname, unsigned int *in )
{
	unsigned int final_key[4];
	char name[256];

	strcpy(name, inname);

	for ( int k=0; name[k]; k ++)
		name[k] = tolower(name[k]);
	final_key[0] = base_key;
	for ( int i = 1; i<4; i++ )
	{
		final_key[i] = final_key[i-1];
		unsigned int cnt = strlen(name);
		for ( unsigned int j=0;j<strlen(name);j++)
		{
			if ( j == 0 )
				cnt *= name[j];
			else
			{
				if ( cnt % 2 )
					cnt /= name[j];
				else
					cnt *= name[j];
			}
		}
		final_key[i] *= cnt;
		while ( final_key[i] > 9999 )
		{
			final_key[i] *= 3;
			final_key[i] /= 10;
		}
	}

	final_key[0] = final_key[3];
	unsigned int cnt = strlen(name);
	for ( unsigned int j=0;j<strlen(name);j++)
	{
		if ( j == 0 )
			cnt *= name[j];
		else
		{
			if ( cnt % 2 )
				cnt /= name[j];
			else
				cnt *= name[j];
		}
	}
	final_key[0] *= cnt;
	while ( final_key[0] > 9999 )
	{
		final_key[0] *= 3;
		final_key[0] /= 10;
	}

	if ( final_key[0] == in[0] &&
		 final_key[1] == in[1] &&
		 final_key[2] == in[2] &&
		 final_key[3] == in[3] )
		 return true;
	return false;
}

basApplication::basApplication()
	: BApplication( "application/x-wnd.BeAndSee" ),
	fQuiting(false), isRegistered(false)
{
	new basLocale();
	Locale->LoadLanguages();

	BFile check_reg;
	if ( check_reg.SetTo("/boot/home/config/settings/BeAndSee_key", B_READ_ONLY) == B_OK )
	{
		char name[512];
		int z = 0;
		name[0] = 0;
		while ( (check_reg.Read( &name[z], 1 ) > 0) && name[z] != '\n' )
			z++;
		if ( name[z] == '\n' )
		{
			name[z] = 0;
			unsigned int keyz[4];
			if ( check_reg.Read( keyz, 4*sizeof(unsigned int) ) == (4*sizeof(unsigned int)))
			{
				isRegistered = check_if_key_is_valid( name, keyz );
			}
		}
	}

	GlobalSettings = new BMessage();
	BFile settings_file;
	if ( settings_file.SetTo( "/boot/home/config/settings/BeAndSee", B_READ_ONLY ) == B_OK )
	{
		GlobalSettings->Unflatten( &settings_file );
	}
	BuildDefaultSettings();

	char *setLang;
	((basApplication*)be_app)->GetSettingString( "basApplication::Language", &setLang);
	Locale->SetLanguage(setLang);

	mainWindow = new basWindow();
}

basApplication::~basApplication()
{
	delete GlobalSettings;
	delete Locale;
}

bool basApplication::QuitRequested()
{
	fQuiting = true;
	SaveSettings();
	return BApplication::QuitRequested();
}

void basApplication::MessageReceived( BMessage *msg )
{
	switch (msg->what)
	{
	case B_SIMPLE_DATA:
		RefsReceived(msg);
		break;
	case B_NODE_MONITOR:
		{
			int32 opcode;
			if ( msg->FindInt32( "opcode", &opcode) != B_OK )
				break;
			switch ( opcode )
			{
			case B_ENTRY_CREATED:
				{
					ino_t pino;
					char *name;
					node_ref nref;
					msg->FindInt32("device", &nref.device);
					msg->FindInt64("node", &nref.node);
					msg->FindInt64( "directory", (int64*)&pino );
					msg->FindString("name", (const char **)&name);
					entry_ref eref;
					eref.device = nref.device;
					eref.directory = pino;
					eref.set_name( name );
					mainWindow->EntryWasCreated(&eref, &nref);
				}
				break;
			case B_ENTRY_MOVED:
				{
					ino_t pino, topino, frompino;
					BEntry e_imat( &mainWindow->FileListView->OpenFolder );
					BDirectory imat( &e_imat );
					char *name;
					msg->FindInt64( "to directory", (int64*)&topino );
					msg->FindInt64( "from directory", (int64*)&frompino );
					node_ref nref;
					imat.GetNodeRef( &nref);
					pino = nref.node;
					msg->FindInt32("device", &nref.device);
					msg->FindInt64("node", &nref.node);
					msg->FindString("name", (const char **)&name);
					entry_ref eref;
					eref.device = nref.device;
					eref.set_name( name );
					if ( frompino == topino )
					{
						eref.directory = topino;
						mainWindow->EntryWasChanged( &eref, &nref, name );
					}
					else if ( pino == topino )
					{
						eref.directory = pino;
						mainWindow->EntryWasCreated(&eref, &nref);
					}
					else
						mainWindow->NodeWasRemoved(&nref);
				}
				break;
			case B_ENTRY_REMOVED:
				{
					node_ref nref;
					msg->FindInt32("device", &nref.device);
					msg->FindInt64("node", &nref.node);
					mainWindow->NodeWasRemoved(&nref);
				}
				break;
			case B_STAT_CHANGED:
				{
					node_ref nref;
					msg->FindInt32("device", &nref.device);
					msg->FindInt64("node", &nref.node);
					mainWindow->EntryWasChanged( NULL, &nref, NULL );
				}
				break;
			case B_DEVICE_UNMOUNTED:
				mainWindow->TreeView->RemoveVolume( (dev_t)msg->FindInt32("device") );
				break;
			case B_DEVICE_MOUNTED:
				mainWindow->TreeView->AddVolume( (dev_t)msg->FindInt32("new device") );
				break;
			}
		}
		break;
	}
}

void basApplication::ReadyToRun()
{
}

void basApplication::RefsReceived( BMessage *msg )
{
	entry_ref ref;
	if ( msg->FindRef("refs", &ref) == B_NO_ERROR )
	{
		BEntry entry(&ref, true);
		mainWindow->Lock();
		mainWindow->DoInvokeFunction(&entry);
		mainWindow->TreeView->GoTo(ref);
		mainWindow->Unlock();
	}
}

void basApplication::SetSettingBool( const char *field, bool b)
{
	type_code tc;
	if ( GlobalSettings->GetInfo( field, &tc ) != B_NAME_NOT_FOUND )
	{
		if ( tc != B_BOOL_TYPE )
		{
			GlobalSettings->RemoveData( field );
			GlobalSettings->AddBool( field, b );
		}
		else
		{
			GlobalSettings->ReplaceBool( field, b );
		}
	}
	else
		GlobalSettings->AddBool( field, b );
}

void basApplication::SetSettingInt( const char *field, int b)
{
	type_code tc;
	if ( GlobalSettings->GetInfo( field, &tc ) != B_NAME_NOT_FOUND )
	{
		if ( tc != B_INT32_TYPE )
		{
			GlobalSettings->RemoveData( field );
			GlobalSettings->AddInt32( field, b );
		}
		else
		{
			GlobalSettings->ReplaceInt32( field, b );
		}
	}
	else
		GlobalSettings->AddInt32( field, b );
}

void basApplication::SetSettingString( const char *field, char *b )
{
	type_code tc;
	if ( GlobalSettings->GetInfo( field, &tc ) != B_NAME_NOT_FOUND )
	{
		if ( tc != B_STRING_TYPE )
		{
			GlobalSettings->RemoveData( field );
			GlobalSettings->AddString( field, b );
		}
		else
		{
			GlobalSettings->ReplaceString( field, b );
		}
	}
	else
		GlobalSettings->AddString( field, b );
}

void basApplication::SetSettingFloat( const char *field, float b)
{
	type_code tc;
	if ( GlobalSettings->GetInfo( field, &tc ) != B_NAME_NOT_FOUND )
	{
		if ( tc != B_FLOAT_TYPE )
		{
			GlobalSettings->RemoveData( field );
			GlobalSettings->AddFloat( field, b );
		}
		else
		{
			GlobalSettings->ReplaceFloat( field, b );
		}
	}
	else
		GlobalSettings->AddFloat( field, b );
}

void basApplication::SetSettingPoint( const char *field, BPoint p )
{
	type_code tc;
	if ( GlobalSettings->GetInfo( field, &tc ) != B_NAME_NOT_FOUND )
	{
		if ( tc != B_POINT_TYPE )
		{
			GlobalSettings->RemoveData( field );
			GlobalSettings->AddPoint( field, p );
		}
		else
		{
			GlobalSettings->ReplacePoint( field, p );
		}
	}
	else
		GlobalSettings->AddPoint( field, p );
}

int32 rgb_to( const rgb_color &co )
{
	return (co.red << 24) | (co.green << 16) | (co.blue << 8) | (co.alpha);
}

void basApplication::BuildDefaultSettings()
{
	bool oldver = false;
	int ver;
	GetSettingInt( "Config:Version", &ver);
	if ( ver < 1 )
		oldver = true;

	struct
	{
		type_code tc;
		void inner_int32( BMessage *m, char *s, int32 val )
		{
			if ( m->GetInfo( s, &tc) != B_OK )
			{
				m->AddInt32( s, val );
			}
		}
		void inner_float( BMessage *m, char *s, float val )
		{
			if ( m->GetInfo( s, &tc) != B_OK )
			{
				m->AddFloat( s, val );
			}
		}
		void inner_string( BMessage *m, char *s, char *val )
		{
			if ( m->GetInfo( s, &tc) != B_OK )
			{
				m->AddString( s, val );
			}
		}
		void inner_bool( BMessage *m, char *s, bool val )
		{
			if ( m->GetInfo( s, &tc) != B_OK )
			{
				m->AddBool( s, val );
			}
		}
		void inner_point( BMessage *m, char *s, BPoint val )
		{
			if ( m->GetInfo( s, &tc) != B_OK )
			{
				m->AddPoint( s, val );
			}
		}
	} local;

	local.inner_int32( GlobalSettings, "basTumbnailView::TumbnailSize", 100 );
	local.inner_float( GlobalSettings, "basWindow::WindowFrame::x", 100 );
	local.inner_float( GlobalSettings, "basWindow::WindowFrame::y", 100 );
	local.inner_float( GlobalSettings, "basWindow::WindowFrame::w", 600 );
	local.inner_float( GlobalSettings, "basWindow::WindowFrame::h", 400 );
	local.inner_string( GlobalSettings, "basWindow::DefaultPlace", "/boot/home/" );
	local.inner_bool( GlobalSettings, "basWindow::ShowFolderMenu", false );
	local.inner_bool( GlobalSettings, "basWindow::ShowFoldersFirst", true );
	local.inner_bool( GlobalSettings, "basWindow::ShowImagesOnly", false );
	local.inner_bool( GlobalSettings, "basWindow::ShowFolderView", false );
	local.inner_bool( GlobalSettings, "basWindow::ShowTumbnailView", true );
	local.inner_point( GlobalSettings, "split:pos", BPoint(250, 250) );
	local.inner_bool( GlobalSettings, "basWindow::ShowNameColumn", true );
	local.inner_bool( GlobalSettings, "basWindow::ShowSizeColumn", true );
	local.inner_bool( GlobalSettings, "basWindow::ShowModifiedTimeColumn", false );
	local.inner_bool( GlobalSettings, "basWindow::ShowCreatedTimeColumn", false );
	local.inner_int32( GlobalSettings, "basTumbnailView::ViewColor", rgb_to(BeBackgroundGrey));
	local.inner_int32( GlobalSettings, "basTumbnailView::TumbnailBackground", rgb_to(BeInactiveControlGrey));
	local.inner_int32( GlobalSettings, "basTumbnailView::TumbnailHover", rgb_to(BeButtonGrey));
	local.inner_int32( GlobalSettings, "basTumbnailView::leftButtonBehaviour", 4);
	local.inner_int32( GlobalSettings, "basTumbnailView::middleButtonBehaviour", 3);
	local.inner_int32( GlobalSettings, "basTumbnailView::rightButtonBehaviour", 2);
	local.inner_int32( GlobalSettings, "basTumbnailView::doubleButtonBehaviour", 1);
	local.inner_int32( GlobalSettings, "basTumbnailView::TumbnailViewWheelMouseBehaviour", 1);
	local.inner_int32( GlobalSettings, "basFileListView::selectBehaviour", 0);
	local.inner_int32( GlobalSettings, "basFileListView::invokeBehaviour", 2);
	local.inner_int32( GlobalSettings, "basFileListView::FileListViewWheelMouseBehaviour", 1);
	local.inner_bool( GlobalSettings, "basTumbnailView::useSmoothScaling", false );
	local.inner_bool( GlobalSettings, "basApplication::TumbnailCachingEnabled", true );
	local.inner_int32( GlobalSettings, "basPictureWindow::BackColor", rgb_to(BeBackgroundGrey) );
	local.inner_string( GlobalSettings, "basApplication::prefered_app", "application/x-sum-becasso" );

	if ( oldver )
	{
		SetSettingBool( "basWindow::ShowFolderMenu", false );
		SetSettingBool( "basWindow::ShowFoldersFirst", true );
		SetSettingBool( "basWindow::ShowImagesOnly", false );
		SetSettingBool( "basWindow::ShowFolderView", false );
		SetSettingBool( "basWindow::ShowTumbnailView", true );
		SetSettingInt( "basTumbnailView::leftButtonBehaviour", 4);
		SetSettingInt( "basTumbnailView::middleButtonBehaviour", 3);
		SetSettingInt( "basTumbnailView::rightButtonBehaviour", 2);
		SetSettingInt( "basTumbnailView::doubleButtonBehaviour", 1);
		SetSettingInt( "basFileListView::selectBehaviour", 0);
		SetSettingInt( "basFileListView::invokeBehaviour", 2);
		SetSettingInt( "basFileListView::FileListViewWheelMouseBehaviour", 1);
		SetSettingBool( "basTumbnailView::useSmoothScaling", true );
		SetSettingInt( "Config:Version", 1);
	}
}

void basApplication::SaveSettings()
{
	BFile settings_file("/boot/home/config/settings/BeAndSee", B_WRITE_ONLY | B_CREATE_FILE );
	GlobalSettings->Flatten( &settings_file );
}

void basApplication::GetSettingBool( const char *field, bool *b)
{
	GlobalSettings->FindBool( field, b );
}

void basApplication::GetSettingInt( const char *field, int *i)
{
	GlobalSettings->FindInt32( field, (int32*)i );
}

void basApplication::GetSettingString( const char *field, char **s )
{
	GlobalSettings->FindString( field, (const char **)s );
}

void basApplication::GetSettingFloat( const char *field, float *f)
{
	GlobalSettings->FindFloat( field, f );
}

void basApplication::GetSettingPoint( const char *field, BPoint *p)
{
	GlobalSettings->FindPoint( field, p );
}
