#include <Path.h>
#include <Directory.h>
#include <Entry.h>
#include <string.h>
#include <File.h>
#include "basBDCPInterface.h"

basBDCP::basBDCP( const char *p )
	: _Caps(0)
{
	iID = load_add_on(p);
	get_image_symbol(iID, "getPluginVersion", B_SYMBOL_TYPE_TEXT, (void **)&getPluginVersion);
	get_image_symbol(iID, "openCamera", B_SYMBOL_TYPE_TEXT, (void **)&openCamera);
	get_image_symbol(iID, "closeCamera", B_SYMBOL_TYPE_TEXT, (void **)&closeCamera);
	get_image_symbol(iID, "getNumberofPics", B_SYMBOL_TYPE_TEXT, (void **)&getNumberofPics);
	get_image_symbol(iID, "setCurrentPicture", B_SYMBOL_TYPE_TEXT, (void **)&setCurrentPicture);
	get_image_symbol(iID, "downloadPicture", B_SYMBOL_TYPE_TEXT, (void **)&downloadPicture);
	get_image_symbol(iID, "getSupportedCameras", B_SYMBOL_TYPE_TEXT, (void **)&getSupportedCameras);

	if ( get_image_symbol(iID, "getThumbnail", B_SYMBOL_TYPE_TEXT, (void **)&getThumbnail) == B_OK )
	{
		_Caps |= _GET_THUMBNAIL;
	}
	if ( get_image_symbol(iID, "getImageName", B_SYMBOL_TYPE_TEXT, (void **)&getImageName) == B_OK )
	{
		_Caps |= _GET_IMAGE_NAME;
	}
	if ( get_image_symbol(iID, "getImageDate", B_SYMBOL_TYPE_TEXT, (void **)&getImageDate) == B_OK )
	{
		_Caps |= _GET_IMAGE_DATE;
	}
	if ( get_image_symbol(iID, "getImageSize", B_SYMBOL_TYPE_TEXT, (void **)&getImageSize) == B_OK )
	{
		_Caps |= _GET_IMAGE_SIZE;
	}


	getPluginVersion(versionInfo);
	getSupportedCameras(supportedCameras);
}

basBDCP::~basBDCP()
{
	unload_add_on(iID);
}

_version_info &basBDCP::VersionInfo()
{
	return versionInfo;
}

vector<string> &basBDCP::SupportedCameras()
{
	return supportedCameras;
}

bool basBDCP::OpenCamera()
{
	return openCamera() == B_OK;
}

bool basBDCP::CloseCamera()
{
	return closeCamera() == B_OK;
}

bool basBDCP::GetNumberOfPics( int &i )
{
	return getNumberofPics(i) == B_OK;
}

bool basBDCP::DownloadPic( int pic, BPath &path )
{
	if (!SetCurrentPic(pic))
		return false;
	return downloadPicture(path) == B_OK;
}

bool basBDCP::SetCurrentPic(int i)
{
	return setCurrentPicture(i) == B_OK;
}

bool basBDCP::GetThumbnail( int pic, BBitmap* &bitmap )
{
	if ( (Caps() & _GET_THUMBNAIL) == _GET_THUMBNAIL )
	{
		if (!SetCurrentPic(pic))
			return false;
		return getThumbnail(bitmap) == B_OK;
	}
	else
		return false;
}

bool basBDCP::GetImageName( int pic, char*& name )
{
	if ( (Caps() & _GET_IMAGE_NAME) == _GET_IMAGE_NAME )
	{
		if (!SetCurrentPic(pic))
			return false;
		return getImageName(name) == B_OK;
	}
	else
		return false;
}

bool basBDCP::GetImageDate( int pic, char*& date )
{
	if ( (Caps() & _GET_IMAGE_DATE) == _GET_IMAGE_DATE )
	{
		if (!SetCurrentPic(pic))
			return false;
		return getImageName(date) == B_OK;
	}
	else
		return false;
}

bool basBDCP::GetImageSize( int pic, int& size )
{
	if ( (Caps() & _GET_IMAGE_SIZE) == _GET_IMAGE_SIZE )
	{
		if (!SetCurrentPic(pic))
			return false;
		return getImageSize(size) == B_OK;
	}
	else
		return false;
}

unsigned int basBDCP::Caps()
{
	return _Caps;
}

const char * const path0 = "/boot/home/config/lib/BDCP_Plugins/";
basBDCPInterface *BDCPInterface;

basBDCPInterface::basBDCPInterface()
{
	BDCPInterface = this;

	char *path = new char [strlen(path0)+1];
	strcpy(path, path0);
	SearchPath.push_back( path );
}

basBDCPInterface::~basBDCPInterface()
{
	for ( unsigned int i=0; i<SearchPath.size(); i++ )
		delete SearchPath[i];
	for ( unsigned int i=0; i<PluginList.size(); i++)
		delete PluginList[i];
}

void basBDCPInterface::AddToSearchPath( char *path )
{
	SearchPath.push_back( path );
}

void basBDCPInterface::LoadAll()
{
	for ( unsigned int i=0; i<SearchPath.size(); i++)
	{
		LoadFromPath(SearchPath[i]);
	}
}

void basBDCPInterface::LoadFromPath( const char *path )
{
	BDirectory dir( path );
	BEntry entry;

	while ( dir.GetNextEntry( &entry, true) == B_OK )
	{
		if ( !entry.IsDirectory() )
		{
			BPath path(&entry);
			image_id iid = load_add_on( path.Path() );
			if ( iid >= 0 )
			{
				int (*get_BDCP_API_Revision)(void);
				if ( get_image_symbol( iid, "get_BDCP_API_Revision", B_SYMBOL_TYPE_TEXT, (void**)&get_BDCP_API_Revision) == B_OK )
				{
					basBDCP *pl = new basBDCP( path.Path() );
					PluginList.push_back(pl);
				}
				else
					unload_add_on(iid);
			}
		}
	}
}

void basBDCPInterface::LoadDefCams()
{
	BMessage msg;
	BFile file("/boot/home/config/settings/BeAndSee_Cameras", B_READ_ONLY);

	if ( file.InitCheck() == B_OK)
	{
		msg.Unflatten(&file);
		char *_string;
		for ( int i=0; msg.FindString("_cam_name", i, (const char **)&_string) == B_OK; i++)
		{
			camDef *def = new camDef;
			def->DefName = _string;
			char *_cam = NULL;
			msg.FindString("_cam_cam", i, (const char **)&_cam);
			if ( _cam == NULL )
				delete def;
			else
			{
				def->DefCam = _cam;
				for ( unsigned int j=0; j<PluginList.size(); j++ )
				{
					vector<string> &supcams = PluginList[j]->SupportedCameras();
					for ( unsigned int k=0; k<supcams.size(); k++)
					{
						if ( supcams[k] == _cam )
						{
							def->plugIndex = j;
							goto out;
						}
					}
				}
out:;
				DefCams.push_back(def);
			}
		}
	}
}

void basBDCPInterface::AddCamDef( const char *name, const char *cam )
{
	BMessage msg;

	for ( unsigned int i=0; i<DefCams.size(); i++ )
	{
		msg.AddString( "_cam_name", DefCams[i]->DefName.c_str() );
		msg.AddString( "_cam_cam", DefCams[i]->DefCam.c_str() );
	}

	msg.AddString( "_cam_name", name);
	msg.AddString( "_cam_cam", cam);

	camDef *def = new camDef;
	def->DefName = name;
	def->DefCam = cam;
	DefCams.push_back(def);

	BFile file("/boot/home/config/settings/BeAndSee_Cameras", B_WRITE_ONLY|B_CREATE_FILE);
	file.SetSize(0);
	msg.Flatten(&file);
}

void basBDCPInterface::RemoveCamDef( const char *name )
{
	int num = -1;
	for ( unsigned int i=0; i<DefCams.size(); i++ )
	{
		if ( DefCams[i]->DefName == name )
		{
			num = i;
			break;
		}
	}

	DefCams.erase( DefCams.begin() + num );

	BMessage msg;

	for ( unsigned int i=0; i<DefCams.size(); i++ )
	{
		msg.AddString( "_cam_name", DefCams[i]->DefName.c_str() );
		msg.AddString( "_cam_cam", DefCams[i]->DefCam.c_str() );
	}

	BFile file("/boot/home/config/settings/BeAndSee_Cameras", B_WRITE_ONLY|B_CREATE_FILE);
	file.SetSize(0);
	msg.Flatten(&file);
}