#ifndef __basBDCPInterface_h__
#define __basBDCPInterface_h__

#include <Bitmap.h>
#include <vector>
#include <string>

struct  _version_info { 
        uint32  major; 
        uint32  middle; 
        uint32  minor; 
        uint32  variety; 
        uint32  internal; 
        char    short_info[64]; 
        char    long_info[256]; 
};

enum pExtraCaps
{
	_DELETE_PICTURE = 1,
	_TAKE_PICTURE = 2,
	_GET_IMAGE_NAME = 4,
	_GET_IMAGE_SIZE = 8,
	_GET_IMAGE_DATE = 16,
	_CONFIGURE_PLUGIN = 32,
	_SET_MESSAGE_TARGET = 64,
	_GET_THUMBNAIL = 128
};

class basBDCP
{
public:
	basBDCP( const char * );
	~basBDCP();

	_version_info &VersionInfo();
	vector<string> &SupportedCameras();

	bool OpenCamera();
	bool CloseCamera();

	bool GetNumberOfPics( int & );
	bool DownloadPic( int pic, BPath & );

	bool GetThumbnail( int pic, BBitmap * &);

	bool GetImageName( int pic, char * & );
	bool GetImageSize( int pic, int &size );
	bool GetImageDate( int pic, char * & );

	unsigned int Caps();

protected:
	bool SetCurrentPic( int );

	_version_info versionInfo;
	vector<string> supportedCameras;

	image_id iID;

	status_t (*get_BDCP_API_Revision)();
	status_t (*getPluginVersion)(_version_info &);
	status_t (*openCamera)();
	status_t (*closeCamera)();
	status_t (*getNumberofPics)(int &);
	status_t (*setCurrentPicture)(int);
	status_t (*downloadPicture)(BPath);
	void (*getSupportedCameras)(vector<string> &);
	status_t (*getThumbnail)(BBitmap *&);
	status_t (*getImageName)(char* &name);
	status_t (*getImageDate)(char* &date);
	status_t (*getImageSize)(int &size);

	unsigned int _Caps;
};

typedef struct
{
	string DefName;
	string DefCam;
	int plugIndex;
} camDef;

class basBDCPInterface
{
public:
	basBDCPInterface();
	~basBDCPInterface();

	void AddToSearchPath( char * );

	void LoadAll();
	void LoadDefCams();

	vector<basBDCP*> &GetPluginList() { return PluginList; }
	vector<camDef*> &GetDefCams() { return DefCams; }

	void AddCamDef( const char *, const char * );
	void RemoveCamDef( const char * );

private:
	void LoadFromPath( const char * );

	vector<char *> SearchPath;
	vector<basBDCP*> PluginList;
	vector<camDef*> DefCams;
};

extern basBDCPInterface *BDCPInterface;

#endif