#include <View.h>
#include <Entry.h>
#include <Path.h>
#include <TranslationKit.h>
#include "basBitmapWorker.h"
#include "BitmapUtil.h"

basBitmapWorker *_BitmapWorker;

basBitmapWorker::basBitmapWorker()
{
	_BitmapWorker = this;
}

BBitmap *basBitmapWorker::CreateTumbnail( entry_ref *ref, int tumbX, int tumbY, bool keep_aspect_ratio, bool anti_alised, bool wasted_space )
{
	BBitmap *b = LoadBitmap(ref);
	BBitmap *tb = CreateTumbnail(b, tumbX, tumbY, keep_aspect_ratio, anti_alised, wasted_space);
	delete b;
	return tb;
}

BBitmap *basBitmapWorker::CreateTumbnail( const char *p, int tumbX, int tumbY, bool keep_aspect_ratio, bool anti_alised, bool wasted_space )
{
	BBitmap *b = LoadBitmap(p);
	BBitmap *tb = CreateTumbnail(b, tumbX, tumbY, keep_aspect_ratio, anti_alised, wasted_space);
	delete b;
	return tb;
}

inline void uncombine_rgb( uint32 color, uint8 &r, uint8 &g, uint8 &b, uint8 &a )
{
	r = (color >> 24) & 0xff;
	g = (color >> 16) & 0xff;
	b = (color >> 8) & 0xff;
	a = color & 0xff;
}

inline uint32 combine_rgb( uint8 r, uint8 g, uint8 b, uint8 a )
{
	return r << 24 | g << 16 | b << 8 | a;
}

BBitmap *basBitmapWorker::CreateTumbnail( BBitmap *b, int tumbX, int tumbY, bool keep_aspect_ratio, bool anti_alised, bool wasted_space )
{
	if ( !anti_alised )
		return ConvertBitmap(b, BRect(0, 0, tumbX-1, tumbY-1), B_RGBA32, keep_aspect_ratio );
	else
		return ConvertBitmap2(b, BRect(0, 0, tumbX-1, tumbY-1), B_RGBA32, keep_aspect_ratio );
}

BBitmap *basBitmapWorker::LoadBitmap( entry_ref *ref )
{
	BEntry entry(ref);
	BPath path(&entry);
	return LoadBitmap(path.Path());
}

BBitmap *basBitmapWorker::LoadBitmap( const char *p )
{
	return BTranslationUtils::GetBitmap(p);
}
