#include <Message.h>
#include <cstring>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <View.h>
#include <ScrollView.h>
#include <Alert.h>
#include "basCameraConfigWindow.h"
#include "basLocale.h"
#include "TextDefs.h"
#include "basBDCPInterface.h"
#include "basCommon.h"
#include "Colors.h"

basCameraConfigAddWindow::basCameraConfigAddWindow( basCameraConfigWindow *parent, const char *name, const char *cam, int arg)
	: BWindow( BRect(100, 100, 400, 195), Locale->Text(TEXT_ADD_CAMERA, "Add Camera ..."), B_TITLED_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL, B_NOT_RESIZABLE|B_NOT_ZOOMABLE ),
	Parent(parent), Arg(arg)
{
	if ( name )
		ArgName = name;

	BView *bview = new BView(Bounds(), NULL, B_FOLLOW_ALL, 0);
	bview->SetViewColor(BeBackgroundGrey);

	vector<basBDCP*> &pluginList = BDCPInterface->GetPluginList();

	for ( unsigned int i=0; i<pluginList.size(); i++)
	{
		vector<string> &supCams = pluginList[i]->SupportedCameras();
		unsigned int _k = supCams.size();
		for ( unsigned int k=0; k<_k; k++)
		{
			bool found = false;
			for ( unsigned int j=0; j<camList.size(); j++)
			{
				if ( camList[j]->String == supCams[k] )
				{
					camList[j]->Index.push_back(i);
					found = true;
				}
			}
			if ( !found )
			{
				camString *cs = new camString;
				cs->String = supCams[k];
				cs->Index.push_back(i);
				camList.push_back( cs );
			}
		}
	}

	BMenu *menu = new BPopUpMenu("");
	menu->SetRadioMode(true);
	for ( unsigned int i=0; i<camList.size(); i++)
	{
		BMenuItem *item = new BMenuItem( camList[i]->String.c_str(), new BMessage(B_CAMERA_SELECT) );
		menu->AddItem( item );
		if ( cam && cam == camList[i]->String )
			item->SetMarked(true);
	}

	CamListField = new BMenuField( BRect(5, 5, 295, 30 ), NULL, Locale->Text(TEXT_DIGITAL_CAMERA_LIST, "Digital Camera List"), menu );
	bview->AddChild( CamListField );

	CamNameTC = new BTextControl( BRect(5, 35, 295, 60), NULL, Locale->Text(TEXT_NAME, "Name"), name?name:"", NULL );
	bview->AddChild( CamNameTC );

	OkButton = new BButton( BRect( 235, 65, 295, 90), NULL, Locale->Text(TEXT_OK, "Ok"), new BMessage(B_BUTTON_OK));
	bview->AddChild(OkButton);

	CancelButton = new BButton( BRect( 170, 65, 230, 90), NULL, Locale->Text(TEXT_CANCEL, "Cancel"), new BMessage(B_BUTTON_CANCEL));
	bview->AddChild(CancelButton);

	AddChild(bview);

	Show();
}

void basCameraConfigAddWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what)
	{
		case B_CAMERA_SELECT:
			{
				int32 index = msg->FindInt32("index");
				CamNameTC->SetText( CamListField->Menu()->ItemAt(index)->Label() );
			}
			break;
		case B_BUTTON_CANCEL:
			PostMessage(B_QUIT_REQUESTED);
			break;
		case B_BUTTON_OK:
			{
				if ( CamListField->Menu()->FindMarked() != NULL )
				{
					vector<camDef*> camDefs = BDCPInterface->GetDefCams();
					bool conti = true;
					for (unsigned int i=0; i<camDefs.size(); i++ )
					{
						if ( camDefs[i]->DefName == CamNameTC->Text() )
						{
							(new BAlert(Locale->Text(TEXT_WARNING, "Warning"), Locale->Text(TEXT_THERE_IS_ALREADY_A_CAMERA_DEFINED_WITH_THE_NAME_YOU_WROTE_PLEASE_CHOOSE_ANOTHER_NAME, "There is already a camera defined with the name you wrote. Please choose another name."), Locale->Text(TEXT_OK, "Ok") ) )->Go();
							conti = false;
						}
					}
					if ( conti )
					{
						if ( Arg == 1 )
							BDCPInterface->RemoveCamDef( ArgName.c_str() );
						BDCPInterface->AddCamDef( CamNameTC->Text(), CamListField->Menu()->FindMarked()->Label() );
						PostMessage(B_QUIT_REQUESTED);
						BMessage *nm = new BMessage(B_ADD_FINISHED);
						nm->AddInt32( "arg", Arg );
						Parent->PostMessage(nm);
					}
				}
				else
				{
					(new BAlert(Locale->Text(TEXT_WARNING, "Warning"), Locale->Text(TEXT_YOU_MUST_SELECT_A_DIGITAL_CAMERA, "You must select a Digital Camera."), Locale->Text(TEXT_OK, "Ok") ) )->Go();
				}
			}
			break;
		default:
			BWindow::MessageReceived(msg);
	}
}

bool basCameraConfigAddWindow::QuitRequested()
{
	return true;
}

basCameraConfigWindow::basCameraConfigWindow()
	: BWindow( BRect(100, 100, 400, 400), Locale->Text(TEXT_BEANDSEE_CAMERA_CONFIG, "BeAndSee Camera Config"), B_TITLED_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL, B_NOT_RESIZABLE|B_NOT_ZOOMABLE ),
	Changed(false)
{
	BView *bview = new BView(Bounds(), NULL, B_FOLLOW_ALL, 0);
	bview->SetViewColor(BeBackgroundGrey);

	configureCamsSView = new BStringView( BRect( 5, 5, 225, 20), NULL, Locale->Text( TEXT_CONFIGURED_CAMERAS, "Configured Cameras") );
	bview->AddChild(configureCamsSView);

	configuredCamsLView = new BListView( BRect( 5, 25, 225 - B_V_SCROLL_BAR_WIDTH, 295), NULL, B_SINGLE_SELECTION_LIST );

	vector<camDef*> &camDefs = BDCPInterface->GetDefCams();
	for ( unsigned int i=0; i<camDefs.size(); i++ )
	{
		configuredCamsLView->AddItem( new BStringItem( camDefs[i]->DefName.c_str()));
	}

	configuredCamsLView->SetSelectionMessage(new BMessage(B_MESSAGE_SELECT));
	configuredCamsLView->SetInvocationMessage(new BMessage(B_EDIT));

	bview->AddChild( new BScrollView( NULL, configuredCamsLView, 0, 0, false, true ) );

	AddButton = new BButton( BRect( 230, 25, 295, 50), NULL, Locale->Text(TEXT_ADD_DOTDOTDOT, "Add..."), new BMessage(B_ADD) );
	bview->AddChild(AddButton);

	RemoveButton = new BButton( BRect( 230, 55, 295, 80), NULL, Locale->Text(TEXT_REMOVE, "Remove"), new BMessage(B_REMOVE) );
	RemoveButton->SetEnabled(false);
	bview->AddChild(RemoveButton);

	EditButton = new BButton( BRect( 230, 85, 295, 110), NULL, Locale->Text(TEXT_EDIT, "Edit"), new BMessage(B_EDIT) );
	EditButton->SetEnabled(false);
	bview->AddChild(EditButton);

	AddChild(bview);

	Show();
}

void basCameraConfigWindow::MessageReceived( BMessage *msg )
{
	switch ( msg->what )
	{
		case B_MESSAGE_SELECT:
			RemoveButton->SetEnabled( configuredCamsLView->CurrentSelection() >= 0 );
			EditButton->SetEnabled( configuredCamsLView->CurrentSelection() >= 0 );
			break;
		case B_ADD:
			new basCameraConfigAddWindow(this);
			break;
		case B_EDIT:
			{
				vector<camDef*> &camDefs = BDCPInterface->GetDefCams();
				int i = configuredCamsLView->CurrentSelection();
				
				new basCameraConfigAddWindow( this, camDefs[i]->DefName.c_str(), camDefs[i]->DefCam.c_str(), 1 );
			}
			break;
		case B_ADD_FINISHED:
			{
				BStringItem *item;
				while ( (item = (BStringItem*)configuredCamsLView->RemoveItem((int32)0) ) )
				{
					delete item;
				}

				vector<camDef*> &camDefs = BDCPInterface->GetDefCams();
				for ( unsigned int i=0; i<camDefs.size(); i++ )
				{
					configuredCamsLView->AddItem( new BStringItem( camDefs[i]->DefName.c_str()));
				}

				Changed = true;
			}
			break;
		case B_REMOVE:
			{
				BDCPInterface->RemoveCamDef( ((BStringItem*)configuredCamsLView->ItemAt( configuredCamsLView->CurrentSelection() ))->Text() );

				BStringItem *item;
				while ( (item = (BStringItem*)configuredCamsLView->RemoveItem((int32)0) ) )
				{
					delete item;
				}

				vector<camDef*> &camDefs = BDCPInterface->GetDefCams();
				for ( unsigned int i=0; i<camDefs.size(); i++ )
				{
					configuredCamsLView->AddItem( new BStringItem( camDefs[i]->DefName.c_str()));
				}

				Changed = true;
			}
			break;
		default:
			BWindow::MessageReceived(msg);
	}
}

bool basCameraConfigWindow::QuitRequested()
{
	if ( Changed )
	{
		(new BAlert(Locale->Text(TEXT_WARNING, "Warning"), Locale->Text(TEXT_YOU_MUST_RESTART_BEANDSEE_FOR_MODIFICATIONS_TO_TAKE_EFFECT, "You must restart BeAndSee for modifications to take effect."), Locale->Text(TEXT_OK, "Ok")))->Go();
	}

	return true;
}