#include <Path.h>
#include <Volume.h>
#include <VolumeRoster.h>
#include <Node.h>
#include <NodeInfo.h>
#include <stdio.h>
#include <SymLink.h>
#include <Region.h>
#include <string.h>
#include <Window.h>
#include "basFileListView.h"
#include "basApplication.h"
#include "basCommon.h"
#include "basLocale.h"
#include "TextDefs.h"

void Entry_StringSize( BEntry *, off_t *, char * );

#ifdef NEXT_GEN

bool Entry_MayIReadIt( BEntry *entry )
{
	mode_t perm;
	if ( entry->GetPermissions( &perm ) != B_OK )
		return false;
	if ( !(perm & S_IROTH) && !(perm & S_IRGRP) && !(perm & S_IRUSR) )
		return false;
	if ( perm & S_IROTH )
		return true;
	else
	{
		if ( perm & S_IRGRP )
		{
			gid_t my_group = getgid();
			gid_t f_group;
			entry->GetGroup(&f_group);
			if ( my_group == f_group )
				return true;
		}

		if ( perm & S_IRUSR )
		{
			uid_t me = getuid();
			uid_t f_who;
			entry->GetOwner(&f_who);
			if ( me == f_who )
				return true;
		}

		return false;
	}
}

#endif

#define ITEM_HEIGHT	20

basFileListItem::basFileListItem( basFileListView *parent, BEntry *entry, node_ref &nref, const char *name )
: CLVListItem(0, false, false, ITEM_HEIGHT), Tumbnail(0), Parent(parent)
{
	entry->GetRef(&Ref);
	entry->GetCreationTime(&createdTime);
	entry->GetModificationTime(&modifiedTime);
	Node = nref;
	if ( name )
	{
		Name = new char[strlen(name)+1];
		strcpy(Name, name);
	}
	else
	{
		Name = new char[strlen(Ref.name)+1];
		strcpy(Name, Ref.name);
	}

	BNode node(entry);
	BNodeInfo nodeinfo(&node);
	Tumbnail = new BBitmap(BRect( 0, 0, B_MINI_ICON-1, B_MINI_ICON-1), B_CMAP8);
	nodeinfo.GetTrackerIcon(Tumbnail, B_MINI_ICON);

	if ( entry->IsDirectory() )
	{
		strcpy(sSize, "-");
		Size = 0;
	}
	else
	{
		Entry_StringSize( entry, &Size, sSize );
	}

}

basFileListItem::~basFileListItem()
{
	if ( Tumbnail )
		delete Tumbnail;
	if ( Name )
		delete Name;
}

#include "Colors.h"

void basFileListItem::DrawItemColumn(BView *owner, BRect draw_rect, int32 column_index, bool complete)
{
	owner->SetDrawingMode(B_OP_COPY);

	if ( IsSelected() )
	{
		owner->SetHighColor(BeListSelectGrey);
		owner->FillRect(draw_rect);
		owner->SetLowColor(BeListSelectGrey);
	}
	else
	{
		if ( !(Parent->IndexOf(this) % 2) )
		{
			owner->SetHighColor(White);
			owner->SetLowColor(White);
		}
		else
		{
			owner->SetHighColor((rgb_color){240, 240, 240});
			owner->SetLowColor((rgb_color){240, 240, 240});
		}
		owner->FillRect( draw_rect );
	}

	owner->SetHighColor(Black);

	BRegion Region;
	Region.Include(draw_rect);
	owner->ConstrainClippingRegion(&Region);
	if ( column_index == 0 )
	{
		if ( Tumbnail )
		{
			draw_rect.right -= 4.0;
			draw_rect.left = draw_rect.right - 15.0;
			draw_rect.top += 2.0;
			if ( Height() > 20.0 )
				draw_rect.top += ceil((Height()-20.0)/2.0);
			draw_rect.bottom = draw_rect.top+15.0;
			if (!IsSelected())
				owner->SetDrawingMode(B_OP_OVER);
			else
				owner->SetDrawingMode(B_OP_BLEND);
			owner->DrawBitmap(Tumbnail, BRect(0,0,15,15), draw_rect);
			owner->SetDrawingMode(B_OP_COPY);
		}
	}
	else if ( column_index >= 1 )
	{
		switch ( column_index )
		{
		case 1:
			owner->DrawString(Name, BPoint(draw_rect.left+8.0, draw_rect.top+fTextOffset));
			break;
		case 2:
			owner->DrawString(sSize, BPoint( draw_rect.right - 8.0 - owner->StringWidth(sSize), draw_rect.top+fTextOffset));
			break;
		case 3:
			{
				char *ct = ctime( &modifiedTime );
				ct[strlen(ct)-1] = 0;
				owner->DrawString(ct, BPoint(draw_rect.left+8.0, draw_rect.top+fTextOffset));
			}
			break;
		case 4:
			{
				char *ct = ctime( &createdTime );
				ct[strlen(ct)-1] = 0;
				owner->DrawString(ct, BPoint(draw_rect.left+8.0, draw_rect.top+fTextOffset));
			}
			break;
		}
	}

	owner->ConstrainClippingRegion(NULL);
}

void basFileListItem::Update(BView *owner, const BFont *font)
{
	CLVListItem::Update(owner, font);
	font_height FontAttributes;
	be_plain_font->GetHeight(&FontAttributes);
	FontHeight = ceil(FontAttributes.ascent) + ceil(FontAttributes.descent);
	fTextOffset = ceil(FontAttributes.ascent) + (Height()-FontHeight)/2.0;
}

void basFileListItem::ColumnWidthChanged(int32 column_index, float column_width, ColumnListView* the_view)
{
	the_view->Invalidate(ItemColumnFrame(column_index, the_view));
}

basFileListView::basFileListView()
	: ColumnListView( BRect(0, 0, 200, 200), &ContainerView, NULL, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE ),
	openThread(-1), fColumnIndex(1)
{
	((basApplication*)be_app)->GetSettingBool( "basWindow::ShowImagesOnly", &fShowImagesOnly );
	((basApplication*)be_app)->GetSettingBool( "basWindow::ShowFoldersFirst", &fShowFoldersFirst );
	((basApplication*)be_app)->GetSettingBool( "basWindow::ShowNameColumn", &fShowColumn[0] );
	((basApplication*)be_app)->GetSettingBool( "basWindow::ShowSizeColumn", &fShowColumn[1] );
	((basApplication*)be_app)->GetSettingBool( "basWindow::ShowModifiedTimeColumn", &fShowColumn[2] );
	((basApplication*)be_app)->GetSettingBool( "basWindow::ShowCreatedTimeColumn", &fShowColumn[3] );

	get_ref_for_path( "/boot/home", &OpenFolder);
	BitmapColumn = new CLVColumn(NULL, 30, CLV_NOT_MOVABLE | CLV_NOT_RESIZABLE | CLV_LOCK_AT_BEGINNING );
	NameColumn = new CLVColumn( Locale->Text(TEXT_NAME, "Name"), 200.0, CLV_SORT_KEYABLE|CLV_HEADER_TRUNCATE|CLV_TELL_ITEMS_WIDTH, 100.0 );
	SizeColumn = new CLVColumn( Locale->Text(TEXT_SIZE, "Size"), 70.0, CLV_SORT_KEYABLE|CLV_HEADER_TRUNCATE|CLV_TELL_ITEMS_WIDTH, 50.0 );
	ModifiedTimeColumn = new CLVColumn( Locale->Text(TEXT_MODIFIED, "Modified"), 200.0, CLV_SORT_KEYABLE|CLV_HEADER_TRUNCATE|CLV_TELL_ITEMS_WIDTH, 50.0 );
	CreatedTimeColumn = new CLVColumn( Locale->Text(TEXT_CREATED, "Created"), 200.0, CLV_SORT_KEYABLE|CLV_HEADER_TRUNCATE|CLV_TELL_ITEMS_WIDTH, 50.0 );
	AddColumn( BitmapColumn);
	AddColumn( NameColumn );
	AddColumn( SizeColumn );
	AddColumn( ModifiedTimeColumn );
	AddColumn( CreatedTimeColumn );
	NameColumn->SetShown(fShowColumn[0]);
	SizeColumn->SetShown(fShowColumn[1]);
	ModifiedTimeColumn->SetShown(fShowColumn[2]);
	CreatedTimeColumn->SetShown(fShowColumn[3]);
	SetSortFunction(basFileListView::TypeCompare);
	SetInvocationMessage(new BMessage(B_MESSAGE_INVOKE));
	SetSelectionMessage(new BMessage(B_MESSAGE_SELECT));
	SetSortKey(1);
}

basFileListView::~basFileListView()
{
	if ( openThread >= 0 )
		kill_thread(openThread);
	CleanListContent();
}

void basFileListView::ColumnWidthChanged(int32 ColumnIndex, float NewWidth)
{
	ColumnListView::ColumnWidthChanged( ColumnIndex, NewWidth);
}

void basFileListView::ShowFolderEaner( BEntry *ent, char *name, node_ref &nref )
{
	if ( ent->IsDirectory() )
	{
		if ( !fShowImagesOnly )
		{
			AddItem( new basFileListItem(this, ent, nref, name));
		}
	}
	else if ( ent->IsSymLink() )
	{
		BSymLink link( ent );
		char _path[B_PATH_NAME_LENGTH];
		if ( link.ReadLink( _path, B_PATH_NAME_LENGTH) > 0 )
		{
			if ( !link.IsAbsolute() )
			{
				BEntry pent;
				ent->GetParent( &pent );
				BPath pent_path( &pent );
				char tmp[B_PATH_NAME_LENGTH];
				sprintf(tmp, "%s/%s", pent_path.Path(), _path );
				strcpy(_path, tmp );
			}
			BEntry lent(_path);
			ShowFolderEaner(&lent, name, nref);
		}
	}
	else
	{
		BNode node(ent);
		BNodeInfo info(&node);
		char type[B_FILE_NAME_LENGTH];

		if ( info.GetType(type) == B_OK )
		{
			if ( strncasecmp(type, "image", 5) == 0 )
			{
				AddItem( new basFileListItem(this, ent, nref, name) );
			}
		}
	}
}

void basFileListView::CleanListContent()
{
	basFileListItem *item;
	do
	{
		item = (basFileListItem*)RemoveItem((int32)0);
		if ( item )
			delete item;
	} while (item);
}

void basFileListView::MessageReceived(BMessage *msg)
{
	switch (msg->what)
	{
		case B_SIMPLE_DATA:
			Window()->PostMessage(msg);
			break;
		case B_KEY_DOWN:
			Window()->PostMessage(msg);
			break;
		default:
			ColumnListView::MessageReceived(msg);
			break;
	}
}

int32 basFileListView::NumberOfVisibleRows()
{
	int32 hei = (int32)(Bounds().bottom - Bounds().top);
	return hei/ITEM_HEIGHT;
}

int basFileListView::TypeCompare(const CLVListItem* Item1, const CLVListItem* Item2, int32 KeyColumn)
{
	switch ( KeyColumn )
	{
		case 1:
			{
				BEntry ent1( &((basFileListItem*)Item1)->Ref );
				BEntry ent2( &((basFileListItem*)Item2)->Ref );
				if ( !((basFileListItem*)Item1)->Parent->fShowFoldersFirst )
					return strcasecmp( ((basFileListItem*)Item1)->Name, ((basFileListItem*)Item2)->Name);
				else
				{
					if ( ent1.IsDirectory() )
					{
						if ( ent2.IsDirectory() )
							return strcasecmp( ((basFileListItem*)Item1)->Name, ((basFileListItem*)Item2)->Name);
						else
							return -1;
					}
					else
					{
						if ( ent2.IsDirectory() )
							return 1;
						else
							return strcasecmp( ((basFileListItem*)Item1)->Name, ((basFileListItem*)Item2)->Name);
					}
				}
			}
			break;
		case 2:
			return (((basFileListItem*)Item1)->Size > ((basFileListItem*)Item2)->Size) ? -1 : (((basFileListItem*)Item1)->Size == ((basFileListItem*)Item2)->Size) ? 0 : 1;
		case 3:
			return (((basFileListItem*)Item1)->modifiedTime > ((basFileListItem*)Item2)->modifiedTime) ? -1 : (((basFileListItem*)Item1)->modifiedTime == ((basFileListItem*)Item2)->modifiedTime) ? 0 : 1;
		case 4:
			return (((basFileListItem*)Item1)->createdTime > ((basFileListItem*)Item2)->createdTime) ? -1 : (((basFileListItem*)Item1)->createdTime == ((basFileListItem*)Item2)->createdTime) ? 0 : 1;
	}
	return 0;
}

int basFileListView::FindItemByNode( node_ref &ref )
{
	int z = CountItems();
	for ( int a = 0; a < z; a++ )
	{
		if ( ((basFileListItem*)ItemAt(a))->Node == ref )
			return a;
	}
	return -1;
}

void basFileListView::RemoveItemInside( int num )
{
	basFileListItem *item = (basFileListItem*)RemoveItem((int32)num);
	if ( item )
		delete item;
}

void basFileListView::UpdateItem( int index, entry_ref *ref, char *name )
{
	basFileListItem *item = (basFileListItem*)ItemAt(index);
	BEntry entry;
	if ( ref )
	{
		entry.SetTo(ref);
		entry.GetRef(&item->Ref);
	}
	else
		entry.SetTo(&item->Ref);
	entry.GetModificationTime(&item->modifiedTime);
	if ( name )
	{
		if ( item->Name )
			delete item->Name;
		item->Name = new char[strlen(name)+1];
		strcpy(item->Name, name);
	}
}

void basFileListView::AddItem( basFileListItem *item )
{
	int _i = CountItems();
	for ( int i = 0; i<_i; i++ )
	{
		if ( !SortingReversed )
		{
			if ( TypeCompare( (CLVListItem*)ItemAt(i), item, fColumnIndex ) > 0 )
			{
				ColumnListView::AddItem(item, i );
				return;
			}
		}
		else
		{
			if ( TypeCompare( (CLVListItem*)ItemAt(i), item, fColumnIndex ) < 0 )
			{
				ColumnListView::AddItem(item, i );
				return;
			}
		}
	}
	ColumnListView::AddItem(item);
}

void basFileListView::SetSortKey(int32 ColumnIndex)
{
	fColumnIndex = ColumnIndex;
	ColumnListView::SetSortKey(ColumnIndex);
}

void basFileListView::SetSortMode(int32 ColumnIndex, CLVSortMode Mode)
{
	if ( Mode == Descending )
		SortingReversed = true;
	else
		SortingReversed = false;
	ColumnListView::SetSortMode(ColumnIndex, Mode);
}