#ifndef __basFileListView_h__
#define __basFileListView_h__

#include <Entry.h>
#include <Bitmap.h>
#include "ColumnListView.h"
#include "CLVColumn.h"
#include "CLVListItem.h"

#include <time.h>

class basFileListView;
class basWindow;
class basMainView;
class basTumbnailView;
class basApplication;
class basIFD;

class basFileListItem : public CLVListItem
{
friend class basFileListView;
friend class basMainView;
friend class basWindow;
friend class basTumbnailView;
public:
	basFileListItem( basFileListView *parent, BEntry *, node_ref &, const char *name = NULL );
	virtual ~basFileListItem();

	void DrawItemColumn( BView *owner, BRect draw_rect, int32 column_index, bool complete );
	virtual void ColumnWidthChanged(int32 column_index, float column_width, ColumnListView* the_view);
	void Update( BView *owner, const BFont *font );
private:
	BBitmap *Tumbnail;
	char *Name;
	float FontHeight, fTextOffset;
	off_t Size;
	char sSize[128];
	entry_ref Ref;
	node_ref Node;
	basFileListView *Parent;
	time_t modifiedTime, createdTime;
};

class basFileListView : public ColumnListView
{
friend class basWindow;
friend class basApplication;
public:
	basFileListView();
	virtual ~basFileListView();

	CLVContainerView *Container() { return ContainerView; };
	virtual void MessageReceived( BMessage *);
	virtual void ColumnWidthChanged(int32 ColumnIndex, float NewWidth);

	int32 NumberOfVisibleRows();
	void RemoveItemInside(int);

	void AddItem( basFileListItem * );
	void SetSortKey(int32 ColumnIndex);
	void SetSortMode(int32 ColumnIndex, CLVSortMode Mode);

private:
	void CleanListContent();
	void ShowFolderEaner( BEntry *, char *name, node_ref & );
	static int TypeCompare(const CLVListItem* Item1, const CLVListItem* Item2, int32 KeyColumn);
	int FindItemByNode( node_ref & );
	void UpdateItem( int, entry_ref *, char * );

	CLVContainerView *ContainerView;

	CLVColumn *BitmapColumn, *NameColumn, *SizeColumn, *ModifiedTimeColumn, *CreatedTimeColumn;
	bool fShowImagesOnly;
	bool fShowFoldersFirst;
	bool fShowColumn[4];
	entry_ref OpenFolder;
	thread_id openThread;
	int32 fColumnIndex;
	bool SortingReversed;
};

#endif