#include <Path.h>
#include <Alert.h>
#include <unistd.h>
#include <Directory.h>
#include <string.h>
#include <malloc.h>
#include <MenuItem.h>
#include <FilePanel.h>
#include <File.h>
#include "basImageList.h"
#include "Colors.h"
#include "basPluginInterface.h"
#include "basWindow.h"
#include "basApplication.h"
#include "basLocale.h"
#include "TextDefs.h"

basImageListList::basImageListList( BRect r, const char *n, list_view_type lvt, uint32 a)
	: BListView( r, n, lvt, a )
{
}

void basImageListList::MessageReceived(BMessage *msg )
{
	switch ( msg->what )
	{
		case B_SIMPLE_DATA:
			Window()->PostMessage(msg);
			break;
		default:
			BListView::MessageReceived(msg);
	}
}

basImageListWindow::basImageListWindow(basWindow *W)
	: BWindow( BRect(-400, -400, -100, -100), Locale->Text(TEXT_BEANDSEE_IMAGELIST, "BeAndSee ImageList"), B_DOCUMENT_WINDOW, 0 ),
	MainWindow(W), Filters(0), numFilters(0)
{
	imageList = new BList();

	MenuBar = new BMenuBar( Bounds(), NULL);

	BMenu *menu = new BMenu(Locale->Text(TEXT_IMAGELIST, "ImageList"));
	menu->AddItem( new BMenuItem(Locale->Text(TEXT_OPEN, "Open"), new BMessage(B_MENU_OPEN), 'O') );
	menu->AddItem( new BMenuItem(Locale->Text(TEXT_SAVE, "Save"), new BMessage(B_MENU_SAVE), 'S') );
	menu->AddSeparatorItem();
	menu->AddItem( new BMenuItem(Locale->Text(TEXT_REMOVE_SELECTED_ITEM, "Remove Selected Item"), new BMessage(B_REMOVE_SELECTED), 'R' ) );
	menu->AddSeparatorItem();
	menu->AddItem( new BMenuItem(Locale->Text(TEXT_CLEAR, "Clear"), new BMessage(B_MENU_CLEAR)) );
	menu->AddSeparatorItem();
	menu->AddItem( new BMenuItem(Locale->Text(TEXT_CLOSE, "Close"), new BMessage(B_MENU_CLOSE), 'W') );

	MenuBar->AddItem( menu );

	menu = new BMenu(Locale->Text(TEXT_PLUGINS, "Plugins"));

	vector<basPluginInterface*> &PluginList = MainWindow->PluginInterfaceManager()->PluginList();

	for ( unsigned int j=0; j < PluginList.size(); j++ )
	{
		basPluginInterface *Pi = PluginList[j];

		if ( Pi->Plug->PluginType() & PLUGIN_TYPE_PROCESS_REFS )
		{
			if (!Filters)
				Filters = (int*)malloc(sizeof(int));
			else
				Filters = (int*)realloc(Filters, sizeof(int)*(numFilters+1));
			Filters[numFilters] = j;
			menu->AddItem( new BMenuItem( Pi->Plug->PluginName(), new BMessage(B_MENU_PICTURE_DO_PLUGIN)));
			numFilters++;
		}
	}

	MenuBar->AddItem(menu);

	AddChild( MenuBar);

	imageListView = new basImageListList( Bounds(), NULL, B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL_SIDES );
	imageListScrollView = new BetterScrollView( NULL, imageListView, B_FOLLOW_ALL_SIDES, B_FRAME_EVENTS|B_WILL_DRAW, true, true, true, B_NO_BORDER );
	AddChild( imageListScrollView );

	imageListView->SetInvocationMessage(new BMessage(B_MESSAGE_INVOKE));

	FrameResized( 300, 300 );

	Show();
	Hide();
}

basImageListWindow::~basImageListWindow()
{
	ClearImageList();
	delete imageList;
	if ( Filters )
	{
		free(Filters);
	}
}

void basImageListWindow::MessageReceived( BMessage *msg )
{
	switch ( msg->what )
	{
	case B_SIMPLE_DATA:
		{
			entry_ref Ref;
			if ( msg->FindRef( "refs", &Ref) == B_OK )
			{
				BEntry entry(&Ref);
				BPath path(&entry);
				imageListView->AddItem( new BStringItem( path.Path() ) );
			}
		}
		break;
	case B_MENU_PICTURE_DO_PLUGIN:
		{
			int32 index = msg->FindInt32("index");
			BMessage *newmsg = new BMessage;
			for ( int i = 0; i< imageList->CountItems(); i ++ )
			{
				entry_ref *tmpref = (entry_ref*)imageList->ItemAt(i);
				newmsg->AddRef( "refs", tmpref );
			}
			MainWindow->PluginInterfaceManager()->PluginList()[Filters[index]]->Plug->ProcessRefs(newmsg);
		}
		break;
	case B_MENU_CLOSE:
		Hide();
		break;
	case B_MENU_OPEN:
		(new BFilePanel( B_OPEN_PANEL, new BMessenger(this), NULL, B_FILE_NODE, false, NULL, NULL, true, true ))->Show();
		break;
	case B_MENU_SAVE:
		(new BFilePanel( B_SAVE_PANEL, new BMessenger(this), NULL, B_FILE_NODE, false, NULL, NULL, true, true ))->Show();
		break;
	case B_MENU_CLEAR:
		ClearImageList();
		break;
	case B_SAVE_REQUESTED:
		{
			entry_ref dirref;
			const char *name;
			char path[B_FILE_NAME_LENGTH];
			msg->FindRef( "directory", &dirref);
			BEntry ent(&dirref);
			BPath pth(&ent);
			strcpy( path, pth.Path() );
			msg->FindString( "name", &name );
			strcat(path, "/");
			strcat(path, name);
			BEntry entry( path );
			SaveImageList(&entry);
		}
		break;
	case B_REFS_RECEIVED:
		{
			RefsReceived(msg);
		}
		break;
	case B_MESSAGE_INVOKE:
	{
		BListView *view;
		msg->FindPointer("source", (void**)&view);
		if (view)
		{
			BStringItem *item = (BStringItem*)view->ItemAt(view->CurrentSelection());
			if ( item )
			{
				BEntry entry( item->Text() );
				MainWindow->DoSelectFunction( &entry );
			}
		}
		break;
	}
	case B_REMOVE_SELECTED:
		{
			BStringItem *item; 
			int32 selected;
			int i = 0;
			while ( (selected = imageListView->CurrentSelection(i)) >= 0 )
			{
				item = (BStringItem*)imageListView->ItemAt(selected);
				entry_ref refz;
				get_ref_for_path( item->Text(), &refz );
				for ( int j = 0; j< imageList->CountItems(); j ++ )
				{
					entry_ref *tmpref = (entry_ref*)imageList->ItemAt(j);
					if ( *tmpref == refz )
					{
						imageList->RemoveItem(tmpref);
						delete tmpref;
						break;
					}
				}
				imageListView->RemoveItem(item);
				i++;
			}
		}
		break;
	case B_CANCEL:
		{
			BFilePanel *panel;
			msg->FindPointer("source", (void**)&panel);
			panel->Window()->Quit();
		}
		break;
	default:
		BWindow::MessageReceived(msg);
	}
}

void basImageListWindow::RefsReceived( BMessage *msg )
{
	entry_ref ref;
	msg->FindRef("refs", &ref);
	BEntry entry( &ref );
	OpenImageList(&entry);
}

bool basImageListWindow::QuitRequested()
{
	if ( ((basApplication*)be_app)->fQuiting )
		return true;
	if (!IsHidden())
		Hide();
	return false;
}

void basImageListWindow::FrameResized(float x, float y )
{
	imageListScrollView->MoveTo(0, MenuBar->Bounds().Height() +1);
	imageListScrollView->ResizeTo(x, y - MenuBar->Bounds().Height() - 1);
}

void basImageListWindow::AddRef( entry_ref *ref )
{
	for ( int i = 0; i< imageList->CountItems(); i ++ )
	{
		entry_ref *tmpref = (entry_ref*)imageList->ItemAt(i);
		if ( *tmpref == *ref )
		{
			return;
		}
	}

	Lock();
	BEntry entry(ref);
	BPath path(&entry);
	imageListView->AddItem( new BStringItem( path.Path() ) );

	entry_ref *Ref = new entry_ref(*ref);
	imageList->AddItem( Ref );
	Unlock();
}

void basImageListWindow::OpenImageList( BEntry *ent )
{
	ClearImageList();
	BFile il( ent, B_READ_ONLY );
	while (1)
	{
		char line[512];
		int z = 0;
		while ( (il.Read( &line[z], 1 ) > 0) && line[z] != '\n' )
			z++;
		if ( line[z] == '\n' )
		{
			line[z] = 0;
			entry_ref ref;
			get_ref_for_path( line, &ref );
			AddRef(&ref);
		}
		else
			break;
	}
}

void basImageListWindow::SaveImageList( BEntry *ent )
{
	if ( ent->Exists() )
		ent->Remove();
	BFile il( ent, B_WRITE_ONLY|B_CREATE_FILE );
	for ( int i = 0; i< imageList->CountItems(); i ++ )
	{
		entry_ref *tmpref = (entry_ref*)imageList->ItemAt(i);
		BEntry entry(tmpref);
		BPath path(&entry);
		char Path[B_PATH_NAME_LENGTH];
		strcpy(Path, path.Path());
		strcat(Path, "\n");
		il.Write( Path, strlen(Path));
	}
}

void basImageListWindow::ClearImageList()
{
	entry_ref *ref;
	while ( (ref = (entry_ref*)imageList->FirstItem()) )
	{
		imageList->RemoveItem(ref);
		delete ref;
	}
	while ( imageListView->ItemAt(0) )
	{
		BStringItem *si = (BStringItem*)imageListView->RemoveItem((int32)0);
		delete si;
	}
}