#include <stdio.h>
#include <string.h>
#include <Path.h>
#include <Entry.h>
#include <Roster.h>
#include <Application.h>
#include <Directory.h>
#include "basLocale.h"

basLocale *Locale;

basLocale::basLocale()
{
	Locale = this;
}

basLocale::~basLocale()
{
	int _i = Languages.CountItems();
	for ( int i = 0; i < _i; i++ )
	{
		Lang *v = (Lang*)Languages.ItemAt(i);
		Languages.RemoveItem(i);
		if ( v )
		{
			if ( v->Name )
				delete v->Name;
			delete v;
		}
	}
	DeleteAllLanguageStrings();
}

void basLocale::DeleteAllLanguageStrings()
{
	int _i = Language.CountItems();
	for ( int i = 0; i < _i; i++ )
	{
		char *v = (char*)Language.ItemAt(i);
		Languages.RemoveItem(i);
		if ( v )
			delete v;
	}
}

void basLocale::LoadLanguages()
{
	char temp[B_PATH_NAME_LENGTH];
	app_info info;
	be_app->GetAppInfo(&info);
	BEntry ent(&info.ref, true);
	BEntry par;
	ent.GetParent(&par);
	BPath path(&par);
	sprintf(temp, "%s/languages", path.Path() );

	BDirectory dir( temp );
	BEntry entry;

	while ( dir.GetNextEntry( &entry, true) == B_OK )
	{
		if ( !entry.IsDirectory() )
		{
			BPath path(&entry);
			FILE *fd = fopen(path.Path(), "r");
			if (fd)
			{
				char line[256];
				fgets( line, 256, fd );
				line[strlen(line)-1]=0;
				Lang *ns = new Lang;
				entry.GetRef(&ns->Ref);
				ns->Name = new char[strlen(line)-1];
				strcpy(ns->Name, line);
				Languages.AddItem(ns);
				fclose(fd);
			}
		}
	}
}

void basLocale::SetLanguage( char *lang )
{
	if (!lang)
	{
		DeleteAllLanguageStrings();
		return;
	}
	int _i = Languages.CountItems();
	for ( int i = 0; i < _i; i++ )
	{
		Lang *v = (Lang*)Languages.ItemAt(i);
		if (!strcmp(v->Name, lang) )
		{
			FILE *fd;
			BEntry entry(&v->Ref);
			BPath path(&entry);
			fd = fopen( path.Path(), "r");
			if ( fd )
			{
				char verybig[4096];
				bool open = false;
				char temp[512];
				fgets(temp, 512, fd);
				while (1)
				{
					if ( feof(fd) || fgets( temp, 512, fd ) == NULL )
					{
						if ( open )
						{
							char *s = new char[strlen(verybig)+1];
							strcpy(s, verybig);
							Language.AddItem(s);
						}
						break;
					}
					temp[strlen(temp)-1] = 0;
					if ( !open )
					{
						if ( !strcmp(temp, "#") )
						{
							open = true;
							verybig[0] = 0;
						}
					}
					else
					{
						if ( !strcmp(temp, "#") )
						{
							if ( strlen(verybig) == 0 )
								Language.AddItem(NULL);
							else
							{
								char *s = new char[strlen(verybig)+1];
								strcpy(s, verybig);
								Language.AddItem(s);
							}
							open = false;
						}
						else
						{
							if ( verybig[0] )
								strcat(verybig, "\n");
							strcat(verybig, temp);
						}
					}
				}
				fclose(fd);
			}
			break;
		}
	}
}

char *basLocale::Text( int id, char *other )
{
	char *Text = (char*)Language.ItemAt(id);
	if (!Text)
		return other;
	return Text;
}