#include <Node.h>
#include <NodeInfo.h>
#include <Path.h>
#include <stdio.h>
#include <malloc.h>
#include "basPictureWindow.h"
#include "basWindow.h"
#include "basApplication.h"
#include "basCommon.h"
#include "basPluginInterface.h"
#include "basLocale.h"
#include "TextDefs.h"

void to_rgb( int32 color, rgb_color &co );

basPictureWindow *_MainPictureWindow = NULL;

BBitmapView::BBitmapView( BRect bounds)
	: BView( bounds, NULL, B_FOLLOW_ALL, B_WILL_DRAW), bitmap(0)
{
	SetViewColor(B_TRANSPARENT_COLOR);
}

void BBitmapView::Draw( BRect draw_rect )
{
	if ( bitmap )
		DrawBitmap(bitmap, Bounds());
}

void BBitmapView::SetBitmap(BBitmap *b)
{
	bitmap = b;
}

void BBitmapView::MouseDown(BPoint where)
{
	Parent()->MouseDown(where);
}

basBigPicture::basBigPicture( BRect rect)
	: BView( rect, NULL, B_FOLLOW_ALL_SIDES, B_WILL_DRAW|B_FRAME_EVENTS),
	Vbitmap(0), fKeepAspectRatio(false)
{
	int col;
	rgb_color color;
	((basApplication*)be_app)->GetSettingInt( "basPictureWindow::BackColor", &col );
	to_rgb(col, color);

	SetViewColor(color);

	BitmapView = new BBitmapView(Bounds());
	AddChild(BitmapView);
}

basBigPicture::~basBigPicture()
{
	if ( Vbitmap )
		delete Vbitmap;
}

bool basBigPicture::LoadFrom( BEntry *entry )
{
	if ( Vbitmap )
	{
		BitmapView->SetBitmap(NULL);
		delete Vbitmap;
		Vbitmap = NULL;
	}

	if ( !entry || entry->IsDirectory() )
	{
		if ( !Window()->IsHidden() )
			Window()->Hide();
		return false;
	}

	BNode node(entry);
	BNodeInfo info(&node);
	char type[B_FILE_NAME_LENGTH];

	info.GetType(type);

	if ( strncasecmp(type, "image", 5) == 0 )
	{
		BPath path(entry);
		Vbitmap = BTranslationUtils::GetBitmap(path.Path());
		if ( Vbitmap )
		{
			if ( ! ((basPictureWindow*)Window())->fViewFullscreen && Window()->IsHidden() )
			{
				BScreen Display(B_MAIN_SCREEN_ID);
				float _w = Vbitmap->Bounds().Width(), _h = Vbitmap->Bounds().Height();

				if (_w > (Display.Frame().Width() - B_V_SCROLL_BAR_WIDTH) || _h > (Display.Frame().Height() - (3*B_V_SCROLL_BAR_WIDTH)))
				{
					if ( _w > (Display.Frame().Width() - B_V_SCROLL_BAR_WIDTH) )
					{
						_w = Display.Frame().Width() - B_V_SCROLL_BAR_WIDTH;
						Window()->MoveTo( 0, Window()->Frame().top );
					}
					if ( _h > (Display.Frame().Height() - (3*B_V_SCROLL_BAR_WIDTH)) )
					{
						_h = Display.Frame().Height() - (3*B_V_SCROLL_BAR_WIDTH);
						Window()->MoveTo( Window()->Frame().left, B_V_SCROLL_BAR_WIDTH );
					}
				}

				Window()->ResizeTo(_w+B_V_SCROLL_BAR_WIDTH+1, _h+B_V_SCROLL_BAR_WIDTH+B_V_SCROLL_BAR_WIDTH+4);
			}

			BetterScrollView *v = (BetterScrollView *)Window()->FindView("basPictureWindow::BigPicture_ScrollView");
			if ( fKeepAspectRatio )
				v->SetDataRect( BRect(0,0,0,0), false );
			else
				v->SetDataRect( Vbitmap->Bounds(), true );
			entry->GetRef(&bitmapRef);
			if ( Window()->IsHidden() )
				Window()->Show();
			char title[20 + B_FILE_NAME_LENGTH];
			BPath path(entry);
			sprintf(title, "BeAndSee - %s - %ix%i", path.Leaf(), (int)Vbitmap->Bounds().Width()+1, (int)Vbitmap->Bounds().Height()+1);
			Window()->SetTitle(title);
			BitmapView->SetBitmap(Vbitmap);
			BitmapView->Invalidate();

			((basPictureWindow*)Window())->ClearUndos();
		}
		else
		{
			if ( !Window()->IsHidden() )
				Window()->Hide();
			return false;
		}
	}
	else
	{
		if ( !Window()->IsHidden() )
			Window()->Hide();
		return false;
	}

	return true;
}

void basBigPicture::MouseDown( BPoint point )
{
	BMessage* msg = Window()->CurrentMessage();
	int32 button = msg->FindInt32("buttons");
	if ( (button & B_SECONDARY_MOUSE_BUTTON ) == B_SECONDARY_MOUSE_BUTTON )
	{
		if ( !Window()->IsHidden() )
			Window()->Hide();
	}
	else if ( ( button & B_TERTIARY_MOUSE_BUTTON) == B_TERTIARY_MOUSE_BUTTON )
	{
		Window()->PostMessage(B_MENU_TOGGLE_FULLSCREEN);
	}
}

void basBigPicture::FrameResized( float x, float y )
{
	BRect new_r;
	if ( !fKeepAspectRatio )
	{
		float wmargin = ((Window()->Bounds().right - Window()->Bounds().left) - Vbitmap->Bounds().right - B_V_SCROLL_BAR_WIDTH)/2;
		float hmargin = ((Window()->Bounds().bottom - Window()->Bounds().top) - Vbitmap->Bounds().bottom - B_V_SCROLL_BAR_WIDTH - B_V_SCROLL_BAR_WIDTH -3)/2;
		if ( wmargin < 0 )
			wmargin = 0;
		if ( hmargin < 0 )
			hmargin = 0;
		new_r = BRect( wmargin, hmargin, Vbitmap->Bounds().right + wmargin, Vbitmap->Bounds().bottom + hmargin );
	}
	else
	{
		float aspect = Vbitmap->Bounds().right / Vbitmap->Bounds().bottom;
		float window_height = Bounds().Height();
		float window_width = Bounds().Width();
		float xsize = window_width;
		float ysize = window_width / aspect;
		float wmargin = 0, hmargin;
		if ( ysize > window_height )
		{
			hmargin = 0;
			ysize = window_height;
			xsize = window_height * aspect;
			wmargin = (window_width - xsize)/2;
		}
		else
			hmargin = (window_height-ysize)/2;
		new_r = BRect( wmargin, hmargin, xsize + wmargin, ysize + hmargin);
	}

	BitmapView->MoveTo(new_r.left, new_r.top);
	BitmapView->ResizeTo(new_r.Width(), new_r.Height());
}

basPictureWindow::basPictureWindow( basWindow *wnd, bool quit_when_done)
	: BWindow( BRect(-400, -400, -100, -100), "BeAndSee", B_DOCUMENT_WINDOW, 0 ),
	MainWindow(wnd), fViewFullscreen(false), OutTranslators(0), numOfOutTranslators(0),
	Filters(0), numFilters(0), navigationPos(0), changedContents(false), QuitWhenDone(quit_when_done)
{
	if (!_MainPictureWindow)
		_MainPictureWindow = this;

	MenuBar = new BMenuBar( Bounds(), "menu_bar" );
	BMenu *menu = new BMenu(Locale->Text(TEXT_FILE, "File"));

	BMenu *submenu = new BMenu( Locale->Text(TEXT_SAVE_AS_DOTDOTDOT, "Save As..."));

	translator_id *translators;
	int32 num_translators;
	BTranslatorRoster *roster = BTranslatorRoster::Default();
	
	roster->GetAllTranslators(&translators, &num_translators);
	for (int32 i=0; i<num_translators;i++)
	{
		const translation_format *fmts;
		int32 num_fmts;
		roster->GetOutputFormats(translators[i], &fmts, &num_fmts);
		for (int32 j=0;j<num_fmts;j++)
		{
			if ( fmts[j].group == B_TRANSLATOR_BITMAP && strcmp(fmts[j].MIME, "image/x-be-bitmap"))
			{
				if ( !OutTranslators )
					OutTranslators = (translator_s*)malloc(sizeof(translator_s));
				else
					OutTranslators = (translator_s*)realloc(OutTranslators, (numOfOutTranslators+1)*sizeof(translator_s));
				OutTranslators[numOfOutTranslators].id = translators[i];
				OutTranslators[numOfOutTranslators++].type = fmts[j].type;
				submenu->AddItem( new BMenuItem( fmts[j].name, new BMessage(B_MENU_SAVE_AS)));
			}
		}
	}

	delete translators;

	if ( numOfOutTranslators )
		menu->AddItem( submenu );
	menu->AddSeparatorItem();
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_ABOUT, "About"), new BMessage(B_ABOUT_REQUESTED), 'A'));
	menu->AddSeparatorItem();
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_CLOSE, "Close"), new BMessage(B_MENU_CLOSE), 'W'));
	MenuBar->AddItem(menu);
	menu = new BMenu(Locale->Text(TEXT_IMAGE, "Image"));
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_UNDO, "Undo"), new BMessage(B_MENU_UNDO), 'Z'));
	menu->AddSeparatorItem();
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_SET_AS_DESKTOP_BACKGROUND, "Set As Desktop Background"), new BMessage(B_MENU_SET_AS_BACKGROUND), 'D'));
	menu->AddSeparatorItem();
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_STRETCH_AND_KEEP_ASPECT_RATIO, "Stretch & Keep aspect ratio"), new BMessage(B_MENU_KEEP_ASPECT_RATIO), 'K'));
	MenuBar->AddItem(menu);
	menu = new BMenu(Locale->Text(TEXT_EFFECTS, "Effects"));

	vector<basPluginInterface*> &PluginList = MainWindow->PluginInterfaceManager()->PluginList();

	for ( unsigned int j=0; j < PluginList.size(); j++ )
	{
		basPluginInterface *Pi = PluginList[j];

		if ( Pi->Plug->PluginType() & PLUGIN_TYPE_PROCESS_IMAGE )
		{
			if (!Filters)
				Filters = (int*)malloc(sizeof(int));
			else
				Filters = (int*)realloc(Filters, sizeof(int)*(numFilters+1));
			Filters[numFilters] = j;
			menu->AddItem( new BMenuItem( Pi->Plug->PluginName(), new BMessage(B_MENU_PICTURE_DO_PLUGIN)));
			numFilters++;
		}
	}

	MenuBar->AddItem(menu);
	menu = new BMenu(Locale->Text(TEXT_VIEW, "View"));
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_FULLSCREEN, "Fullscreen"), new BMessage(B_MENU_TOGGLE_FULLSCREEN), 'F'));
	MenuBar->AddItem(menu);
	AddChild( MenuBar );

	BigPicture = new basBigPicture(Bounds());
	AddChild( new BetterScrollView( "basPictureWindow::BigPicture_ScrollView", BigPicture, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS, true, true, true, B_NO_BORDER ));

	UndoStack = new basStack<BBitmap*>(3);

	if (!QuitWhenDone)
	{
		Show();
		Hide();
		MoveTo( 100, 100 );
	}
}

basPictureWindow::~basPictureWindow()
{
	ClearUndos();

	if ( OutTranslators )
		free(OutTranslators);

	if ( Filters )
	{
		free(Filters);
	}
}

void basPictureWindow::MessageReceived( BMessage *msg )
{
	switch (msg->what)
	{
	case B_SIMPLE_DATA:
		be_app->PostMessage(msg);
		break;
	case B_MOUSE_WHEEL_CHANGED:
		{
			float mv = msg->FindFloat("be:wheel_delta_y");
			if ( mv > 0 )
			{
				if ( navigationPos < (MainWindow->imageNavigationList->CountItems()-1) )
				{
					navigationPos ++;
					entry_ref *ref = (entry_ref*)MainWindow->imageNavigationList->ItemAt(navigationPos);
					BEntry ent(ref);
					BigPicture->LoadFrom(&ent);
					BigPicture->FrameResized(BigPicture->Bounds().Width(), BigPicture->Bounds().Height());
					BigPicture->BitmapView->Invalidate();
				}
			}
			else if ( mv < 0 )
			{
				if ( navigationPos > 0 )
				{
					navigationPos --;
					entry_ref *ref = (entry_ref*)MainWindow->imageNavigationList->ItemAt(navigationPos);
					BEntry ent(ref);
					BigPicture->LoadFrom(&ent);
					BigPicture->FrameResized(BigPicture->Bounds().Width(), BigPicture->Bounds().Height());
					BigPicture->BitmapView->Invalidate();
				}
			}
		}
		break;
	case B_KEY_DOWN:
		{
			int32 byte;
			msg->FindInt32( "raw_char", &byte );
			switch ( byte )
			{
				case B_UP_ARROW:
					break;
				case B_PAGE_UP:
					if ( navigationPos > 0 )
					{
						navigationPos --;
						entry_ref *ref = (entry_ref*)MainWindow->imageNavigationList->ItemAt(navigationPos);
						BEntry ent(ref);
						BigPicture->LoadFrom(&ent);
						BigPicture->FrameResized(BigPicture->Bounds().Width(), BigPicture->Bounds().Height());
						BigPicture->BitmapView->Invalidate();
					}
					break;
				case B_PAGE_DOWN:
					if ( navigationPos < (MainWindow->imageNavigationList->CountItems()-1) )
					{
						navigationPos ++;
						entry_ref *ref = (entry_ref*)MainWindow->imageNavigationList->ItemAt(navigationPos);
						BEntry ent(ref);
						BigPicture->LoadFrom(&ent);
						BigPicture->FrameResized(BigPicture->Bounds().Width(), BigPicture->Bounds().Height());
						BigPicture->BitmapView->Invalidate();
					}
					break;
				case B_HOME:
					{
						navigationPos = 0;
						entry_ref *ref = (entry_ref*)MainWindow->imageNavigationList->ItemAt(navigationPos);
						BEntry ent(ref);
						BigPicture->LoadFrom(&ent);
						BigPicture->FrameResized(BigPicture->Bounds().Width(), BigPicture->Bounds().Height());
						BigPicture->BitmapView->Invalidate();
					}
					break;
				case B_END:
					{
						navigationPos = (MainWindow->imageNavigationList->CountItems()-1);
						entry_ref *ref = (entry_ref*)MainWindow->imageNavigationList->ItemAt(navigationPos);
						BEntry ent(ref);
						BigPicture->LoadFrom(&ent);
						BigPicture->FrameResized(BigPicture->Bounds().Width(), BigPicture->Bounds().Height());
						BigPicture->BitmapView->Invalidate();
					}
					break;
			}
		}
		break;
	case B_MENU_CLOSE:
		Hide();
		if ( QuitWhenDone )
			PostMessage(B_QUIT_REQUESTED);
		break;
	case B_ABOUT_REQUESTED:
		MainWindow->PostMessage(msg);
		break;
	case B_MENU_SAVE_AS:
		{
			int32 a;
			msg->FindInt32("index", &a);
			selectedTranslator = a;
			(new BFilePanel( B_SAVE_PANEL, new BMessenger(this), NULL, B_FILE_NODE, false, NULL, NULL, true, true ))->Show();
		}
		break;
	case B_MENU_TOGGLE_FULLSCREEN:
		{
		fViewFullscreen = !fViewFullscreen;
		MenuBar->SubmenuAt(3)->ItemAt(0)->SetMarked(fViewFullscreen);
		if ( fViewFullscreen )
		{
			fRectBeforeFullscreen = Frame();
			SetLook( B_NO_BORDER_WINDOW_LOOK );
			MoveTo( 0, 0 );
			BScreen Screen;
			ResizeTo( Screen.Frame().Width(), Screen.Frame().Height() );
		}
		else
		{
			SetLook( B_DOCUMENT_WINDOW_LOOK );
			MoveTo( fRectBeforeFullscreen.left, fRectBeforeFullscreen.top );
			ResizeTo( fRectBeforeFullscreen.Width(), fRectBeforeFullscreen.Height() );
		}
		}
		break;
	case B_MENU_SET_AS_BACKGROUND:
		{
			BEntry entry( &BigPicture->bitmapRef );
			basSetBackgroundWindow *wnd = new basSetBackgroundWindow();
			wnd->Show();
			wnd->ShowWith(&entry);
		}
		break;
	case B_MENU_KEEP_ASPECT_RATIO:
		{
			int32 index = msg->FindInt32("index");
			BigPicture->fKeepAspectRatio = !BigPicture->fKeepAspectRatio;
			MenuBar->SubmenuAt(1)->ItemAt(index)->SetMarked(BigPicture->fKeepAspectRatio);
			BetterScrollView *v = (BetterScrollView *)FindView("basPictureWindow::BigPicture_ScrollView");
			if ( BigPicture->fKeepAspectRatio )
				v->SetDataRect( BRect(0,0,0,0), false );
			else
				v->SetDataRect( BigPicture->Vbitmap->Bounds(), true );
			BigPicture->FrameResized(BigPicture->Bounds().Width(), BigPicture->Bounds().Height());
		}
		break;
	case B_SAVE_REQUESTED:
		SaveImage(msg);
		break;
	case B_CANCEL:
		{
			BFilePanel *panel;
			msg->FindPointer("source", (void**)&panel);
			panel->Window()->Quit();
		}
		break;
	case B_MENU_PICTURE_DO_PLUGIN:
		{
			int index;
			msg->FindInt32("index", (int32*)&index);
			BMessage *newmsg = new BMessage(*msg);
			newmsg->AddPointer("Window", this);
			resume_thread(spawn_thread( PluginThread, "PluginThread", B_NORMAL_PRIORITY, (void*)newmsg));
		}
		break;
	case B_MENU_UNDO:
		{
			BBitmap *old = BigPicture->Vbitmap;
			BigPicture->Vbitmap = UndoMem[UndoMem.size()-1];
			BigPicture->BitmapView->SetBitmap(BigPicture->Vbitmap);
			BigPicture->BitmapView->Invalidate();
/*			if ( UndoMem.size() <= 1 )
				UndoMem.empty();
			else
				UndoMem.erase(UndoMem.end());*/
			UndoMem.erase(UndoMem.begin() + UndoMem.size()-1 );
			delete old;
			if ( UndoMem.size() == 0 )
				SetUndoMenuEnabled(false);
		}
		break;
	default:
		BWindow::MessageReceived(msg);
	}
}

void basPictureWindow::SaveImage( BMessage *msg )
{
	entry_ref dirref;
	const char *name;
	char path[B_FILE_NAME_LENGTH];
	msg->FindRef( "directory", &dirref);
	BEntry ent(&dirref);
	BPath pth(&ent);
	strcpy( path, pth.Path() );
	msg->FindString( "name", &name );
	strcat(path, "/");
	strcat(path, name);
	BBitmapStream bstream(BigPicture->Vbitmap);
	BFile outfile( path, B_CREATE_FILE | B_WRITE_ONLY);
	BTranslatorRoster *roster = BTranslatorRoster::Default();
	if ( roster->Translate( OutTranslators[selectedTranslator].id, &bstream, NULL, &outfile, OutTranslators[selectedTranslator].type) != B_OK )
	{
		(new BAlert(Locale->Text(TEXT_WARNING, "Warning"), Locale->Text(TEXT_TEXT3, "Failed to save the image on the desired format."), Locale->Text(TEXT_DARN_DOTDOT, "Darn..")))->Go();
	}
	bstream.DetachBitmap(&BigPicture->Vbitmap);
	BEntry _newentry(path);
	_newentry.GetRef(&BigPicture->bitmapRef);
	char title[B_FILE_NAME_LENGTH];
	sprintf(title, "BeAndSee - %s - %ix%i", name, (int)BigPicture->Vbitmap->Bounds().Width()+1, (int)BigPicture->Vbitmap->Bounds().Height()+1);
	SetTitle(title);
}

bool basPictureWindow::QuitRequested()
{
	if ( ((basApplication*)be_app)->fQuiting )
		return true;
	if (!IsHidden())
		Hide();
	return QuitWhenDone;
}

void basPictureWindow::FrameResized( float x, float y)
{
	BetterScrollView *sc = (BetterScrollView*)FindView("basPictureWindow::BigPicture_ScrollView");
	sc->MoveTo(0, MenuBar->Bounds().Height()+1);
	sc->ResizeTo( x, y - MenuBar->Bounds().Height() -1 );
}

int32 basPictureWindow::PluginThread( void *arg )
{
	BMessage *msg = (BMessage*)arg;
	basPictureWindow *wnd;
	msg->FindPointer("Window", (void**)&wnd);

	BBitmap *newbitmap = NULL;
	if ( wnd->MainWindow->PluginInterfaceManager()->PluginList()[wnd->Filters[msg->FindInt32("index")]]->Plug->ProcessImage( wnd->BigPicture->Vbitmap, &newbitmap ) == 0 )
	{
		if ( newbitmap )
		{
			wnd->Lock();
			wnd->BigPicture->BitmapView->SetBitmap(newbitmap);
			if ( wnd->UndoMem.size() == 3 )
			{
				delete wnd->UndoMem[0];
				wnd->UndoMem.erase( wnd->UndoMem.begin() );
			}

			wnd->UndoMem.push_back(wnd->BigPicture->Vbitmap);
			wnd->SetUndoMenuEnabled(true);

			wnd->BigPicture->Vbitmap = newbitmap;
			BetterScrollView *v = (BetterScrollView *)wnd->FindView("basPictureWindow::BigPicture_ScrollView");
			v->SetDataRect( newbitmap->Bounds(), true );
			wnd->BigPicture->FrameResized(wnd->BigPicture->Bounds().Width(), wnd->BigPicture->Bounds().Height());
			wnd->BigPicture->BitmapView->Invalidate();
			char title[B_PATH_NAME_LENGTH];
			BEntry entry(&wnd->BigPicture->bitmapRef);
			BPath path(&entry);
			sprintf(title, "BeAndSee - %s - %ix%i", path.Leaf(), (int)newbitmap->Bounds().Width()+1, (int)newbitmap->Bounds().Height()+1);
			wnd->SetTitle(title);
			wnd->Unlock();
			wnd->changedContents = true;
		}
	}
	return 0;
}

void basPictureWindow::SetUndoMenuEnabled(bool b)
{
	MenuBar->SubmenuAt(1)->ItemAt(0)->SetEnabled(b);
}

void basPictureWindow::ClearUndos()
{
	for (unsigned int i=0; i<UndoMem.size(); i++ )
	{
		delete UndoMem[i];
	}
	UndoMem.empty();
	SetUndoMenuEnabled(false);
}

void basPictureWindow::SetPicture(BEntry *entry)
{
	BigPicture->LoadFrom(entry);
}

void PictureWindow_SetPicture( BEntry *e )
{
	_MainPictureWindow->SetPicture(e);
}