#ifndef __basPictureWindow_h__
#define __basPictureWindow_h__

#include <View.h>
#include <MenuField.h>
#include <Entry.h>
#include <Window.h>
#include <TranslationKit.h>
#include <image.h>
#include <vector>
#include "basSetBackgroundWindow.h"
#include "basPlugin.h"
#include "basCommon.h"
#include "basSupport.h"

class basBigPicture;
class basPictureWindow;
class basWindow;

class BBitmapView : public BView
{
public:
	BBitmapView(BRect);
	virtual void Draw( BRect );
	void SetBitmap( BBitmap * );
	virtual void MouseDown( BPoint);

protected:
	BBitmap *bitmap;
};

class basBigPicture : public BView
{
friend class basPictureWindow;
public:
	basBigPicture( BRect );
	~basBigPicture();
	bool LoadFrom( BEntry * );
	virtual void MouseDown( BPoint);
	virtual void FrameResized( float x, float y );

	void SetKeepAspectRatio(bool b) { fKeepAspectRatio = b; }
	bool GetKeepAspectRatio() { return fKeepAspectRatio; }

private:
	BBitmap *Vbitmap;
	bool fKeepAspectRatio;
	entry_ref bitmapRef;
	BBitmapView *BitmapView;
};

typedef struct
{
	translator_id id;
	uint32 type;
} translator_s;

class basPictureWindow : public BWindow
{
friend class basBigPicture;
friend class basWindow;
public:
	basPictureWindow( basWindow *, bool quit_when_done = false);
	~basPictureWindow();

	virtual void MessageReceived( BMessage *);
	virtual bool QuitRequested();
	virtual void FrameResized( float, float );

	void SetUndoMenuEnabled( bool );
	void ClearUndos();

	void SetPicture( BEntry * );

private:
	void SaveImage( BMessage * );
	static int32 PluginThread( void * );
	int selectedTranslator;
	basBigPicture *BigPicture;
	BMenuField *ZoomMenu;
	basWindow *MainWindow;
	BMenuBar *MenuBar;
	bool fViewFullscreen;
	BRect fRectBeforeFullscreen;
	translator_s *OutTranslators;
	int numOfOutTranslators;
	int *Filters;
	int numFilters;
	int navigationPos;
	bool changedContents;
	bool QuitWhenDone;
	vector<BBitmap*> UndoMem;
	basStack<BBitmap*> *UndoStack;
};

#endif