#include <PopUpMenu.h>
#include <Message.h>
#include <MenuItem.h>
#include <Alert.h>
#include "basPreferencesWindow.h"
#include "basApplication.h"
#include "Colors.h"
#include "basCommon.h"
#include "basLocale.h"
#include "TextDefs.h"
#include "Colors.h"
#include "ColorPickerWindow.h"

void to_rgb( int32 color, rgb_color &co );
int32 rgb_to( rgb_color &co );

BColorShowControl::BColorShowControl( BRect rect, const char *name, const char *label, BMessage *msg, uint32 resizing_mode, uint32 flags)
	: BView(rect, name, resizing_mode, B_WILL_DRAW|flags ),
	Label(label), Message(msg), Value((rgb_color){0, 0, 0, 255})
{
	SetViewColor(BeBackgroundGrey);
	SetLowColor(BeBackgroundGrey);
}

void BColorShowControl::Draw(BRect draw_rect)
{
	SetHighColor(Black);
	DrawString( Label.String(), BPoint(5, Bounds().bottom -5 ) );
	float _x = Bounds().Width() / 2;
	SetHighColor(Value);
	FillRect( BRect( _x, Bounds().bottom -21, _x+16, Bounds().bottom -5) );
	SetHighColor(Black);
	SetDrawingMode(B_OP_INVERT);
	StrokeRect( BRect( _x, Bounds().bottom -21, _x+16, Bounds().bottom -5) );
	SetDrawingMode(B_OP_OVER);
}

void BColorShowControl::MouseDown(BPoint where)
{
	float _x = Bounds().Width() / 2;
	if ( BRect( _x, Bounds().bottom -21, _x+16, Bounds().bottom -5).Contains(where) )
	{
		BColorPickerWindow *wnd = new BColorPickerWindow();
		wnd->SetMessage(new BMessage(B_SET_COLOR));
		wnd->SetTarget(Window());
		wnd->SetColor(Value);
		wnd->MoveTo(200, 200);
		wnd->Show();
	}
}

basPreferencesWindow::basPreferencesWindow(BWindow *parent)
	: BWindow( BRect(100, 100, 420, 565), Locale->Text(TEXT_PREFERENCES, "Preferences"), B_TITLED_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL, B_NOT_RESIZABLE|B_NOT_ZOOMABLE ),
	ParentWindow(parent)
{
	backView = new BView( Bounds(), NULL, B_FOLLOW_ALL_SIDES, 0 );
	backView->SetViewColor( BeBackgroundGrey );

	BMenu *menu;

	char *setLang;

	((basApplication*)be_app)->GetSettingString( "basApplication::Language", &setLang);

	int fl1, fl2, fl3;

	((basApplication*)be_app)->GetSettingInt( "basFileListView::selectBehaviour", &fl1);
	((basApplication*)be_app)->GetSettingInt( "basFileListView::invokeBehaviour", &fl2 );
	((basApplication*)be_app)->GetSettingInt( "basFileListView::FileListViewWheelMouseBehaviour", &fl3 );

	int tv1, tv2, tv3, tv4, tv5;

	((basApplication*)be_app)->GetSettingInt( "basTumbnailView::leftButtonBehaviour", &tv1);
	((basApplication*)be_app)->GetSettingInt( "basTumbnailView::middleButtonBehaviour", &tv2 );
	((basApplication*)be_app)->GetSettingInt( "basTumbnailView::rightButtonBehaviour", &tv3 );
	((basApplication*)be_app)->GetSettingInt( "basTumbnailView::TumbnailViewWheelMouseBehaviour", &tv4 );
	((basApplication*)be_app)->GetSettingInt( "basTumbnailView::doubleButtonBehaviour", &tv5 );

	boxFL = new BBox( BRect(5, 5, 315, 115 ), NULL );
	boxFL->SetLabel( Locale->Text(TEXT_FOLDERVIEWS_MOUSE_BEHAVIOUR, "FolderView's Mouse behaviour"));

	menu = new BPopUpMenu( "");
	menu->SetRadioMode(true);
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DO_NOTHING, "Do Nothing"), new BMessage(B_MENU_PREFS_SET_MOUSE_F_L) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_IMAGE_AND_DONT_OPEN_FOLDER, "Open Image and don\'t Open Folder"), new BMessage(B_MENU_PREFS_SET_MOUSE_F_L) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_IMAGE_OR_FOLDER, "Open Image or Folder"), new BMessage(B_MENU_PREFS_SET_MOUSE_F_L) ) );
	menu->ItemAt(fl1)->SetMarked(true);

	menuFLL = new BMenuField( BRect( 5, 20, 305, 45 ), NULL, Locale->Text(TEXT_SELECT_SINGLE_CLICK, "Select (Single Click)"), menu );
	boxFL->AddChild(menuFLL);

	menu = new BPopUpMenu( "");
	menu->SetRadioMode(true);
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DO_NOTHING, "Do Nothing"), new BMessage(B_MENU_PREFS_SET_MOUSE_F_M) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_IMAGE_AND_DONT_OPEN_FOLDER, "Open Image and don\'t Open Folder"), new BMessage(B_MENU_PREFS_SET_MOUSE_F_M) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_IMAGE_OR_FOLDER, "Open Image or Folder"), new BMessage(B_MENU_PREFS_SET_MOUSE_F_M) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_FOLDER_AND_DONT_OPEN_IMAGE, "Open Folder and don\'t Open Image"), new BMessage(B_MENU_PREFS_SET_MOUSE_F_M) ) );
	menu->ItemAt(fl2)->SetMarked(true);

	menuFLM = new BMenuField( BRect( 5, 50, 305, 75 ), NULL, Locale->Text(TEXT_INVOKE_DOUBLE_CLICK, "Invoke (Double Click)"), menu );
	boxFL->AddChild(menuFLM);

	menu = new BPopUpMenu( "");
	menu->SetRadioMode(true);
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DO_NOTHING, "Do Nothing"), new BMessage(B_MENU_PREFS_SET_MOUSE_F_W) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_SCROLL_VIEW_UP_DOWN_1_ITEM, "Scroll View Up/Down 1 item"), new BMessage(B_MENU_PREFS_SET_MOUSE_F_W) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_SCROLL_VIEW_UP_DOWN_1_PAGE, "Scroll View Up/Down 1 page"), new BMessage(B_MENU_PREFS_SET_MOUSE_F_W) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_SCROLL_VIEW_UP_DOWN_1_ITEM_AND_SELECT_IT, "Scroll View Up/Down 1 item and select it"), new BMessage(B_MENU_PREFS_SET_MOUSE_F_W) ) );
	menu->ItemAt(fl3)->SetMarked(true);

	menuFLW = new BMenuField( BRect( 5, 80, 305, 105 ), NULL, Locale->Text(TEXT_MOUSE_WHEEL, "MouseWheel"), menu );
	boxFL->AddChild(menuFLW);

	backView->AddChild(boxFL);

	boxTV = new BBox( BRect(5, 125, 315, 295 ), NULL );
	boxTV->SetLabel( Locale->Text(TEXT_TUMBNAILS_MOUSE_BEHAVIOUR, "TumbnailView's Mouse behaviour"));

	menu = new BPopUpMenu( "");
	menu->SetRadioMode(true);
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DO_NOTHING, "Do Nothing"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_L) ));
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_IMAGE, "Open Image"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_L) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_MENU, "Open Menu"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_L) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DRAG_AND_DROP, "Drag&Drop"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_L) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DO_SELECTION, "Do Selection"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_L) ) );
	menu->ItemAt(tv1)->SetMarked(true);

	menuTVL = new BMenuField( BRect( 5, 20, 305, 45 ), NULL, Locale->Text(TEXT_LEFT_BUTTON, "Left Button"), menu );
	boxTV->AddChild(menuTVL);

	menu = new BPopUpMenu( "");
	menu->SetRadioMode(true);
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DO_NOTHING, "Do Nothing"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_M) ));
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_IMAGE, "Open Image"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_M) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_MENU, "Open Menu"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_M) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DRAG_AND_DROP, "Drag&Drop"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_M) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DO_SELECTION, "Do Selection"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_M) ) );
	menu->ItemAt(tv2)->SetMarked(true);

	menuTVM = new BMenuField( BRect( 5, 50, 305, 75 ), NULL, Locale->Text(TEXT_MIDDLE_BUTTON, "Middle Button"), menu );
	boxTV->AddChild(menuTVM);

	menu = new BPopUpMenu( "");
	menu->SetRadioMode(true);
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DO_NOTHING, "Do Nothing"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_R) ));
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_IMAGE, "Open Image"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_R) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_MENU, "Open Menu"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_R) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DRAG_AND_DROP, "Drag&Drop"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_R) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DO_SELECTION, "Do Selection"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_R) ) );
	menu->ItemAt(tv3)->SetMarked(true);

	menuTVR = new BMenuField( BRect( 5, 80, 305, 105 ), NULL, Locale->Text(TEXT_RIGHT_BUTTON, "Right Button"), menu );
	boxTV->AddChild(menuTVR);

	menu = new BPopUpMenu( "");
	menu->SetRadioMode(true);
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DO_NOTHING, "Do Nothing"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_W) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_SCROLL_VIEW_UP_DOWN_1_ITEM, "Scroll View Up/Down 1 item"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_W) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_SCROLL_VIEW_UP_DOWN_1_PAGE, "Scroll View Up/Down 1 page"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_W) ) );
	menu->ItemAt(tv4)->SetMarked(true);

	menuTVW = new BMenuField( BRect( 5, 110, 305, 135 ), NULL, Locale->Text(TEXT_MOUSE_WHEEL, "Mouse Wheel"), menu );
	boxTV->AddChild(menuTVW);

	menu = new BPopUpMenu( "");
	menu->SetRadioMode(true);
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DO_NOTHING, "Do Nothing"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_D) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_OPEN_IMAGE, "Open Image"), new BMessage(B_MENU_PREFS_SET_MOUSE_T_D) ) );
	menu->ItemAt(tv5)->SetMarked(true);

	menuTVD = new BMenuField( BRect( 5, 140, 305, 165 ), NULL, Locale->Text(TEXT_DOUBLE_CLICK, "Double Click"), menu );
	boxTV->AddChild(menuTVD);

	backView->AddChild(boxTV);

	boxGeneralOptions = new BBox( BRect(5, 305, 315, 410 ), NULL );
	boxGeneralOptions->SetLabel( Locale->Text(TEXT_GENERAL_OPTIONS, "General Options") );

	menu = new BPopUpMenu( "");
	menu->SetRadioMode(true);
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_DEFAULT, "Default"), new BMessage(B_MENU_SET_LANGUAGE)));
	menu->AddSeparatorItem();
	BList *list = Locale->LanguageList();
	int _i = list->CountItems();
	for (int i=0; i<_i; i++)
	{
		menu->AddItem( new BMenuItem( ((Lang*)list->ItemAt(i))->Name, new BMessage(B_MENU_SET_LANGUAGE)) );
		if ( setLang && !strcmp( ((Lang*)list->ItemAt(i))->Name, setLang) )
			menu->ItemAt(i+2)->SetMarked(true);
	}
	if (!menu->FindMarked())
		menu->ItemAt(0)->SetMarked(true);
	Languages = new BMenuField( BRect( 5, 15, 305, 40 ), NULL, Locale->Text(TEXT_LANGUAGES, "Languages"), menu );
	boxGeneralOptions->AddChild(Languages);

	bool temp_bool;
	((basApplication*)be_app)->GetSettingBool( "basApplication::TumbnailCachingEnabled", &temp_bool );

	menu = new BPopUpMenu( "");
	menu->SetRadioMode(true);
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_NO, "No"), new BMessage(B_MENU_PREFS_SET_TUMBNAIL_CACHING) ) );
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_YES, "Yes"), new BMessage(B_MENU_PREFS_SET_TUMBNAIL_CACHING) ) );
	menu->ItemAt(temp_bool ? 1: 0)->SetMarked( true );
	TumbnailCachingEnabled = new BMenuField( BRect( 5, 45, 305, 70 ), NULL, Locale->Text(TEXT_ENABLE_TUMBNAIL_CACHING, "Enable Thumbnail Caching"), menu );

	boxGeneralOptions->AddChild(TumbnailCachingEnabled);

	char *_app;
	((basApplication*)be_app)->GetSettingString( "basApplication::prefered_app", &_app );

	PreferedAppTC = new BTextControl( BRect(5, 75, 305, 100), NULL, Locale->Text(TEXT_PREFERED_APPLICATION, "Prefered application"), _app, NULL );
	boxGeneralOptions->AddChild(PreferedAppTC);

	backView->AddChild(boxGeneralOptions);

	int col;
	rgb_color color;
	((basApplication*)be_app)->GetSettingInt( "basPictureWindow::BackColor", &col );
	to_rgb(col, color);

	boxColors = new BBox(BRect(5, 415, 315, 460), NULL );
	boxColors->SetLabel( Locale->Text(TEXT_COLORS, "Colors") );

	controlColor1 = new BColorShowControl( BRect(5, 15, 305, 40), NULL, Locale->Text(TEXT_VIEWERS_BACKCOLOR, "Viewer's backColor"), NULL, 0, 0 );
	controlColor1->SetValue(color);
	boxColors->AddChild(controlColor1);

	backView->AddChild(boxColors );

	AddChild( backView );

	Show();
}

basPreferencesWindow::~basPreferencesWindow()
{
}

void basPreferencesWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what)
	{
	case B_MENU_PREFS_SET_MOUSE_T_L:
		{
			int32 index;
			msg->FindInt32("index", &index);
			((basApplication*)be_app)->SetSettingInt( "basTumbnailView::leftButtonBehaviour", index);
		}
		break;
	case B_MENU_PREFS_SET_MOUSE_T_M:
		{
			int32 index;
			msg->FindInt32("index", &index);
			((basApplication*)be_app)->SetSettingInt( "basTumbnailView::middleButtonBehaviour", index);
		}
		break;
	case B_MENU_PREFS_SET_MOUSE_T_R:
		{
			int32 index;
			msg->FindInt32("index", &index);
			((basApplication*)be_app)->SetSettingInt( "basTumbnailView::rightButtonBehaviour", index);
		}
		break;
	case B_MENU_PREFS_SET_MOUSE_T_D:
		{
			int32 index;
			msg->FindInt32("index", &index);
			((basApplication*)be_app)->SetSettingInt( "basTumbnailView::doubleButtonBehaviour", index);
		}
		break;
	case B_MENU_PREFS_SET_MOUSE_T_W:
		{
			int32 index;
			msg->FindInt32("index", &index);
			((basApplication*)be_app)->SetSettingInt( "basTumbnailView::TumbnailViewWheelMouseBehaviour", index );
		}
		break;
	case B_MENU_PREFS_SET_MOUSE_F_L:
		{
			int32 index;
			msg->FindInt32("index", &index);
			((basApplication*)be_app)->SetSettingInt( "basFileListView::selectBehaviour", index );
		}
		break;
	case B_MENU_PREFS_SET_MOUSE_F_M:
		{
			int32 index;
			msg->FindInt32("index", &index);
			((basApplication*)be_app)->SetSettingInt( "basFileListView::invokeBehaviour", index );
		}
		break;
	case B_MENU_PREFS_SET_MOUSE_F_W:
		{
			int32 index;
			msg->FindInt32("index", &index);
			((basApplication*)be_app)->SetSettingInt( "basFileListView::FileListViewWheelMouseBehaviour", index );
		}
		break;
	case B_MENU_SET_LANGUAGE:
		{
			int32 index;
			msg->FindInt32("index", &index);
			(new BAlert(Locale->Text(TEXT_WARNING, "Warning"), Locale->Text(TEXT_YOU_MUST_RESTART_BEANDSEE_FOR_MODIFICATIONS_TO_TAKE_EFFECT, "You must restart BeAndSee for modifications to take effect."), Locale->Text(TEXT_OK, "Ok")))->Go();
			if ( index == 0 )
				((basApplication*)be_app)->SetSettingString( "basApplication::Language", "");
			else
				((basApplication*)be_app)->SetSettingString( "basApplication::Language", (char*)Languages->Menu()->ItemAt(index)->Label());
		}
		break;
	case B_MENU_PREFS_SET_TUMBNAIL_CACHING:
		{
			int32 index;
			msg->FindInt32("index", &index);
			((basApplication*)be_app)->SetSettingBool( "basApplication::TumbnailCachingEnabled", index == 1 );
		}
		break;
	case B_SET_COLOR:
		{
			int32 color;
			msg->FindInt32("Color", &color);
			rgb_color col;
			to_rgb( color, col );
			controlColor1->SetValue(col);
			((basApplication*)be_app)->SetSettingInt( "basPictureWindow::BackColor", color );
		}
		break;
	default:
		BWindow::MessageReceived(msg);
	}
}

bool basPreferencesWindow::QuitRequested()
{
	((basApplication*)be_app)->SetSettingString( "basApplication::prefered_app", (char*)PreferedAppTC->Text() );

	ParentWindow->PostMessage(B_RELOAD_SETTINGS);
	PostMessage(B_QUIT_REQUESTED);
	return true;
}