#include "basSetBackgroundWindow.h"
#include "basApplication.h"
#include "Colors.h"
#include "BitmapUtil.h"
#include <Path.h>
#include <Directory.h>
#include <FindDirectory.h>
#include <stdio.h>
#include <Messenger.h>
#include <Background.h>
#include <Node.h>
#include <fs_attr.h>
#include <PopUpMenu.h>
#include <Menu.h>
#include <MenuItem.h>
#include <Alert.h>
#include <File.h>
#include <Screen.h>
#include <TranslationKit.h>
#include "basCommon.h"
#include "basLocale.h"
#include "TextDefs.h"

#define B_FIELD_BACKTYPE		0x70002
#define B_CHECKBOX_COPY_TO_HOME	0x70003

basSetBackgroundWindow_View::basSetBackgroundWindow_View( BRect rect )
	: BView( rect, "basSetBackgroundWindow::View", B_FOLLOW_ALL_SIDES, B_WILL_DRAW|B_FRAME_EVENTS )
{
	SetHighColor(Black);
}

basSetBackgroundWindow_View::~basSetBackgroundWindow_View()
{
}

void basSetBackgroundWindow_View::Draw( BRect )
{
	if (((basSetBackgroundWindow*)Window())->ScreenDump)
	{
		DrawBitmap( ((basSetBackgroundWindow*)Window())->ScreenDump, ((basSetBackgroundWindow*)Window())->ScreenDumpRect );
		BRect newr( ((basSetBackgroundWindow*)Window())->ScreenDumpRect);
		newr.left -= 1;
		newr.top -= 1;
		newr.bottom += 1;
		newr.right += 1;
		StrokeRect( newr );
	}
}

basSetBackgroundWindow::basSetBackgroundWindow()
	: BWindow( BRect(-415, -400, -100, -100), "BeAndSee SetBackground", B_TITLED_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL, B_NOT_RESIZABLE|B_NOT_ZOOMABLE|B_NOT_CLOSABLE ),
		ScreenDump(0)
{
	basSetBackgroundWindow_View *view = new basSetBackgroundWindow_View(Bounds());
	AddChild(view);
	view->SetViewColor (BeBackgroundGrey);
	OkButton = new BButton( BRect( 265, 270, 305, 290), NULL, Locale->Text(TEXT_OK, "Ok"), new BMessage(B_BUTTON_OK));
	view->AddChild(OkButton);
	CancelButton = new BButton( BRect( 205, 270, 255, 290), NULL, Locale->Text(TEXT_CANCEL, "Cancel"), new BMessage(B_BUTTON_CANCEL));
	view->AddChild(CancelButton);
	BPopUpMenu *menu = new BPopUpMenu( "" );
//	menu->AddItem( new BMenuItem( "Manual", new BMessage(B_FIELD_BACKTYPE)));
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_CENTERED, "Centered"), new BMessage(B_FIELD_BACKTYPE)));
	BMenuItem *menuitem = new BMenuItem( Locale->Text(TEXT_SCALED_TO_FIT, "Scaled to Fit"), new BMessage(B_FIELD_BACKTYPE));
	menuitem->SetMarked(true);
	menu->AddItem( menuitem);
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_TILED, "Tiled"), new BMessage(B_FIELD_BACKTYPE)));
	BackTypeField = new BMenuField( BRect( 10, 240, 305, 260 ), NULL, Locale->Text(TEXT_BACKGROUND_PLACEMENT, "Background Placement"), menu );
	BackTypeField->SetDivider( 210 );
	view->AddChild(BackTypeField);
	CopyToHomeCheckBox = new BCheckBox( BRect( 10, 215, 305, 230 ), NULL, Locale->Text(TEXT_COPY_IMAGE_TO_HOME_BACKGROUNDS_AND_SET_FROM_THERE, "Copy image to \"(Home)/Backgrounds\" and set from there?"), new BMessage(B_CHECKBOX_COPY_TO_HOME));
	view->AddChild(CopyToHomeCheckBox);
	MoveTo(100, 100);
}

basSetBackgroundWindow::~basSetBackgroundWindow()
{
}

void basSetBackgroundWindow::MessageReceived( BMessage *msg)
{
	switch (msg->what)
	{
		case B_BUTTON_OK:
			{
				char background_path[512];
				if ( CopyToHomeCheckBox->Value() )
				{
					BPath path;
					if (find_directory( B_USER_DIRECTORY, &path ) != B_OK )
					{
						(new BAlert(Locale->Text(TEXT_ERROR, "Error!"), Locale->Text(TEXT_TEXT4, "Failed to locate your home directory!\nBeAndSee failed to set this image as the background."), Locale->Text(TEXT_DARN, "Darn")))->Go();
					}
					else
					{
						BDirectory home( path.Path() );
						if ( home.Contains( "Backgrounds" ) == false )
						{
							BDirectory backgrounds_dir;
							home.CreateDirectory( "Backgrounds", &backgrounds_dir );
						}
						BFile in_file( &ScreenFile, B_READ_ONLY );
						BEntry entry(&ScreenFile);
						BPath ScreenPath(&entry);
						sprintf(background_path, "%s/Backgrounds/%s", path.Path(), ScreenPath.Leaf() );
						BFile out_file( background_path, B_WRITE_ONLY | B_CREATE_FILE );
						char dumb;
						while ( in_file.Read((void*)&dumb, 1) > 0 )
						{
							out_file.Write((const void*)&dumb, 1);
						}
					}
				}
				else
				{
					BEntry entry(&ScreenFile);
					BPath path(&entry);
					strcpy( background_path, path.Path());
				}
				BMessage backgrounds; 

				backgrounds.AddString( B_BACKGROUND_IMAGE, background_path ); 
				backgrounds.AddInt32( B_BACKGROUND_WORKSPACES, 0xffffffff ); 
				if ( BackTypeField->Menu()->FindMarked() )
				{
					int index = BackTypeField->Menu()->IndexOf(BackTypeField->Menu()->FindMarked());
					switch (index)
					{
						case 0:
							backgrounds.AddInt32( B_BACKGROUND_MODE, B_BACKGROUND_MODE_CENTERED );
							break;
						case 2:
							backgrounds.AddInt32( B_BACKGROUND_MODE, B_BACKGROUND_MODE_TILED );
							break;
						default:
							backgrounds.AddInt32( B_BACKGROUND_MODE, B_BACKGROUND_MODE_SCALED );
							break;
					}
				}
				else
					backgrounds.AddInt32( B_BACKGROUND_MODE, B_BACKGROUND_MODE_SCALED ); 
				backgrounds.AddPoint( B_BACKGROUND_ORIGIN, BPoint( 0.0, 0.0 ) ); 
				backgrounds.AddBool( B_BACKGROUND_ERASE_TEXT, true );

				BPath desktopPath;
				find_directory(B_DESKTOP_DIRECTORY, &desktopPath);
				BDirectory desktopDirectory(desktopPath.Path());

				char *buffer = new char[backgrounds.FlattenedSize()];
				backgrounds.Flatten(buffer, backgrounds.FlattenedSize());
				attr_info d_info;
				if (desktopDirectory.GetAttrInfo(B_BACKGROUND_INFO, &d_info) == B_OK)
				{
					desktopDirectory.WriteAttr(B_BACKGROUND_INFO, d_info.type, 0, buffer, backgrounds.FlattenedSize());
				}
				delete buffer;

				BMessenger tracker( "application/x-vnd.Be-TRAK" ); 

				int32 i = -1; 
				BMessage reply; 
				int32 err; 

				do {
					i++; 

					BMessage msg( B_GET_PROPERTY ); 

					msg.AddSpecifier( "Poses" ); 
					msg.AddSpecifier( "Window", i );

					reply.MakeEmpty(); 

					tracker.SendMessage( &msg, &reply ); 

					if( reply.what == B_MESSAGE_NOT_UNDERSTOOD &&
						reply.FindInt32( "error", &err ) == B_OK &&
						err == B_BAD_INDEX ) 
						break;

					if( reply.what == B_MESSAGE_NOT_UNDERSTOOD &&
						reply.FindInt32( "error", &err ) == B_OK &&
						err != B_BAD_SCRIPT_SYNTAX ) 
						continue;

					BMessenger m; 
					if ( reply.FindMessenger( "result", &m ) != B_OK) continue; 

					msg.MakeEmpty(); 
					msg.what = B_GET_PROPERTY; 
					msg.AddSpecifier( "Path" ); 
					msg.AddSpecifier( "Poses" ); 
					msg.AddSpecifier( "Window", i ); 

					reply.MakeEmpty(); 

					tracker.SendMessage( &msg, &reply ); 

					if( reply.what == B_MESSAGE_NOT_UNDERSTOOD ) continue; 

					entry_ref ref; 
					if( reply.FindRef( "result", &ref ) == B_OK ) { 
						BEntry ent( &ref ); 
						BPath path( &ent ); 

						if( strcmp( desktopPath.Path(), path.Path() ) ) continue; 
						else
						{
							m.SendMessage( B_RESTORE_BACKGROUND_IMAGE ); 
							break;
						}
					}

				} while( true );

 				if (!IsHidden())
					Hide();
			}
			break;
		case B_BUTTON_CANCEL:
			if (!IsHidden())
				Hide();
			break;
		default:
			BWindow::MessageReceived(msg);
	}
}

bool basSetBackgroundWindow::QuitRequested()
{
	return true;
}

void basSetBackgroundWindow::ShowWith( BEntry *entry )
{
	if ( ScreenDump )
	{
		delete ScreenDump;
		ScreenDump = NULL;
	}
	float wmargin, wwidth;
	BScreen Screen;
	wwidth = 200 * (Screen.Frame().Width() / Screen.Frame().Height());
	wmargin = (Bounds().Width() - wwidth) / 2;
	ScreenDumpRect = BRect( wmargin, 5, wwidth + wmargin, 205 );
	BPath path(entry);
	BBitmap *bb = BTranslationUtils::GetBitmap(path.Path());
	if ( bb )
	{
		ScreenDump = ConvertBitmap( bb, BRect( 0, 0, wwidth, 200), B_RGBA32, false);
	}
	if ( IsHidden() )
		Show();
	Lock();
	FindView("basSetBackgroundWindow::View")->Invalidate();
	Unlock();
	entry->GetRef(&ScreenFile);
}
