#include <Region.h>
#include <Directory.h>
#include <VolumeRoster.h>
#include <Volume.h>
#include <NodeInfo.h>
#include <malloc.h>
#include <stdio.h>
#include <Path.h>
#include <Application.h>
#include <Roster.h>
#include "basTreeView.h"
#include "Colors.h"
#include "basCommon.h"
#include "basBDCPInterface.h"
#include "basLocale.h"
#include "TextDefs.h"

bool Entry_IsImageList( BEntry *entry );

unsigned char camera_icon[] = {
255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 
255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 
255, 255, 255, 174, 64, 97, 0, 181, 181, 181, 181, 215, 0, 255, 255, 255, 
255, 255, 28, 174, 215, 215, 215, 215, 215, 215, 215, 215, 215, 0, 255, 255, 
255, 174, 213, 135, 135, 135, 135, 135, 135, 135, 135, 135, 135, 16, 215, 255, 
174, 134, 134, 0, 174, 174, 135, 135, 135, 135, 213, 0, 0, 0, 0, 255, 
174, 134, 174, 95, 0, 174, 0, 64, 102, 174, 0, 16, 237, 173, 174, 0, 
174, 134, 174, 174, 0, 174, 0, 0, 0, 0, 137, 215, 221, 94, 213, 174, 
174, 134, 174, 174, 0, 174, 174, 174, 174, 0, 102, 237, 31, 108, 0, 135, 
174, 134, 174, 174, 0, 174, 174, 174, 174, 0, 176, 237, 76, 76, 0, 64, 
174, 134, 174, 174, 0, 135, 0, 0, 135, 174, 174, 237, 108, 173, 213, 174, 
174, 134, 174, 174, 0, 135, 0, 203, 135, 135, 174, 0, 8, 135, 174, 255, 
174, 134, 174, 174, 0, 135, 135, 135, 174, 174, 174, 174, 174, 174, 0, 255, 
255, 213, 213, 213, 0, 0, 0, 0, 0, 141, 64, 109, 141, 174, 0, 255, 
255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 
255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  };

basTreeItem::basTreeItem( BBitmap *b, const char *n, node_ref &nref, entry_ref &eref, bool super, int level, int type )
	: CLVListItem( level, super, false, 20 ), Bitmap(b), Text(n),
	ExpandedOnce(false), NodeRef(nref), EntryRef(eref), Type(type)
{
}

basTreeItem::basTreeItem( BBitmap *b, const char *n, bool super, int level, int type, int arg )
	: CLVListItem( level, super, false, 20 ), Bitmap(b), Text(n),
	ExpandedOnce(false), Type(type), Arg(arg)
{
}

void basTreeItem::DrawItemColumn(BView *owner, BRect draw_rect, int32 column_index, bool complete)
{
	BRegion Region;
	Region.Include(draw_rect);
	owner->ConstrainClippingRegion(&Region);

	if ( column_index == 2 && IsSelected() )
	{
		owner->SetLowColor( Black );
		owner->SetHighColor( Black );
		owner->FillRect( BRect( draw_rect.left + 5, draw_rect.top + fTextOffset - FontHeight + 2, draw_rect.left + owner->StringWidth(Text.String()) + 7, draw_rect.top + fTextOffset + 2));
		owner->SetHighColor( White );
	}
	else
	{
		owner->SetLowColor(White);
		owner->SetHighColor( White );
		owner->FillRect( draw_rect );
		owner->SetHighColor( Black );
	}

	if ( column_index == 1 )
	{
		if ( Bitmap )
		{
			owner->SetDrawingMode(B_OP_OVER);
			owner->DrawBitmap( Bitmap, BRect( draw_rect.left+2, draw_rect.top+2, draw_rect.left+18, draw_rect.top+18 ) );
		}
	}
	else if ( column_index == 2 )
	{
		owner->DrawString( Text.String(), BPoint( draw_rect.left+7, draw_rect.top+fTextOffset ) );
	}

	owner->ConstrainClippingRegion(NULL);
}

void basTreeItem::Update(BView *owner, const BFont *font)
{
	CLVListItem::Update(owner, font);
	font_height FontAttributes;
	be_plain_font->GetHeight(&FontAttributes);
	FontHeight = ceil(FontAttributes.ascent) + ceil(FontAttributes.descent);
	fTextOffset = ceil(FontAttributes.ascent) + (Height()-FontHeight)/2.0;
}

basTreeView::basTreeView()
	: ColumnListView( BRect(0, 0, 300, 200), &ContainerView, NULL, B_FOLLOW_ALL_SIDES, B_WILL_DRAW|B_FRAME_EVENTS|B_NAVIGABLE, B_SINGLE_SELECTION_LIST, true, false )
{
	AddColumn(new CLVColumn( NULL, 20, CLV_EXPANDER|CLV_NOT_MOVABLE | CLV_LOCK_AT_BEGINNING ));
	AddColumn(new CLVColumn( NULL, 20, CLV_LOCK_AT_BEGINNING|CLV_NOT_MOVABLE|CLV_NOT_RESIZABLE|CLV_PUSH_PASS|CLV_MERGE_WITH_RIGHT ));
	AddColumn(new CLVColumn( NULL, 150, CLV_LOCK_AT_END|CLV_LOCK_WITH_RIGHT|CLV_NOT_RESIZABLE ) );

	BVolumeRoster vRoster;
	vRoster.Rewind();
	BVolume volume;

	basTreeItem *item;

	while (vRoster.GetNextVolume(&volume) == B_OK )
	{
		char name[B_FILE_NAME_LENGTH];
		volume.GetName(name);

		if ( strlen(name) > 0 )
		{
			BDirectory root;
			volume.GetRootDirectory(&root);
			BEntry ent;
			root.GetEntry(&ent);
			node_ref nref;
			root.GetNodeRef(&nref);
			entry_ref eref;
			ent.GetRef(&eref);

			BNode node(&ent);
			BNodeInfo nodeinfo(&node);
			BBitmap *b = new BBitmap(BRect( 0, 0, B_MINI_ICON-1, B_MINI_ICON-1), B_CMAP8);
			nodeinfo.GetTrackerIcon(b, B_MINI_ICON);

			AddItem( item = new basTreeItem(b, name, nref, eref, true));

			DevItem *dev_i = new DevItem;
			dev_i->Item = item;
			dev_i->Dev = volume.Device();
			MountList.push_back( dev_i );
		}
	}

#ifdef CAMERA_SUPPORT
	BBitmap *cambm = new BBitmap( BRect( 0, 0, B_MINI_ICON-1, B_MINI_ICON-1), B_CMAP8);
	memcpy( cambm->Bits(), camera_icon, cambm->BitsLength() );

	AddItem( item = new basTreeItem( cambm, Locale->Text(TEXT_CAMERAS, "Cameras"), true, 0, 1) );

	vector<camDef*> &camDefs = BDCPInterface->GetDefCams();
	for ( unsigned int i=0; i<camDefs.size(); i++ )
	{
		AddUnder( new basTreeItem( cambm, camDefs[i]->DefName.c_str(), false, 0, 2, i), item );
	}
#endif

	SetSelectionMessage(new BMessage(B_MESSAGE_INVOKE2));
}

basTreeView::~basTreeView()
{
	thread_id *id;
	while ( (id = (thread_id*)openThreadList.FirstItem() ) )
	{
		openThreadList.RemoveItem(id);
		kill_thread(*id);
		delete id;
	}
}

void basTreeView::Expand( CLVListItem *item )
{
	ColumnListView::Expand(item);
	if ( static_cast<basTreeItem*>(item)->Type == 0 )
	{
		BMessage *newmsg = new BMessage;
		newmsg->AddPointer("View", this);
		newmsg->AddPointer("Item", item);
		thread_id *id = new thread_id;
		*id = spawn_thread( ExpandThread, "ExpandThread", B_NORMAL_PRIORITY, (void*)newmsg);
		if ( *id >= 0 )
		{
			openThreadList.AddItem(id);
			resume_thread(*id);
		}
		else
			delete id;
	}
}

int32 basTreeView::ExpandThread(void *arg)
{
	BMessage *msg = (BMessage*)arg;
	basTreeView *View;
	basTreeItem *Item;
	msg->FindPointer("View", (void**)&View);
	msg->FindPointer("Item", (void**)&Item);

	int a = 0;
	if ( !Item->ExpandedOnce )
	{
		BDirectory dir(&Item->EntryRef);
		Item->ExpandedOnce = true;

		char buf[4096];
		dirent *dent;
		int count;
		while( ( count = dir.GetNextDirents( (dirent*)buf, 4096)) > 0 )
		{
			dent = (dirent*)buf;
			while ( count -- )
			{
				if ( !strcmp( dent->d_name, "." ) || !strcmp(dent->d_name, ".."))
					continue;
				entry_ref eref;
				eref.device = dent->d_pdev;
				eref.directory = dent->d_pino;
				eref.set_name( dent->d_name );
				BEntry ent(&eref);
				node_ref nref;
				nref.device = dent->d_dev;
				nref.node = dent->d_ino;

				if (ent.IsDirectory() || Entry_IsImageList( &ent ))
				{
					BNode node(&ent);
					BNodeInfo nodeinfo(&node);
					BBitmap *b = new BBitmap(BRect( 0, 0, B_MINI_ICON-1, B_MINI_ICON-1), B_CMAP8);
					nodeinfo.GetTrackerIcon(b, B_MINI_ICON);
					View->LockLooper();
					View->AddUnder( new basTreeItem( b, dent->d_name, nref, eref, ent.IsDirectory() ), Item );
					View->UnlockLooper();
					a++;
				}

				dent = (dirent *)((char *)dent + dent->d_reclen);
			}
		}
//		REMOVED:
/*		if ( !a )
		{
			Item->SetSuperItem(false);
			View->LockLooper();
			View->InvalidateItem( View->IndexOf(Item) );
			View->UnlockLooper();
		}*/
	}
	delete msg;
	for ( int i = 0; i<View->openThreadList.CountItems(); i++)
	{
		thread_id *id = (thread_id*)View->openThreadList.ItemAt(i);
		if ( *id == find_thread(NULL) )
		{
			View->openThreadList.RemoveItem(id);
			delete id;
		}
	}

	return 0;
}

void *_realloc( void *arg, int size )
{
	if (!arg)
		return malloc(size);
	else
		return realloc(arg, size);
}

void basTreeView::GoTo( entry_ref &Ref )
{
	BEntry Ent(&Ref);
	BList list;
	int _i = 0;
	do
	{
		BEntry ent;
		if ( _i == 0 )
		{
			ent = Ent;
		}
		else
		{
			BEntry entr((entry_ref*)list.ItemAt(_i-1));
			status_t t = entr.GetParent(&ent);
			if ( t != B_OK && t != B_ENTRY_NOT_FOUND )
				return;
			if ( t == B_ENTRY_NOT_FOUND )
				break;
		}
		entry_ref *RefZ = new entry_ref;
		ent.GetRef(RefZ);
		list.AddItem(RefZ);
		_i ++;
	} while(1);
	if ( _i == 0 )
		return;
	_i-=2;
	basTreeItem *Item = NULL;
	for ( int b=_i; b>=0; b-- )
	{
		int _k = CountItems();
		for ( int k=0;k<_k;k++)
		{
			basTreeItem *item = (basTreeItem*)ItemAt(k);
			if ( item->EntryRef == *((entry_ref*)list.ItemAt(b)) )
			{
				Item = item;
				break;
			}
		}
		if ( !Item)
			return;
		if ( !Item->ExpandedOnce )
		{
			BDirectory dir(&Item->EntryRef);
			Item->ExpandedOnce = true;
			int a = 0;
			char buf[4096];
			dirent *dent;
			int count;
			while( ( count = dir.GetNextDirents( (dirent*)buf, 4096)) > 0 )
			{
				dent = (dirent*)buf;
				while ( count -- )
				{
					if ( !strcmp( dent->d_name, "." ) || !strcmp(dent->d_name, ".."))
						continue;
					entry_ref eref;
					eref.device = dent->d_pdev;
					eref.directory = dent->d_pino;
					eref.set_name( dent->d_name );
					BEntry ent(&eref);
					node_ref nref;
					nref.device = dent->d_dev;
					nref.node = dent->d_ino;

					if (ent.IsDirectory() || Entry_IsImageList( &ent ))
					{
						BNode node(&ent);
						BNodeInfo nodeinfo(&node);
						BBitmap *b = new BBitmap(BRect( 0, 0, B_MINI_ICON-1, B_MINI_ICON-1), B_CMAP8);
						nodeinfo.GetTrackerIcon(b, B_MINI_ICON);
						AddUnder( new basTreeItem( b, dent->d_name, nref, eref, ent.IsDirectory() ), Item );
						a++;
					}

					dent = (dirent *)((char *)dent + dent->d_reclen);
				}
			}
//			REMOVED:
/*			if ( !a )
			{
				Item->SetSuperItem(false);
				InvalidateItem( IndexOf(Item) );
			}*/
		}
		ColumnListView::Expand(Item);
	}
	Select(IndexOf(Item));
	ScrollToSelection();
}

void basTreeView::FrameResized(float x, float y)
{
	ColumnAt(2)->SetWidth(x);
}

void basTreeView::RemoveVolume( dev_t dev )
{
	for ( unsigned int i=0; i<MountList.size(); i++ )
	{
		if ( MountList[i]->Dev == dev )
		{
			LockLooper();
			RemoveItem( MountList[i]->Item );
			delete MountList[i];
			MountList.erase( MountList.begin() + i );
			UnlockLooper();
			break;
		}
	}
}

void basTreeView::AddVolume( dev_t dev )
{
	char name[B_FILE_NAME_LENGTH];
	BVolume volume(dev);

	volume.GetName(name);

	if ( strlen(name) > 0 )
	{
		BDirectory root;
		volume.GetRootDirectory(&root);
		BEntry ent;
		root.GetEntry(&ent);
		node_ref nref;
		root.GetNodeRef(&nref);
		entry_ref eref;
		ent.GetRef(&eref);

		BNode node(&ent);
		BNodeInfo nodeinfo(&node);
		BBitmap *b = new BBitmap(BRect( 0, 0, B_MINI_ICON-1, B_MINI_ICON-1), B_CMAP8);
		nodeinfo.GetTrackerIcon(b, B_MINI_ICON);

		LockLooper();
		basTreeItem *item;
		AddItem( item = new basTreeItem(b, name, nref, eref, true));

		DevItem *dev_i = new DevItem;
		dev_i->Item = item;
		dev_i->Dev = volume.Device();
		MountList.push_back( dev_i );

		UnlockLooper();
	}
}