#ifndef __basTumbnailView_h__
#define __basTumbnailView_h__

#include <View.h>
#include <Entry.h>
#include <PopUpMenu.h>
#include <Bitmap.h>
#include <List.h>
#include "BetterScrollView.h"

#define TBV_DO_NOTHING		0
#define TBV_OPEN_IMAGE		1
#define TBV_OPEN_MENU		2
#define TBV_DRAG_DROP		3
#define TBV_DO_SELECTION	4

class basTumbnailView;

class basTumbnailWorker
{
public:
	basTumbnailWorker( basTumbnailView *);
	~basTumbnailWorker();

	void PostData( BMessage *);
	void Stop();
private:
	static int32 doWork( void * );
	basTumbnailView *TV;
	thread_id tID;
	BList *workList;
	sem_id Semaphore;
};

class basStatusBarView : public BView
{
public:
	basStatusBarView( BView *view, BView *statusbar );

	virtual void FrameResized( float, float );
	virtual void MessageReceived( BMessage *);

private:
	BView *View, *StatusBar;
};

class basStatusBar : public BView
{
public:
	basStatusBar();

	void SetLabel( char * );
	void SetNextLabel( char * );
	virtual void Draw(BRect);
private:
	char *Text;
	char *NextText;
};

class Tumbnail
{
public:
	Tumbnail()
		: realName(0), Name(0), Image(0)
		{}
	~Tumbnail()
	{
		if ( realName )
			delete realName;
		if ( Name )
			delete Name;
		if ( Image )
			delete Image;
	}
	char *realName;
	char *Name;
	BBitmap *Image;
	entry_ref Entry;
	node_ref Node;
	float _fNameWidth;
	char sizeString[128];
	char bitmapSizeString[128];
	float _fsizeStringWidth;
	float _fbitmapSizeStringWidth;
};

class basTumbnailView;

typedef struct
{
	BView *View;
	BEntry *Entry;
} _VEThreadArg;

class basTumbnailView : public BView
{
friend class basWindow;
public:
	basTumbnailView(BRect);
	~basTumbnailView();

	virtual void MouseDown( BPoint );
	virtual void MouseUp( BPoint );
	virtual void MouseMoved( BPoint, uint32, const BMessage* );
	virtual void Draw( BRect );
	void UpdateScrollBars();
	virtual void FrameResized( float, float );
	virtual void MessageReceived( BMessage *);
	BView *Container();
	void AddedItem(int);
	void SetTumbnailSize( int );
	void UpdateScrollbars();
	void SetLeftMouseButtonBehaviour(int);
	void SetMiddleMouseButtonBehaviour(int);
	void SetRightMouseButtonBehaviour(int);
	void SetDoubleMouseButtonBehaviour(int);
	void SetTumbnailBackgroundColor(rgb_color);
	void SetTumbnailHoverColor(rgb_color);
	void ShowFolderEaner( BEntry *, char *, entry_ref &, node_ref & );
	void ShowFolderEaner( char *, int, BBitmap* );
	void SetStatusLabel( char * );
private:
	void CleanContents();
	void RemovedTumbnail(int);
	int FindTumbnailByNode( node_ref & );
	void UpdateTumbnail( int, entry_ref *, char * );
	BList TumbnailList;
	BetterScrollView *myContainer;
	basStatusBarView *allContainer;
	basStatusBar *StatusBar;
	int lastTumbnailsPerRow, tumbnailsPerRow;
	int tumbnailSize;
	BPopUpMenu *PopUp, *nosPopUp, *msPopUp;
	BMenu *TumbnailSizePopUp;
	Tumbnail *PopUpedTumbnail;
	BList *tumbnailsSelection;
	int PopUpedTumbnailIndex;
	bool useSmoothScaling;
	int leftButtonBehaviour, middleButtonBehaviour, rightButtonBehaviour, doubleButtonBehaviour;
	Tumbnail *movedTumbnail;
	int movedTumbnailIndex;
	rgb_color TumbnailBackgroundColor;
	rgb_color TumbnailHoverColor;
	Tumbnail *FirstClickTumbnail;
	bool doingDraggingSelection;
	bool startedDoingDraggingSelection;
	BPoint doingDraggingSelectionPoint;
	BRect dragRectArea;
	bool RemoveInstedOfAdd;
};

#define TUMBNAILLIST(x)		((Tumbnail*)TumbnailList.ItemAt(x))
#define numberOfTumbnails	TumbnailList.CountItems()

#endif