#include <stdio.h>
#include <string.h>
#include <Application.h>
#include <Path.h>
#include <Directory.h>
#include <Region.h>
#include <math.h>
#include <NodeInfo.h>
#include <TranslationKit.h>
#include <Alert.h>
#include <MenuItem.h>
#include <File.h>
#include <Font.h>
#include <PopUpMenu.h>
#include <MenuField.h>
#include <Roster.h>
#include <malloc.h>
#include <Screen.h>
#include <BitmapStream.h>
#include <FindDirectory.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <VolumeRoster.h>
#include <Volume.h>
#include <fs_attr.h>
#include <SymLink.h>
#include <NodeMonitor.h>
#include <Roster.h>
#include "basWindow.h"
#include "basApplication.h"
#include "ColorPickerWindow.h"
#include "BitmapUtil.h"
#include "basCommon.h"
#include "basImageList.h"
#include "basPluginInterface.h"
#include "basBitmapWorker.h"
#include "basPreferencesWindow.h"
#include "basLocale.h"
#include "TextDefs.h"
#include "basSendEmailWindow.h"
#include "TumbnailCache.h"
#include "basNagScreen.h"
#include "basTreeView.h"
#include "basBDCPInterface.h"
#include "basCameraConfigWindow.h"

basWindow *_MainWindow;

void Entry_StringSize( BEntry *entry, off_t *size, char *out );
void MoveToTrash( BEntry * );

void Entry_StringSize( BEntry *entry, off_t *size, char *out )
{
	off_t tmp;
	if (!size)
		size = &tmp;
	entry->GetSize(size);
	if ( *size < 1024 )
	{
		sprintf(out, "%u B", (unsigned int)*size);
	}
	else if ( *size < 1048576 )
	{
		sprintf(out, "%4.2f KB", (float) ((float)*size)/ 1024.0f );
	}
	else if ( *size < 1073741824)
	{
		sprintf(out, "%4.2f MB", (float) ((float)*size)/ 1048576.0);
	}
	else
	{
		sprintf(out, "%.2f GB", (float) ((float)*size)/ 1048576.0 / 1024.0);
	}
}

void Entry_StringSize( int size, char *out )
{
	if ( size == -1 )
	{
		strcpy(out, "-");
	}
	else if ( size < 1024 )
	{
		sprintf(out, "%u B", (unsigned int)size);
	}
	else if ( size < 1048576 )
	{
		sprintf(out, "%4.2f KB", (float) ((float)size)/ 1024.0f );
	}
	else if ( size < 1073741824)
	{
		sprintf(out, "%4.2f MB", (float) ((float)size)/ 1048576.0);
	}
	else
	{
		sprintf(out, "%.2f GB", (float) ((float)size)/ 1048576.0 / 1024.0);
	}
}

void MoveToTrash( BEntry *ent )
{
	BPath trashPath;
	find_directory(B_TRASH_DIRECTORY, &trashPath);
	BDirectory trashDirectory(trashPath.Path());
	ent->MoveTo( &trashDirectory, NULL, true );
}

void to_rgb( int32 color, rgb_color &co )
{
	co.red = (color >> 24) & 0xff;
	co.green = (color >> 16) & 0xff;
	co.blue = (color >> 8) & 0xff;
	co.alpha = color & 0xff;
}

int32 rgb_to( rgb_color &co )
{
	return (co.red << 24) | (co.green << 16) | (co.blue << 8) | (co.alpha);
}

int registerWindowCount = 0;

basRegisterWindow::basRegisterWindow()
	: BWindow( BRect(100, 100, 350, 200), "BeAndSee Register", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE )
{
	registerWindowCount++;

	BView *view = new BView( Bounds(), NULL, B_FOLLOW_ALL_SIDES, 0 );
	view->SetViewColor(BeBackgroundGrey);

	OkButton = new BButton( BRect( 190, 70, 245, 95 ), NULL, Locale->Text(TEXT_OK, "Ok"), new BMessage(B_BUTTON_OK));
	view->AddChild(OkButton);
	CancelButton = new BButton( BRect( 130, 70, 185, 95), NULL, Locale->Text(TEXT_CANCEL, "Cancel"), new BMessage(B_BUTTON_CANCEL));
	view->AddChild(CancelButton);

	NameText = new BTextControl( BRect( 5, 5, 245, 30 ), NULL, Locale->Text(TEXT_NAME, "Name"), NULL, NULL);
	NameText->SetDivider( 60 );
	view->AddChild(NameText);

	CodeText[0] = new BTextControl( BRect( 5, 40, 110, 65), NULL, Locale->Text(TEXT_CODE, "Code"), NULL, NULL );
	CodeText[0]->SetDivider( 60 );
	view->AddChild(CodeText[0]);
	CodeText[1] = new BTextControl( BRect( 115, 40, 155, 65), NULL, NULL, NULL, NULL );
	view->AddChild(CodeText[1]);
	CodeText[2] = new BTextControl( BRect( 160, 40, 200, 65), NULL, NULL, NULL, NULL );
	view->AddChild(CodeText[2]);
	CodeText[3] = new BTextControl( BRect( 205, 40, 245, 65), NULL, NULL, NULL, NULL );
	view->AddChild(CodeText[3]);

	AddChild(view);

	Show();
}

bool check_if_key_is_valid( char *inname, unsigned int *in );

void basRegisterWindow::MessageReceived( BMessage *msg )
{
	switch ( msg->what )
	{
		case B_BUTTON_CANCEL:
			PostMessage( B_QUIT_REQUESTED );
			break;
		case B_BUTTON_OK:
			{
				unsigned int inz[4];
				for ( int i=0; i<4; i++ )
				{
					inz[i] = atoi(CodeText[i]->Text());
				}
				if ( strlen(NameText->Text() ) > 0 )
				{
					if ( check_if_key_is_valid( (char*)NameText->Text(), inz ) )
					{
						(new BAlert(Locale->Text(TEXT_WARNING, "Warning"), "The Name and Code Match! Thanks for registering BeAndSee and helping in the growth of BeOS!", Locale->Text(TEXT_THANK_YOU, "Thank You")))->Go();
						BFile check_reg( "/boot/home/config/settings/BeAndSee_key", B_WRITE_ONLY|B_CREATE_FILE);
						char namez[512];
						strcpy(namez, NameText->Text());
						strcat(namez, "\n");
						check_reg.Write( namez, strlen(namez) );
						check_reg.Write( inz, 4 * sizeof(unsigned int));
						PostMessage( B_QUIT_REQUESTED );
					}
					else
					{
						(new BAlert(Locale->Text(TEXT_DARN, "Darn"), "The Name and Code you inserted don't match!\nCheck if you didn't mistake while inserting them or if you still don't have one check the README file to see out to buy BeAndSee.", "Ok!"))->Go();
					}
				}
				else
					(new BAlert(Locale->Text(TEXT_WARNING, "Warning"), "You need to write up the name you used to register BeAndSee in the Name field.", Locale->Text(TEXT_OK_DOTDOT, "Ok..")))->Go();
			}
		default:
			BWindow::MessageReceived(msg);
	}
}

bool basRegisterWindow::QuitRequested()
{
	registerWindowCount--;
	Quit();
	return true;
}

basWindow::basWindow()
	: BWindow( BRect(100, 100, 200, 200), "BeAndSee", B_DOCUMENT_WINDOW, B_WILL_ACCEPT_FIRST_CLICK ),
	SelectFunction(0), InvokeFunction(0), DirLooperTID(-1)
{
	_MainWindow = this;

	if ( !((basApplication*)be_app)->isRegistered )
	{
		basNagScreenWindow *wnd = new basNagScreenWindow();
		status_t status;
		wait_for_thread(wnd->Thread(), &status);
	}

	_PluginInterfaceManager = new basPluginInterfaceManager();
	_PluginInterfaceManager->LoadPlugins();

#ifdef CAMERA_SUPPORT

	new basBDCPInterface();
	BDCPInterface->LoadAll();
	BDCPInterface->LoadDefCams();

#endif

	((basApplication*)be_app)->GetSettingBool( "basWindow::ShowFolderMenu", &fShowFolderMenu );
	((basApplication*)be_app)->GetSettingBool( "basWindow::ShowFolderView", &fShowFolderView );
	((basApplication*)be_app)->GetSettingBool( "basWindow::ShowTumbnailView", &fShowTumbnailView );

	((basApplication*)be_app)->GetSettingInt( "basTumbnailView::TumbnailViewWheelMouseBehaviour", &TumbnailViewWheelMouseBehaviour );

	((basApplication*)be_app)->GetSettingInt( "basFileListView::selectBehaviour", &FileListViewSelectBehaviour );
	((basApplication*)be_app)->GetSettingInt( "basFileListView::invokeBehaviour", &FileListViewInvokeBehaviour );
	((basApplication*)be_app)->GetSettingInt( "basFileListView::FileListViewWheelMouseBehaviour", &FileListViewWheelMouseBehaviour );

	if (!fShowTumbnailView && !fShowFolderView )
	{
		fShowTumbnailView = true;
		fShowFolderView = false;
	}

	if ( fShowTumbnailView && fShowFolderView )
	{
		fShowTumbnailView = true;
		fShowFolderView = false;
	}

	float wx, wy, ww, wh;
	((basApplication*)be_app)->GetSettingFloat("basWindow::WindowFrame::x", &wx);
	((basApplication*)be_app)->GetSettingFloat("basWindow::WindowFrame::y", &wy);
	((basApplication*)be_app)->GetSettingFloat("basWindow::WindowFrame::w", &ww);
	((basApplication*)be_app)->GetSettingFloat("basWindow::WindowFrame::h", &wh);
	MoveTo( wx, wy );
	ResizeTo(ww, wh);

	SetType( B_TITLED_WINDOW );

	FileListView = new basFileListView();
	TumbnailView = new basTumbnailView(Bounds());
	TreeView = new basTreeView();

	watch_node( NULL, B_WATCH_MOUNT, be_app_messenger);

	TumbnailWorker = new basTumbnailWorker(TumbnailView);

	imageNavigationList = new BList();

	BMenuItem * item;

	MenuBar = new BMenuBar( Bounds(), "menu_bar" );
	BMenu *menu = new BMenu(Locale->Text(TEXT_FILE, "File"));
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_SAVE_SETTINGS, "Save Settings"), new BMessage(B_MENU_SAVE_SETTINGS), 'G'));

	menu->AddItem( new BMenuItem( Locale->Text(TEXT_PREFERENCES, "Preferences"), new BMessage(B_MENU_SHOW_PREFERENCES), 'P' ) );

	menu->AddSeparatorItem();
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_SHOW_IMAGELIST, "Show ImageList Window"), new BMessage(B_SHOW_IMAGELISTWINDOW), 'I' ) );

	if ( !((basApplication*)be_app)->isRegistered )
	{
		menu->AddSeparatorItem();
		menu->AddItem( new BMenuItem( Locale->Text(TEXT_ENTER_REGISTRATION_CODE, "Enter Registration code..."), new BMessage(B_MENU_REGISTER) ) );
	}

	menu->AddSeparatorItem();
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_ABOUT, "About"), new BMessage(B_ABOUT_REQUESTED), 'A'));
	menu->AddSeparatorItem();
	menu->AddItem( new BMenuItem( Locale->Text(TEXT_QUIT, "Quit"), new BMessage(B_MENU_QUIT_REQUESTED), 'Q'));
	MenuBar->AddItem(menu);

	menu = new BMenu(Locale->Text(TEXT_VIEW, "View"));
	BMenu *submenu = new BMenu( Locale->Text(TEXT_FOLDERVIEW, "FolderView") );

	item = new BMenuItem( Locale->Text(TEXT_SHOW_IMAGES_ONLY, "Show Images Only"), new BMessage(B_MENU_SHOW_IMAGES_ONLY), 'O');
	item->SetMarked(FileListView->fShowImagesOnly);
	submenu->AddItem( item );
	item = new BMenuItem( Locale->Text(TEXT_SHOW_FOLDERS_FIRST, "Show Folders First"), new BMessage(B_MENU_SHOW_FOLDERS_FIRST), 'F');
	item->SetMarked(FileListView->fShowFoldersFirst);
	submenu->AddItem( item );
	submenu->AddSeparatorItem();

	BMenu *omenu = new BMenu( Locale->Text(TEXT_FOLDERVIEW_COLUMNS, "FolderView Columns") );
	item = new BMenuItem(Locale->Text(TEXT_NAME, "Name"), new BMessage(B_MENU_SET_FOLDER_COL_SHOWN));
	item->SetMarked(FileListView->fShowColumn[0]);
	omenu->AddItem( item);
	item = new BMenuItem(Locale->Text(TEXT_SIZE, "Size"), new BMessage(B_MENU_SET_FOLDER_COL_SHOWN));
	item->SetMarked(FileListView->fShowColumn[1]);
	omenu->AddItem( item);
	item = new BMenuItem(Locale->Text(TEXT_MODIFIED, "Modified"), new BMessage(B_MENU_SET_FOLDER_COL_SHOWN));
	item->SetMarked(FileListView->fShowColumn[2]);
	omenu->AddItem( item);
	item = new BMenuItem(Locale->Text(TEXT_CREATED, "Created"), new BMessage(B_MENU_SET_FOLDER_COL_SHOWN));
	item->SetMarked(FileListView->fShowColumn[3]);
	omenu->AddItem( item);
	submenu->AddItem(omenu);

	menu->AddItem(submenu);
	menu->AddSeparatorItem();

	item = new BMenuItem( Locale->Text(TEXT_SHOW_FOLDER_VIEW, "Show Folder View"), new BMessage(B_MENU_SHOW_FOLDER_VIEW), 'V');
	item->SetMarked(fShowFolderView);
	menu->AddItem( item );
	item = new BMenuItem( Locale->Text(TEXT_SHOW_TUMBNAILS, "Show Thumbnails"), new BMessage(B_MENU_SHOW_TUMBNAILS), 'T');
	item->SetMarked(fShowTumbnailView);
	menu->AddItem( item);

	MenuBar->AddItem(menu);

#ifdef CAMERA_SUPPORT

	menu = new BMenu( Locale->Text( TEXT_TOOLS, "Tools") );

	menu->AddItem( new BMenuItem(Locale->Text( TEXT_DIGITAL_CAMERA_CONFIG_DOTDOTDOT, "Digital Camera Config..."), new BMessage(B_MENU_CAMERA_CONFIG) ));

	MenuBar->AddItem(menu);

#endif

	AddChild(MenuBar);
	SetFileListInvokeFunction( basWindow::DefaultOnInvoke );
	SetFileListSelectFunction( basWindow::DefaultOnSelect );

	SetSizeLimits( 400, 4000, 200, 4000 );

	BRect rect(Bounds());
	rect.top += MenuBar->Bounds().Height();
	if ( fShowTumbnailView )
		Split = new SplitPane( rect, TreeView->Container(), TumbnailView->Container(), B_FOLLOW_ALL_SIDES );
	else if ( fShowFolderView )
		Split = new SplitPane( rect, TreeView->Container(), FileListView->Container(), B_FOLLOW_ALL_SIDES );

	Split->SetMinSizeOne( BPoint(200, 200) );
	Split->SetMinSizeTwo( BPoint(200, 200) );

	AddChild(Split);

	BitmapWorker = new basBitmapWorker();

	char *def_place;
	((basApplication*)be_app)->GetSettingString( "basWindow::DefaultPlace", &def_place );
	BEntry home( def_place );
	if ( !home.Exists() )
	{
		BPath path;
		find_directory( B_USER_DIRECTORY, &path );
		home.SetTo(path.Path());
	}
	DoInvokeFunction(&home);
	entry_ref Ref;
	home.GetRef(&Ref);
	TreeView->GoTo(Ref);

	PictureWindow = new basPictureWindow(this);
	ImageListWindow = new basImageListWindow(this);
	ImageListWindow->MoveTo( 100, 100 );

	SetType( B_DOCUMENT_WINDOW );

	Show();
}

basWindow::~basWindow()
{
	delete _PluginInterfaceManager;
	delete BitmapWorker;
	if (DirLooperTID >= 0)
		kill_thread(DirLooperTID);
	entry_ref *ref;
	while ( (ref = (entry_ref*)imageNavigationList->FirstItem()) )
	{
		imageNavigationList->RemoveItem(ref);
		delete ref;
	}
	delete imageNavigationList;
	node_ref *noderef;
	while ( ( noderef = (node_ref*)DirectoryNodeList.FirstItem() ) )
	{
		watch_node( noderef, B_WATCH_ALL, be_app_messenger );
		DirectoryNodeList.RemoveItem(noderef);
		delete noderef;
	}
	delete BDCPInterface;
}

bool Entry_IsImageList( BEntry *entry )
{
	BNode node(entry);
	BNodeInfo info(&node);
	char type[B_FILE_NAME_LENGTH];
	if ( info.GetType( type ) != B_OK )
		return false;
	return strcmp( type, "application/BeAndSee-ImageList" ) == 0;
}

void basWindow::DefaultOnInvoke( basWindow *W, BEntry *entry )
{
	if ( entry->IsDirectory() || Entry_IsImageList( entry ))
	{
		W->Lock();

		node_ref ref, lref;
		BDirectory dir(entry);
		dir.GetNodeRef(&ref);
		BEntry ent( &W->FileListView->OpenFolder );
		BDirectory ldir( &ent );
		ldir.GetNodeRef( &lref );
		watch_node( &lref, B_STOP_WATCHING, be_app_messenger );
//		stop_watching(be_app_messenger);
		watch_node( &ref, B_WATCH_DIRECTORY, be_app_messenger );
		watch_node( &ref, B_WATCH_NAME, be_app_messenger );

		BPath path(entry);
		entry->GetRef(&W->FileListView->OpenFolder);
		((basApplication*)be_app)->SetSettingString( "basWindow::DefaultPlace", (char*)path.Path() );
		char title[20 + B_FILE_NAME_LENGTH];
		sprintf(title, "BeAndSee - %s", path.Path());
		W->SetTitle(title);

		BMessage *newmsg = new BMessage;
		entry_ref eref;
		entry->GetRef(&eref);
		newmsg->AddPointer("basWindow", W);
		newmsg->AddRef("refs", &eref);

		if ( W->DirLooperTID >= 0 )
		{
			kill_thread(W->DirLooperTID);
		}

		W->DirLooperTID = spawn_thread( DirLooper, "DirLooper", B_NORMAL_PRIORITY, (void*)newmsg);
		resume_thread(W->DirLooperTID);

		W->Unlock();
	}
}

void basWindow::InvokeCleanAll()
{
	if ( DirLooperTID >= 0 )
		kill_thread(DirLooperTID);

	stop_watching(be_app_messenger);
	SetTitle("BeAndSee");

	if ( fShowFolderView )
		FileListView->CleanListContent();
	if ( fShowTumbnailView )
	{
		TumbnailWorker->Stop();
		TumbnailView->CleanContents();
	}

	TumbnailView->SetStatusLabel("");

	entry_ref *tmpref;
	while ( (tmpref = (entry_ref*)imageNavigationList->FirstItem() ) )
	{
		imageNavigationList->RemoveItem(tmpref);
		delete tmpref;
	}

	node_ref *noderef;
	while ( ( noderef = (node_ref*)DirectoryNodeList.FirstItem() ) )
	{
		watch_node( noderef, B_STOP_WATCHING, be_app_messenger );
		DirectoryNodeList.RemoveItem(noderef);
		delete noderef;
	}
}

void basWindow::InvokeListCameraContents( int i )
{
	if ( DirLooperTID >= 0 )
		kill_thread(DirLooperTID);

	stop_watching(be_app_messenger);

	if ( fShowFolderView )
		FileListView->CleanListContent();
	if ( fShowTumbnailView )
	{
		TumbnailWorker->Stop();
		TumbnailView->CleanContents();
	}

	TumbnailView->SetStatusLabel("");

	entry_ref *tmpref;
	while ( (tmpref = (entry_ref*)imageNavigationList->FirstItem() ) )
	{
		imageNavigationList->RemoveItem(tmpref);
		delete tmpref;
	}

	node_ref *noderef;
	while ( ( noderef = (node_ref*)DirectoryNodeList.FirstItem() ) )
	{
		watch_node( noderef, B_STOP_WATCHING, be_app_messenger );
		DirectoryNodeList.RemoveItem(noderef);
		delete noderef;
	}

	char title[20 + B_FILE_NAME_LENGTH];
	sprintf(title, "BeAndSee - %s", BDCPInterface->GetDefCams()[i]->DefName.c_str());

	SetTitle(title);

	BMessage *newmsg = new BMessage;
	newmsg->AddPointer("basWindow", this);
	newmsg->AddInt32("camera_index", i);
	resume_thread(spawn_thread( InvokeListCameraContentsThread, "InvokeListCameraContentsThread", B_NORMAL_PRIORITY, (void*)newmsg));
}

int32 basWindow::InvokeListCameraContentsThread( void *arg )
{
	BMessage *msg = (BMessage*)arg;
	basWindow *wnd;
	msg->FindPointer("basWindow", (void**)&wnd);
	int32 c_i = msg->FindInt32("camera_index");

	basBDCP *pi = BDCPInterface->GetPluginList() [ BDCPInterface->GetDefCams()[ c_i ]->plugIndex ];

	char _msg[256];
	sprintf(_msg, "trying to connect to [%s]...", BDCPInterface->GetDefCams()[ c_i ]->DefName.c_str() );
	wnd->Lock();
	wnd->TumbnailView->SetStatusLabel(_msg);
	wnd->Unlock();

	if ( pi->OpenCamera() )
	{
		wnd->Lock();
		wnd->TumbnailView->SetStatusLabel("connected successfully!");
		wnd->Unlock();

		int npics;
		pi->GetNumberOfPics(npics);
		for (int i=0; i<npics; i++)
		{
			BBitmap *tb;

			if ( (pi->Caps() & _GET_THUMBNAIL) == _GET_THUMBNAIL )
			{
				pi->GetThumbnail(i, tb);
				if ( tb->Bounds().Width() > wnd->TumbnailView->tumbnailSize ||
					 tb->Bounds().Height() > wnd->TumbnailView->tumbnailSize )
				{
					BBitmap *_tb = _BitmapWorker->CreateTumbnail( tb, wnd->TumbnailView->tumbnailSize, wnd->TumbnailView->tumbnailSize, true, true, false );
					delete tb;
					tb = _tb;
				}
			}
			else
				tb = NULL;

			char fname[256];

			if ( (pi->Caps() & _GET_IMAGE_NAME) == _GET_IMAGE_NAME )
			{
				char *name;
				pi->GetImageName(i, name);
				strcpy(fname, name);
			}
			else
			{
				if ( (pi->Caps() & _GET_IMAGE_DATE) == _GET_IMAGE_DATE )
				{
					char *date;
					pi->GetImageDate(i, date);
					strcpy(fname, date);
				}
				else
				{
					sprintf(fname, "Image %i", i+1);
				}
			}

			int size = -1;
			if ( (pi->Caps() & _GET_IMAGE_SIZE) == _GET_IMAGE_SIZE )
				pi->GetImageSize(i, size);

			if ( wnd->fShowTumbnailView )
			{
				wnd->TumbnailView->ShowFolderEaner( fname, size, tb );
			}
		}
	}
	else
	{
		wnd->Lock();
		wnd->TumbnailView->SetStatusLabel("failed to connect!");
		wnd->Unlock();
		(new BAlert(Locale->Text(TEXT_ERROR, "Error"), "Failed to connect to the camera!\nIs it correctly plugged to the computer?", Locale->Text(TEXT_OK, "Ok")))->Go();
	}

	return 0;
}

void basWindow::ReloadFileListView()
{
	BMessage *newmsg = new BMessage;
	newmsg->AddPointer("basWindow", this);
	newmsg->AddRef("refs", &FileListView->OpenFolder);
	newmsg->AddInt32("flags", 4 );
	if ( DirLooperTID >= 0 )
		kill_thread(DirLooperTID);
	DirLooperTID = spawn_thread( DirLooper, "DirLooper", B_NORMAL_PRIORITY, (void*)newmsg);
	resume_thread(DirLooperTID);
}

void basWindow::ReloadTumbnailView()
{
	BMessage *newmsg = new BMessage;
	newmsg->AddPointer("basWindow", this);
	newmsg->AddRef("refs", &FileListView->OpenFolder);
	newmsg->AddInt32("flags", 2 );
	if ( DirLooperTID >= 0 )
		kill_thread(DirLooperTID);
	DirLooperTID = spawn_thread( DirLooper, "DirLooper", B_NORMAL_PRIORITY, (void*)newmsg);
	resume_thread(DirLooperTID);
}

void basWindow::DefaultOnSelect( basWindow *W, BEntry *entry )
{
	W->PictureWindow->Lock();
	int _i = W->imageNavigationList->CountItems();
	for ( int i = 0; i< _i; i ++ )
	{
		entry_ref *tmpref = (entry_ref*)W->imageNavigationList->ItemAt(i);
		entry_ref ref;
		entry->GetRef(&ref);
		if ( *tmpref == ref )
		{
			W->PictureWindow->navigationPos = i;
			break;
		}
	}
	W->PictureWindow->BigPicture->LoadFrom(entry);
	W->PictureWindow->BigPicture->Invalidate();
	W->PictureWindow->Unlock();
}

void basWindow::OnSelect(basWindow *W, BEntry *ent )
{
	switch ( W->FileListViewSelectBehaviour )
	{
		case 1:
			DefaultOnSelect(W, ent);
			break;
		case 2:
			if ( ent->IsDirectory() )
				DefaultOnInvoke(W, ent);
			else
				DefaultOnSelect(W, ent);
			break;
	}
}

void basWindow::OnInvoke(basWindow *W, BEntry *ent)
{
	switch ( W->FileListViewInvokeBehaviour )
	{
		case 1:
			DefaultOnSelect(W, ent);
			break;
		case 2:
			if ( ent->IsDirectory() )
				DefaultOnInvoke(W, ent);
			else
				DefaultOnSelect(W, ent);
			break;
		case 3:
			DefaultOnInvoke(W, ent);
			break;
	}
}

void basWindow::MessageReceived( BMessage *msg )
{
	switch (msg->what)
	{
	case B_SIMPLE_DATA:
		be_app->PostMessage(msg);
		break;
	case B_MOUSE_WHEEL_CHANGED:
		{
			float mv = msg->FindFloat("be:wheel_delta_y");
			BPoint where;
			uint32 bts;

			Split->GetMouse( &where, &bts, false );

			if ( TreeView->Container()->Bounds().Contains(where) )
			{
				TreeView->ScrollBy( 0, (mv*20) );
			}
			else if ( fShowFolderView && FileListView->Container()->Bounds().Contains(where) )
			{
				switch ( FileListViewWheelMouseBehaviour )
				{
					case 1:
						FileListView->ScrollBy( 0, (mv*20) );
						break;
					case 2:
						FileListView->ScrollBy( 0, mv*FileListView->Bounds().Height() );
						break;
					case 3:
						{
							int _mv = (int)mv;
							int32 a = FileListView->CurrentSelection();
							a += _mv;
							if ( a < 0 ) a = 0;
							if ( a >= FileListView->CountItems() ) a = FileListView->CountItems() - 1;
							FileListView->Select(a);
							FileListView->ScrollToSelection();
						}
						break;
				}
			}
			else if ( fShowTumbnailView )
			{
				switch ( TumbnailViewWheelMouseBehaviour )
				{
					case 0:
						break;
					case 1:
						{
							float zx = mv * (TumbnailView->tumbnailSize+28+24);
							TumbnailView->ScrollBy( 0, zx );
						}
						break;
					case 2:
						{
							float zx = (TumbnailView->Bounds().Height());
							TumbnailView->ScrollBy( 0, zx );
						}
						break;
				}
			}
		}
		break;
	case B_KEY_DOWN:
		{
			int32 byte;
			msg->FindInt32( "raw_char", &byte );

			if (fShowFolderView )
			{
				int32 a = FileListView->CurrentSelection();
				switch ( byte )
				{
					case B_UP_ARROW:
						{
							if ( a > 0 )
								FileListView->Select( a -1 );
						}
						break;
					case B_DOWN_ARROW:
						{
							if ( FileListView->ItemAt( a +1) != NULL )
								FileListView->Select(a+1);
						}
						break;
#define HUM	FileListView->NumberOfVisibleRows()
					case B_PAGE_UP:
						{
							if ( (a-HUM) > 0)
								FileListView->Select(a-HUM);
							else
								FileListView->Select( 0 );
						}
						break;
					case B_PAGE_DOWN:
						{
							if ( FileListView->ItemAt( a +HUM) != NULL )
								FileListView->Select(a+HUM);
							else
								FileListView->Select( FileListView->CountItems()-1 );
						}
						break;
#undef HUM
					case B_ENTER:
						FileListView->Invoke();
						break;
				}
				FileListView->ScrollToSelection();
			}
		}
		break;
	case B_MESSAGE_INVOKE:
	{
		basFileListView *view;
		msg->FindPointer("source", (void**)&view);
		if (view)
		{
			basFileListItem *item = (basFileListItem*)view->ItemAt(view->CurrentSelection());
			if ( item )
			{
				BEntry entry( &item->Ref );
				if ( InvokeFunction )
					OnInvoke(this, &entry);
			}
		}
		break;
	}
	case B_MESSAGE_INVOKE2:
	{
		basFileListView *view;
		msg->FindPointer("source", (void**)&view);
		if (view)
		{
			basTreeItem *item = (basTreeItem*)view->ItemAt(view->CurrentSelection());
			if ( item )
			{
				if ( item->GetType() == 0 )
				{
					BEntry entry( &item->Ref() );
					DefaultOnInvoke(this, &entry);
				}
				else if ( item->GetType() == 1 )
				{
					InvokeCleanAll();
				}
				else if ( item->GetType() == 2 )
				{
					InvokeListCameraContents( item->GetArg() );
				}
			}
		}
		break;
	}
	case B_MESSAGE_SELECT:
	{
		basFileListView *view;
		msg->FindPointer("source", (void**)&view);
		if (view)
		{
			basFileListItem *item = (basFileListItem*)view->ItemAt(view->CurrentSelection());
			if ( item )
			{
				BEntry entry( &item->Ref );
				if ( SelectFunction )
					OnSelect(this, &entry);
			}
		}
		break;
	}
	case B_ABOUT_REQUESTED:
		ShowAboutBox();
		break;
	case B_MENU_QUIT_REQUESTED:
		QuitRequested();
		break;
	case B_MENU_SHOW_IMAGES_ONLY:
		FileListView->fShowImagesOnly = !FileListView->fShowImagesOnly;
		MenuBar->SubmenuAt(1)->SubmenuAt(0)->ItemAt(0)->SetMarked( FileListView->fShowImagesOnly );
		((basApplication*)be_app)->SetSettingBool( "basWindow::ShowImagesOnly", FileListView->fShowImagesOnly);
		ReloadFileListView();
		break;
	case B_MENU_OPEN_PARENT_FOLDER:
		{
			int32 a;
			msg->FindInt32("index", &a);
			BEntry ent( &FileListView->OpenFolder );
			BPath dirpath(&ent);
			while ( a >= 0 )
			{
				dirpath.GetParent(&dirpath);
				a--;
			}
			ent.SetTo( dirpath.Path());
			DoInvokeFunction(&ent);
		}
		break;
	case B_MENU_SHOW_FOLDER_VIEW:
	case B_MENU_SHOW_TUMBNAILS:
		{
			fShowTumbnailView = !fShowTumbnailView;
			fShowFolderView = !fShowTumbnailView;
			((basApplication*)be_app)->SetSettingBool( "basWindow::ShowTumbnailView", fShowTumbnailView );
			((basApplication*)be_app)->SetSettingBool( "basWindow::ShowFolderView", fShowFolderView );

			MenuBar->SubmenuAt(1)->ItemAt(2)->SetMarked(!fShowTumbnailView);
			MenuBar->SubmenuAt(1)->ItemAt(3)->SetMarked(fShowTumbnailView);

			if ( fShowTumbnailView )
			{
				SetType( B_TITLED_WINDOW );
				Split->RemoveChild( FileListView->Container() );
				Split->AddChildTwo( TumbnailView->Container() );
				ReloadTumbnailView();
				SetType( B_DOCUMENT_WINDOW );
			}
			else
			{
				Split->RemoveChild( TumbnailView->Container() );
				Split->AddChildTwo( FileListView->Container() );
				ReloadFileListView();
			}

			Split->Update();
		}
		break;
	case B_UPDATE_TUMBNAIL_VIEW:
		break;
	case B_TUMBNAIL_VIEW_ADDED_ITEM:
		if ( fShowTumbnailView )
		{
			int32 i;
			msg->FindInt32( "tumbnail_item_index", &i );
			TumbnailView->AddedItem( i);
		}
		break;
	case B_MENU_SHOW_FOLDERS_FIRST:
		FileListView->fShowFoldersFirst = !FileListView->fShowFoldersFirst;
		MenuBar->SubmenuAt(1)->SubmenuAt(0)->ItemAt(1)->SetMarked( FileListView->fShowFoldersFirst );
		((basApplication*)be_app)->SetSettingBool( "basWindow::ShowFoldersFirst", FileListView->fShowFoldersFirst);
		ReloadFileListView();
		break;
	case B_MENU_SHOW_IMAGE:
		{
			BEntry entry(&TumbnailView->PopUpedTumbnail->Entry);
			PictureWindow->Lock();
			PictureWindow->BigPicture->LoadFrom(&entry);
			PictureWindow->BigPicture->Invalidate();
			PictureWindow->Unlock();
		}
		break;
	case B_MENU_SHOW_IMAGE_IN_NEW_WINDOW:
		{
			BEntry entry(&TumbnailView->PopUpedTumbnail->Entry);
			basPictureWindow *wnd = new basPictureWindow( this, true );
			wnd->Lock();
			wnd->MoveTo(100, 100);
			wnd->BigPicture->LoadFrom(&entry);
			wnd->Unlock();
			wnd->Show();
		}
		break;
	case B_MENU_SET_TUMBNAIL_SIZE:
		{
			int32 index;
			msg->FindInt32("index", &index);
			int _i = TumbnailView->nosPopUp->SubmenuAt(2)->CountItems();
			for ( int z = 0; z < _i; z ++ )
			{
				TumbnailView->nosPopUp->SubmenuAt(2)->ItemAt(z)->SetMarked(false);
				TumbnailView->msPopUp->SubmenuAt(2)->ItemAt(z)->SetMarked(false);
				TumbnailView->PopUp->SubmenuAt(13)->ItemAt(z)->SetMarked(false);
			}
			TumbnailView->PopUp->SubmenuAt(13)->ItemAt(index)->SetMarked(true);
			TumbnailView->msPopUp->SubmenuAt(2)->ItemAt(index)->SetMarked(true);
			TumbnailView->nosPopUp->SubmenuAt(2)->ItemAt(index)->SetMarked(true);
			switch (index)
			{
				case 0:
					SetTumbnailSize( 25 );
					break;
				case 1:
					SetTumbnailSize( 50 );
					break;
				case 2:
					SetTumbnailSize( 75 );
					break;
				case 3:
					SetTumbnailSize( 100 );
					break;
				case 4:
					SetTumbnailSize( 150 );
					break;
				case 5:
					SetTumbnailSize( 200 );
					break;
			}
		}
		break;
	case B_MENU_SAVE_SETTINGS:
		SaveSettings();
		break;
	case B_MENU_SET_FOLDER_COL_SHOWN:
		{
			int32 a;
			msg->FindInt32( "index", &a);
			FileListView->fShowColumn[a] = !FileListView->fShowColumn[a];
			MenuBar->SubmenuAt(1)->SubmenuAt(0)->SubmenuAt(3)->ItemAt(a)->SetMarked( FileListView->fShowColumn[a]);
			FileListView->ColumnAt(a+1)->SetShown( FileListView->fShowColumn[a] );
			switch ( a )
			{
				case 0:
					((basApplication*)be_app)->SetSettingBool( "basWindow::ShowNameColumn", FileListView->fShowColumn[0] );
					break;
				case 1:
					((basApplication*)be_app)->SetSettingBool( "basWindow::ShowSizeColumn", FileListView->fShowColumn[1] );
					break;
				case 2:
					((basApplication*)be_app)->SetSettingBool( "basWindow::ShowModifiedTimeColumn", FileListView->fShowColumn[1] );
					break;
				case 3:
					((basApplication*)be_app)->SetSettingBool( "basWindow::ShowCreatedTimeColumn", FileListView->fShowColumn[1] );
					break;
			}
		}
		break;
	case B_MENU_DELETE_IMAGE:
		{
			if ( TumbnailView->PopUpedTumbnail )
			{
				char msg[2048];
				sprintf(msg, Locale->Text(TEXT_DO_YOU_REALLY_WISH_TO_MOVE_S_TO_TRASH, "Do you really wish to move %s to Trash?"), TumbnailView->PopUpedTumbnail->realName );
				int res = (new BAlert(Locale->Text(TEXT_QUESTION, "Question"), msg, Locale->Text(TEXT_YES, "Yes"), Locale->Text(TEXT_NO, "No"), NULL, B_WIDTH_AS_USUAL, B_WARNING_ALERT))->Go();
				if ( res == 0 )
				{
					BEntry ben( &TumbnailView->PopUpedTumbnail->Entry );
					MoveToTrash( &ben );
				}
			}
		}
		break;
	case B_MENU_PREFS_SET_COLOR_T_V:
		{
			BColorPickerWindow *wnd = new BColorPickerWindow();
			wnd->SetMessage( new BMessage( B_MENU_PREFS_SET_COLOR_T_V_MSG ));
			wnd->SetColor( TumbnailView->ViewColor() );
			wnd->SetTitle("ColorPicker: TumbnailView's Background");
			wnd->MoveTo(200, 200);
			wnd->Show();
		}
		break;
	case B_MENU_PREFS_SET_COLOR_T_V_MSG:
		{
			int32 color = msg->FindInt32("Color");
			rgb_color C;
			to_rgb(color, C);
			((basApplication*)be_app)->SetSettingInt( "basTumbnailView::ViewColor", color);
			TumbnailView->SetViewColor(C);
			TumbnailView->Invalidate();
		}
		break;
	case B_MENU_PREFS_SET_COLOR_T_B:
		{
			BColorPickerWindow *wnd = new BColorPickerWindow();
			wnd->SetMessage( new BMessage( B_MENU_PREFS_SET_COLOR_T_B_MSG ));
			wnd->SetColor( TumbnailView->TumbnailBackgroundColor );
			wnd->SetTitle("ColorPicker: Tumbnail's Background");
			wnd->MoveTo(200, 200);
			wnd->Show();
		}
		break;
	case B_MENU_PREFS_SET_COLOR_T_B_MSG:
		{
			int32 color = msg->FindInt32("Color");
			rgb_color C;
			to_rgb(color, C);
			((basApplication*)be_app)->SetSettingInt( "basTumbnailView::TumbnailBackground", color);
			TumbnailView->SetTumbnailBackgroundColor(C);
			TumbnailView->Invalidate();
		}
		break;
	case B_MENU_PREFS_SET_COLOR_T_H:
		{
			BColorPickerWindow *wnd = new BColorPickerWindow();
			wnd->SetMessage( new BMessage( B_MENU_PREFS_SET_COLOR_T_H_MSG ));
			wnd->SetColor( TumbnailView->TumbnailHoverColor );
			wnd->SetTitle("ColorPicker: Tumbnail's onMouseOver Color");
			wnd->MoveTo(200, 200);
			wnd->Show();
		}
		break;
	case B_MENU_PREFS_SET_COLOR_T_H_MSG:
		{
			int32 color = msg->FindInt32("Color");
			rgb_color C;
			to_rgb(color, C);
			((basApplication*)be_app)->SetSettingInt( "basTumbnailView::TumbnailHover", color);
			TumbnailView->SetTumbnailHoverColor(C);
			TumbnailView->Invalidate();
		}
		break;
	case B_FOLDERVIEW_SORT_ITEMS:
		break;
	case B_MENU_REGISTER:
		if ( registerWindowCount < 1 )
			new basRegisterWindow();
		break;
	case B_MENU_COPY_IMAGE:
		SavePanelCopyMode = 1;
		(new BFilePanel( B_SAVE_PANEL, new BMessenger(this), NULL, B_FILE_NODE, false, NULL, NULL, true, true ))->Show();
		break;
	case B_MENU_MOVE_IMAGE:
		SavePanelCopyMode = 2;
		(new BFilePanel( B_SAVE_PANEL, new BMessenger(this), NULL, B_FILE_NODE, false, NULL, NULL, true, true ))->Show();
		break;
	case B_SAVE_REQUESTED:
		{
			BFile in_file( &TumbnailView->PopUpedTumbnail->Entry, B_READ_ONLY );
			entry_ref dir;
			msg->FindRef("directory", &dir);
			char *name;
			msg->FindString("name", (const char **)&name);
			BEntry entry(&dir);
			BPath Path(&entry);
			char path[B_PATH_NAME_LENGTH];
			sprintf(path, "%s/%s", Path.Path(), name );
			BFile out_file( path, B_WRITE_ONLY | B_CREATE_FILE );
			char dumb;
			while ( in_file.Read((void*)&dumb, 1) > 0 )
			{
				out_file.Write((const void*)&dumb, 1);
			}

			if ( SavePanelCopyMode == 2)
			{
				BEntry ent(&TumbnailView->PopUpedTumbnail->Entry);
				ent.Remove();
			}
		}
		break;
	case B_CANCEL:
		{
			BFilePanel *panel;
			msg->FindPointer("source", (void**)&panel);
			panel->Window()->Quit();
		}
		break;
	case B_SHOW_IMAGELISTWINDOW:
		if (ImageListWindow->IsHidden())
			ImageListWindow->Show();
		break;
	case B_MENU_ADD_IMAGE_TO_IMAGELIST:
		if ( TumbnailView->tumbnailsSelection->CountItems() >= 2 )
		{
			int _i = TumbnailView->tumbnailsSelection->CountItems();
			for ( int i=0; i<_i; i++ )
			{
				ImageListWindow->AddRef( &((Tumbnail*)TumbnailView->tumbnailsSelection->ItemAt(i))->Entry );
			}
		}
		else
			ImageListWindow->AddRef( &TumbnailView->PopUpedTumbnail->Entry );
		break;
	case B_MENU_ADD_ALL_IMAGES_TO_IMAGELIST:
		{
			for ( int i=0; i<TumbnailView->numberOfTumbnails; i++ )
			{
				ImageListWindow->AddRef( &((Tumbnail*)TumbnailView->TumbnailList.ItemAt(i))->Entry );
			}
		}
		break;
	case B_MENU_SHOW_PREFERENCES:
		new basPreferencesWindow(this);
		break;
	case B_RELOAD_SETTINGS:
		{
			((basApplication*)be_app)->GetSettingInt( "basFileListView::selectBehaviour", &FileListViewSelectBehaviour );
			((basApplication*)be_app)->GetSettingInt( "basFileListView::invokeBehaviour", &FileListViewInvokeBehaviour );
			((basApplication*)be_app)->GetSettingInt( "basFileListView::FileListViewWheelMouseBehaviour", &FileListViewWheelMouseBehaviour );

			((basApplication*)be_app)->GetSettingInt( "basTumbnailView::TumbnailViewWheelMouseBehaviour", &TumbnailViewWheelMouseBehaviour );

			int tv1, tv2, tv3, tv4;

			((basApplication*)be_app)->GetSettingInt( "basTumbnailView::leftButtonBehaviour", &tv1);
			((basApplication*)be_app)->GetSettingInt( "basTumbnailView::middleButtonBehaviour", &tv2 );
			((basApplication*)be_app)->GetSettingInt( "basTumbnailView::rightButtonBehaviour", &tv3 );
			((basApplication*)be_app)->GetSettingInt( "basTumbnailView::doubleButtonBehaviour", &tv4 );

			TumbnailView->SetLeftMouseButtonBehaviour( tv1 );
			TumbnailView->SetMiddleMouseButtonBehaviour( tv2 );
			TumbnailView->SetRightMouseButtonBehaviour( tv3 );
			TumbnailView->SetDoubleMouseButtonBehaviour( tv4 );

			bool temp_bool;
			((basApplication*)be_app)->GetSettingBool( "basApplication::TumbnailCachingEnabled", &temp_bool );
			TumbnailCache->Enable(temp_bool);

			int col;
			rgb_color color;
			((basApplication*)be_app)->GetSettingInt( "basPictureWindow::BackColor", &col );
			to_rgb(col, color);

			PictureWindow->Lock();
			PictureWindow->BigPicture->SetViewColor(color);
			PictureWindow->BigPicture->Invalidate();
			PictureWindow->Unlock();
		}
		break;
	case B_MENU_SEND_IMAGE_BY_EMAIL:
		{
			BEntry entry(&TumbnailView->PopUpedTumbnail->Entry);
			BPath path(&entry);
			new basSendEmailWindow( path.Path() );
		}
		break;
	case B_MENU_CAMERA_CONFIG:
		new basCameraConfigWindow();
		break;
	case B_MENU_OPEN_WITH_PREFERED_APPLICATION:
		{
			BMessage *smsg = new BMessage(B_REFS_RECEIVED);
			smsg->AddRef( "refs", &TumbnailView->PopUpedTumbnail->Entry);
			char *_app;
			((basApplication*)be_app)->GetSettingString( "basApplication::prefered_app", &_app );
			be_roster->Launch( _app, smsg );
		}
		break;
	default:
		BWindow::MessageReceived(msg);
	}
}

bool basWindow::QuitRequested()
{
	Split->GetState();
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

void basWindow::ShowAboutBox()
{
	char msg[4096] = "BeAndSee v0.4.5\n\nCopyright (c) Hugo Santos (hugo_santos@bigfoot.com)\n\n";
	strcat(msg, Locale->Text(TEXT_ABOUT_TEXT, "Thanks to all the people that gave me ideas for BeAndSee and helped me testing it as well to all the beos user and developer community.\n\n - With code (ColumnListView Class) by Brian Tietz (briant@timelinevista.com.)\n"));
	(new BAlert( Locale->Text(TEXT_ABOUT_DOTDOTDOT, "About..."), msg, Locale->Text(TEXT_YEAH_DOTDOT_RIGHT, "Yeah.. right")))->Go();
}

void basWindow::FrameMoved( BPoint point )
{
	// TODO: Window Snapping
	((basApplication*)be_app)->SetSettingFloat("basWindow::WindowFrame::x", point.x);
	((basApplication*)be_app)->SetSettingFloat("basWindow::WindowFrame::y", point.y);
}

void basWindow::FrameResized( float width, float height )
{
	((basApplication*)be_app)->SetSettingFloat("basWindow::WindowFrame::w", width);
	((basApplication*)be_app)->SetSettingFloat("basWindow::WindowFrame::h", height);

	Split->MoveTo(0, MenuBar->Bounds().Height()+1);
	Split->ResizeTo( width, height - MenuBar->Bounds().Height()-1 );
}

void basWindow::RefsReceived( entry_ref *ref )
{
	BEntry entry(ref);
	DefaultOnInvoke( this, &entry );
}

void basWindow::SetTumbnailSize( int new_size )
{
	TumbnailView->SetTumbnailSize( new_size );
	if ( fShowTumbnailView )
	{
		ReloadTumbnailView();
	}
}

void basWindow::SaveSettings()
{
	Split->GetState();
	((basApplication*)be_app)->SaveSettings();
}

void basWindow::EntryWasCreated( entry_ref *ref, node_ref *nref )
{
	if ( nref )
	{
//		TODO: FIX THIS
		BNode nodez(&FileListView->OpenFolder);
		node_ref nnref;
		nodez.GetNodeRef(&nnref);
		if ( nref->node != nnref.node )
			return;
		watch_node( nref, B_WATCH_ALL, be_app_messenger );
	}
	BEntry ent(ref);
	if ( fShowTumbnailView )
	{
		TumbnailView->ShowFolderEaner(&ent, ref->name, *ref, *nref );
	}
	if ( fShowFolderView )
	{
		Lock();
		FileListView->ShowFolderEaner(&ent, ref->name, *nref );
		Unlock();
	}
}

void basWindow::NodeWasRemoved( node_ref *nref )
{
	if ( nref )
		watch_node( nref, B_STOP_WATCHING, be_app_messenger );
	if ( fShowTumbnailView )
	{
		int a = TumbnailView->FindTumbnailByNode( *nref );
		if ( a >= 0 )
			TumbnailView->RemovedTumbnail(a);
	}
	if ( fShowFolderView )
	{
		int a = FileListView->FindItemByNode( *nref );
		if ( a >= 0 )
		{
			Lock();
			FileListView->RemoveItemInside(a);
			Unlock();
		}
	}
}

void basWindow::EntryWasChanged( entry_ref *eref, node_ref *nref, char *newname )
{
	if ( fShowTumbnailView )
	{
		int a = TumbnailView->FindTumbnailByNode( *nref );
		if ( a >= 0 )
			TumbnailView->UpdateTumbnail( a, eref, newname );
	}
	if ( fShowFolderView )
	{
		int a = FileListView->FindItemByNode( *nref );
		if ( a >= 0 )
			FileListView->UpdateItem( a, eref, newname );
	}
}

int32 basWindow::DirLooper( void *_msg )
{
	BMessage *msg = (BMessage*)_msg;
	basWindow *Me;
	entry_ref Ref;
	msg->FindRef("refs", &Ref);
	msg->FindPointer("basWindow", (void**)&Me );

	int32 _flags = msg->FindInt32("flags");
	bool show_fv = !(_flags&2);
	bool show_tv = !(_flags&4);

	BEntry entry(&Ref);
	BDirectory bdir(&entry);

	Me->Lock();

	if ( Me->fShowFolderView && show_fv)
		Me->FileListView->CleanListContent();
	if ( Me->fShowTumbnailView && show_tv)
	{
		Me->TumbnailWorker->Stop();
		Me->TumbnailView->CleanContents();
	}

	Me->TumbnailView->SetStatusLabel("");

	Me->Unlock();

	entry_ref *tmpref;
	while ( (tmpref = (entry_ref*)Me->imageNavigationList->FirstItem() ) )
	{
		Me->imageNavigationList->RemoveItem(tmpref);
		delete tmpref;
	}

	node_ref *noderef;
	while ( ( noderef = (node_ref*)Me->DirectoryNodeList.FirstItem() ) )
	{
//		watch_node( noderef, B_STOP_WATCHING, be_app_messenger );
		Me->DirectoryNodeList.RemoveItem(noderef);
		delete noderef;
	}

	BPath path(&entry);

	if ( strcmp( path.Path(), "/" ) == 0 )
	{
		if ( show_fv )
		{
			BVolumeRoster vRoster;
			vRoster.Rewind();
			BVolume volume;
			while (vRoster.GetNextVolume(&volume) == B_OK )
			{
				char name[B_FILE_NAME_LENGTH];
				volume.GetName(name);

				if ( strlen(name) > 0 )
				{
					Me->Lock();

					BDirectory root;
					volume.GetRootDirectory(&root);
					BEntry ent;
					root.GetEntry(&ent);
					node_ref nref;
					root.GetNodeRef(&nref);
					Me->FileListView->AddItem( new basFileListItem(Me->FileListView, &ent, nref, name));

					Me->Unlock();
				}
			}
		}
	}
	else
	{
		char buf[4096];
		dirent *dent;
		int count;
		while( ( count = bdir.GetNextDirents( (dirent*)buf, 4096)) > 0 )
		{
			dent = (dirent*)buf;
			while ( count -- )
			{
				if ( !strcmp( dent->d_name, "." ) )
					continue;
				entry_ref eref;
				eref.device = dent->d_pdev;
				eref.directory = dent->d_pino;
				eref.set_name( dent->d_name );
				BEntry ent(&eref);
				node_ref nref;
				nref.device = dent->d_dev;
				nref.node = dent->d_ino;

				if (!ent.IsDirectory() )
				{
					BNode node(&ent);
					BNodeInfo info(&node);
					char type[B_FILE_NAME_LENGTH];
					if ( info.GetType(type) == B_OK )
					{
						if ( strncasecmp(type, "image", 5) == 0 )
						{
							entry_ref *P_ref = new entry_ref;
							ent.GetRef(P_ref);
							Me->imageNavigationList->AddItem(P_ref);
						}
					}
				}

				if ( Me->fShowFolderView && show_fv)
				{
					Me->Lock();
					Me->FileListView->ShowFolderEaner( &ent, dent->d_name, nref );
					Me->Unlock();
				}
				if ( Me->fShowTumbnailView && show_tv )
				{
					BMessage * newmsg = new BMessage(B_LOAD_TUMBNAIL);
					newmsg->AddRef("_ref", &eref);
					newmsg->AddString("_name", dent->d_name);
					newmsg->AddInt32( "_dev", dent->d_dev);
					newmsg->AddInt64( "_ino", dent->d_ino );

					Me->TumbnailWorker->PostData(newmsg);
				}

				node_ref *newnoderef = new node_ref;
				*newnoderef = nref;
				Me->DirectoryNodeList.AddItem(newnoderef);
				watch_node( &nref, B_WATCH_ALL, be_app_messenger );

				dent = (dirent *)((char *)dent + dent->d_reclen);
			}
		}
	}

	if ( Me->fShowFolderView && show_fv)
	{
		Me->PostMessage(new BMessage(B_FOLDERVIEW_SORT_ITEMS));
	}

	delete msg;

	Me->DirLooperTID = -1;

	return 0;
}