#ifndef __basWindow_h__
#define __basWindow_h__

#include <Window.h>
#include <View.h>
#include <Bitmap.h>
#include <String.h>
#include <Entry.h>
#include <MenuBar.h>
#include <MenuField.h>
#include <ScrollView.h>
#include <TranslationKit.h>
#include <FilePanel.h>
#include <Button.h>
#include <TextControl.h>
#include "SplitPane.h"

#include "basPictureWindow.h"
#include "basTumbnailView.h"
#include "basFileListView.h"

class basFileListView;
class basWindow;
class basMainView;
class basTumbnailView;
class BColorPickerWindow;
class basApplication;
class basImageListWindow;
class basPluginInterfaceManager;
class basBitmapWorker;
class basTreeView;
class basMultiplePanel;

class basRegisterWindow : public BWindow
{
public:
	basRegisterWindow();

	virtual void MessageReceived( BMessage * );
	virtual bool QuitRequested();

private:
	BButton *OkButton, *CancelButton;
	BTextControl *NameText, *CodeText[4];
};

typedef void (*_SelectFunction)( basWindow *, BEntry *);

class basWindow : public BWindow
{
friend class basApplication;
friend class basFileListView;
friend class basTumbnailView;
friend class basPictureWindow;
public:
	basWindow();
	~basWindow();

	virtual void MessageReceived( BMessage *);
	virtual bool QuitRequested();
	virtual void FrameMoved( BPoint );
	virtual void FrameResized( float, float );
	void RefsReceived( entry_ref *);
	void ShowAboutBox();
	void SetFileListInvokeFunction( _SelectFunction F) { InvokeFunction = F; }
	void SetFileListSelectFunction( _SelectFunction F) { SelectFunction = F; }
	void DoSelectFunction( BEntry *e ) { if ( SelectFunction ) SelectFunction(this, e); }
	void DoInvokeFunction( BEntry *e ) { if ( InvokeFunction ) InvokeFunction(this, e); }
	void SetTumbnailSize( int );
	void FillFolderMenu( BEntry * );
	void SaveSettings();
	void EntryWasCreated( entry_ref *, node_ref * );
	void NodeWasRemoved( node_ref * );
	void EntryWasChanged( entry_ref *, node_ref *, char * );
	basPluginInterfaceManager *PluginInterfaceManager()
	{
		return _PluginInterfaceManager;
	}
private:
	static void DefaultOnInvoke( basWindow *, BEntry * );
	static void DefaultOnSelect( basWindow *, BEntry * );
	static void OnInvoke( basWindow *, BEntry * );
	static void OnSelect( basWindow *, BEntry * );
	void CleanAll();
	static int32 DirLooper( void * );
	void ReloadFileListView();
	void ReloadTumbnailView();
	void InvokeCleanAll();
	void InvokeListCameraContents( int );
	static int32 InvokeListCameraContentsThread( void* );
	basFileListView *FileListView;
	basPictureWindow *PictureWindow;
	basTumbnailView *TumbnailView;
	BMenuBar *MenuBar;
	BMenuField *FolderMenu;
	SplitPane *Split;
	bool fShowFolderMenu;
	bool fShowTumbnailView;
	bool fShowFolderView;
	_SelectFunction SelectFunction;
	_SelectFunction InvokeFunction;
	int TumbnailViewWheelMouseBehaviour;
	basTumbnailWorker *TumbnailWorker;
	thread_id DirLooperTID;
	BList *imageNavigationList;
	int SavePanelCopyMode;
	basImageListWindow *ImageListWindow;
	int FileListViewWheelMouseBehaviour;
	int FileListViewInvokeBehaviour;
	int FileListViewSelectBehaviour;
	basPluginInterfaceManager *_PluginInterfaceManager;
	basBitmapWorker *BitmapWorker;
	BList DirectoryNodeList;
	basTreeView *TreeView;
};

extern basWindow *_MainWindow;

#endif