#include <Window.h>
#include <Slider.h>
#include <Button.h>
#include <Bitmap.h>
#include "basPlugin.h"
#include "basCommon.h"
#include "Colors.h"

class basBrightnessWindow : public BWindow
{
public:
	basBrightnessWindow( BBitmap *sc, BBitmap **wb )
		: BWindow( BRect(100, 100, 400, 175), "Brightness Plugin", B_TITLED_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL, B_NOT_RESIZABLE|B_NOT_ZOOMABLE ),
			SourceBitmap(sc), WorkBitmap(wb)
	{
		BView *view = new BView( Bounds(), NULL, B_FOLLOW_ALL_SIDES, 0 );
		view->SetViewColor( BeBackgroundGrey );

		Brightness = new BSlider( BRect( 5, 5, 295, 30 ), NULL, "Brightness", NULL, 0, 200 );
		Brightness->SetHashMarks(B_HASH_MARKS_BOTTOM);
		Brightness->SetHashMarkCount(3);
		Brightness->SetPosition(0.5);
		view->AddChild(Brightness);

		OkButton = new BButton( BRect( 250, 45, 295, 70), NULL, "Ok", new BMessage(B_BUTTON_OK));
		view->AddChild(OkButton);

		did_it = false;

		AddChild(view);
		Show();
	}

	virtual void MessageReceived( BMessage *msg )
	{
		switch (msg->what)
		{
		case B_BUTTON_OK:
			{

			float brighten_fac = Brightness->Value();

			if (brighten_fac == 100)
			{
				memcpy((*WorkBitmap)->Bits(), SourceBitmap->Bits(), SourceBitmap->BitsLength());			
			}	
	
			long drows, dcols, srows, scols;
			unsigned long *spixptr, *spix1;
			unsigned char *dpixptr, *dpix;
			unsigned long slb, dlb;
			long i, j;
			unsigned char *a;

			srows = SourceBitmap->Bounds().IntegerHeight()+1;
			scols = SourceBitmap->Bounds().IntegerWidth()+1;
			drows = (*WorkBitmap)->Bounds().IntegerHeight();
			dcols = (*WorkBitmap)->Bounds().IntegerWidth();
		
			// Get values from image
			dpixptr = (unsigned char *)(*WorkBitmap)->Bits();
			dpix = (unsigned char *)dpixptr;
			spixptr = (unsigned long *)SourceBitmap->Bits();
			slb = SourceBitmap->BytesPerRow()/ 4;
			dlb = (*WorkBitmap)->BytesPerRow();

			float ar,ag,ab;

			//brighten/darken image
			for(i=0; i <= drows;i++)
			{
				spix1 = spixptr + i*slb;
				for (j =0; j <= dcols; j++)
				{
					a = (unsigned char *)(spix1 + j);	
					ar = (float)a[0]*(brighten_fac/100);
					ag = (float)a[1]*(brighten_fac/100);
					ab = (float)a[2]*(brighten_fac/100);

					dpix[0] = (unsigned char) ((ar >255) ? 255 : ar);
					dpix[1] = (unsigned char) ((ag >255) ? 255 : ag);
					dpix[2] = (unsigned char) ((ab >255) ? 255 : ab);
					dpix += 4;
				}		
				dpix = (dpixptr += dlb);
			}
			did_it = true;
			PostMessage(B_QUIT_REQUESTED);
			}
			break;
		default:
			BWindow::MessageReceived(msg);
		}
	}

	virtual bool QuitRequested()
	{
		if (!did_it)
		{
			delete *WorkBitmap;
			*WorkBitmap = NULL;
		}
		PostMessage(B_QUIT_REQUESTED);
		return true;
	}
private:
	BBitmap *SourceBitmap, **WorkBitmap;
	BButton *OkButton;
	BSlider *Brightness;
	bool did_it;
};

class basBrightnessPlugin : public basPlugin
{
public:
	basBrightnessPlugin(unsigned int);

	virtual int ProcessImage( BBitmap *, BBitmap ** );
	virtual int ProcessRefs( BMessage *)
	{
		return -1;
	}

	virtual void Configure()
	{
	}

	virtual const char *PluginName() { return "Brightness Plugin"; };
	virtual const int PluginVersion() { return ((0 << 24) | (1 << 16) | ( 0 << 8 ) | 0 ); };
	virtual const int PluginType() { return PLUGIN_TYPE_PROCESS_IMAGE; }
};

basBrightnessPlugin::basBrightnessPlugin(unsigned int params)
	: basPlugin(params)
{

}

int basBrightnessPlugin::ProcessImage( BBitmap *SourceBitmap, BBitmap **out )
{
	BRect bounds = SourceBitmap->Bounds();
	
	*out = new BBitmap(bounds,B_RGBA32);
	if (*out == NULL)
	{
		return -1;
	}

	basBrightnessWindow *wnd = new basBrightnessWindow( SourceBitmap, out );
	status_t status;
	wait_for_thread(wnd->Thread(), &status);

	return 0;
}

extern "C" basPlugin* ReturnPlugin( unsigned int params)
{
	return new basBrightnessPlugin(params);
}

int main()
{
	return 0;
}