#include <Entry.h>
#include <File.h>
#include <Path.h>
#include <stdio.h>
#include <Alert.h>
#include <TranslationKit.h>
#include <BitmapStream.h>
#include <View.h>
#include "basPlugin.h"
#include "basCommon.h"
#include "basHTMLConfig.h"
#include "htmlgenerator.h"

basHTMLGentPlugin::basHTMLGentPlugin(unsigned int ver)
	: basPlugin(ver), htmlConfig(0)
{
}

basHTMLGentPlugin::~basHTMLGentPlugin()
{
	if ( htmlConfig )
		htmlConfig->Quit();
}

int basHTMLGentPlugin::ProcessImage( BBitmap *, BBitmap ** )
{
	return -1;
}

int basHTMLGentPlugin::ProcessRefs( BMessage *msg )
{
	if ( !htmlConfig )
		htmlConfig = new basHTMLConfig(this, msg);
	return 0;
}

void basHTMLGentPlugin::Configure()
{
}

bool basHTMLGentPlugin::GenerateHTML( BMessage *refsList, const char *f )
{
	FILE *file;
	file = fopen( f, "w+b" );
	if ( !file )
	{
		(new BAlert("Error", "Failed to open and create the file you specified.", "Darn.."))->Go();
		return false;
	}

	fprintf( file, "<HTML>\n<HEAD>\n<TITLE>BeAndSee</TITLE>\n</HEAD>\n<BODY bgcolor=\"#ffffff\" text=\"#000000\">\n" );

	fprintf( file, "<P align=center>\n<TABLE border=1>\n" );

	bool openedRowOnce = false;

	BEntry _ent(f);
	BEntry _pent;
	_ent.GetParent(&_pent);
	BPath _ppath(&_pent);

	translator_id *translators;
	int32 num_translators;
	BTranslatorRoster *roster = BTranslatorRoster::Default();

	translator_id jpegid = 0;
	uint32 jpegtype = 0;
	
	roster->GetAllTranslators(&translators, &num_translators);
	for (int32 i=0; i<num_translators;i++)
	{
		const translation_format *fmts;
		int32 num_fmts;
		roster->GetOutputFormats(translators[i], &fmts, &num_fmts);
		for (int32 j=0;j<num_fmts;j++)
		{
			if ( !strcmp(fmts[j].MIME, "image/jpeg"))
			{
				jpegid = translators[i];
				jpegtype = fmts[j].type;
				goto out;
			}
		}
	}
out:;

	delete translators;

	if ( jpegid == 0 )
		return true;

	entry_ref ref;
	int i, j;
	for ( i = 0, j = 0; refsList->FindRef( "refs", i, &ref) == B_OK; i ++ )
	{
		if ( (j % 5) == 0 )
		{
			if ( openedRowOnce )
				fprintf( file, "</TR>\n");
			openedRowOnce = true;
			fprintf( file, "<TR>\n" );
		}

		BBitmap *tumb = _BitmapWorker->CreateTumbnail( &ref, 100, 100, true, true, false );

		BEntry entry(&ref);
		BPath path(&entry);

		if ( tumb )
		{
			char tbpath[ B_PATH_NAME_LENGTH ];
			sprintf(tbpath, "%s/tb%i.jpg", _ppath.Path(), j );
			BBitmapStream bstream(tumb);
			BFile outfile( tbpath, B_CREATE_FILE | B_WRITE_ONLY);
			if ( roster->Translate( jpegid, &bstream, NULL, &outfile, jpegtype) == B_OK)
			{
				fprintf( file, "<TD ALIGN=CENTER VALIGN=CENTER><A HREF=\"%s\"><IMG SRC=\"%s\" BORDER=0></IMG></A></TD>\n", path.Path(), tbpath );
				j++;
			}
			bstream.DetachBitmap(&tumb);
			delete tumb;
		}

	}

	if ( j > 0 )
		fprintf( file, "</TR>\n");

	fprintf( file, "</TABLE></P>\n");

	fprintf( file, "</BODY>\n</HTML>\n");

	return true;
}

extern "C" basPlugin* ReturnPlugin(unsigned int ver)
{
	return new basHTMLGentPlugin(ver);
}

int main()
{
	return 0;
}