#include <Window.h>
#include <Button.h>
#include <Bitmap.h>
#include <image.h>
#include <Alert.h>
#include <ListView.h>
#include <ScrollView.h>
#include <StringView.h>
#include <vector>
#include "basPlugin.h"
#include "basCommon.h"
#include "Colors.h"

#include "ImageManip.h"
#include "BBitmapAccessor.h"

#define B_USE		'buse'
#define B_SELECTED	'sltd'
#define B_CONFIG	'bcfg'

class BModalWindow : public BWindow
{
public:
	BModalWindow( BView *view )
		: BWindow( BRect(100, 100, 400, 375), "Configuration", B_TITLED_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL, B_NOT_RESIZABLE|B_NOT_ZOOMABLE )
	{
		AddChild(view);
		ResizeTo(view->Bounds().Width(), view->Bounds().Height());
		MoveTo(100, 100);
		Show();
	}
};

class basIMWindow : public BWindow
{
public:
	basIMWindow( BBitmap *sc, BBitmap **wb )
		: BWindow( BRect(100, 100, 400, 375), "LibImageManip Plugin", B_TITLED_WINDOW_LOOK, B_MODAL_APP_WINDOW_FEEL, B_NOT_RESIZABLE|B_NOT_ZOOMABLE ),
			SourceBitmap(sc), WorkBitmap(wb)
	{
		BView *view = new BView( Bounds(), NULL, B_FOLLOW_ALL_SIDES, 0 );
		view->SetViewColor( BeBackgroundGrey );

		sView0 = new BStringView( BRect(5, 5, 225, 20), NULL, "LibImageManip Plugin List");
		view->AddChild(sView0);

		pluginListView = new BListView( BRect(5, 25, 225 - B_V_SCROLL_BAR_WIDTH, 270), NULL );

		BBitmapAccessor accessor(*wb);
		accessor.SetDispose(false);

		const char *addonName;
		const char *addonInfo;
		const char *addonCategory;
		int32 addonVersion;
		image_addon_id *outList;
		int32 outCount = 0;
		if (Image_GetManipulators(&accessor, NULL, &outList, &outCount) == B_OK)
		{
			for (int i = 0; i < outCount; ++i)
			{
				if (Image_GetAddonInfo(outList[i], &addonName, &addonInfo,
						&addonCategory, &addonVersion) == B_OK)
				{
					pluginListView->AddItem( new BStringItem( addonName) );
					aid.push_back( outList[i] );
				}
			}
			delete[] outList;
		}

		pluginListView->SetSelectionMessage( new BMessage(B_SELECTED) );
		pluginListView->SetInvocationMessage( new BMessage(B_USE) );

		view->AddChild( new BScrollView(NULL, pluginListView, 0, 0, false, true) );

		UseButton = new BButton(BRect( 230, 25, 295, 50), NULL, "Use", new BMessage(B_USE));
		UseButton->SetEnabled(false);
		view->AddChild( UseButton );

		ConfigButton = new BButton(BRect( 230, 55, 295, 80), NULL, "Config", new BMessage(B_CONFIG));
		ConfigButton->SetEnabled(false);
		view->AddChild( ConfigButton );

		did_it = false;

		AddChild(view);
		Show();
	}

	virtual void MessageReceived( BMessage *msg )
	{
		switch (msg->what)
		{
		case B_SELECTED:
			UseButton->SetEnabled(pluginListView->CurrentSelection() >= 0);
			ConfigButton->SetEnabled(pluginListView->CurrentSelection() >= 0);
			break;
		case B_USE:
			{
				did_it = true;

				BBitmapAccessor src_accessor(*WorkBitmap);
				src_accessor.SetDispose(false);

				Image_Manipulate(aid[pluginListView->CurrentSelection()], &src_accessor, 0);

				PostMessage(B_QUIT_REQUESTED);
			}
			break;
		case B_CONFIG:
			{
				BView *bview;
				if ( Image_MakeConfigurationView(aid[pluginListView->CurrentSelection()], NULL, &bview) == B_OK )
				{
					new BModalWindow( bview );
				}
			}
			break;
		default:
			BWindow::MessageReceived(msg);
		}
	}

	virtual bool QuitRequested()
	{
		if (!did_it)
		{
			delete *WorkBitmap;
			*WorkBitmap = NULL;
		}
		return true;
	}
private:
	BBitmap *SourceBitmap, **WorkBitmap;
	bool did_it;
	BListView *pluginListView;
	BButton *UseButton, *ConfigButton;
	BStringView *sView0;
	vector<image_addon_id> aid;
};

class basIMPlugin : public basPlugin
{
public:
	basIMPlugin(unsigned int);
	virtual ~basIMPlugin();

	virtual int ProcessImage( BBitmap *, BBitmap ** );
	virtual int ProcessRefs( BMessage *)
	{
		return -1;
	}

	virtual void Configure()
	{
	}

	virtual const char *PluginName() { return "LibImageManip Plugin"; };
	virtual const int PluginVersion() { return ((0 << 24) | (1 << 16) | ( 0 << 8 ) | 0 ); };
	virtual const int PluginType() { return PLUGIN_TYPE_PROCESS_IMAGE; }

private:
	bool use_this_so;
};

basIMPlugin::basIMPlugin(unsigned int params)
	: basPlugin(params)
{
	int32 curVersion;
	int32 minVersion;
	Image_Version(&curVersion, &minVersion);

	if ( (curVersion < IMAGE_LIB_MIN_VERSION) || (minVersion > IMAGE_LIB_CUR_VERSION) )
	{
		use_this_so = false;
	}
	else
		use_this_so = true;

	Image_Init(NULL);
}

basIMPlugin::~basIMPlugin()
{
	Image_Shutdown();
}

int basIMPlugin::ProcessImage( BBitmap *SourceBitmap, BBitmap **out )
{
	if ( !use_this_so )
	{
		*out = NULL;
		(new BAlert("Error", "Couldn't load libimagemanip.so! This means that or you don't have LibImageManip correctly instaled or you don't have LibImageManip at all or you have a old version of libimagemanip.so", "Hm.."))->Go();
		return -1;
	}
	else
	{
		*out = new BBitmap(SourceBitmap);
		if (*out == NULL)
		{
			return -1;
		}

		basIMWindow *wnd = new basIMWindow( SourceBitmap, out );
		status_t status;
		wait_for_thread(wnd->Thread(), &status);

		return 0;
	}
}

extern "C" basPlugin* ReturnPlugin( unsigned int params)
{
	return new basIMPlugin(params);
}

int main()
{
	return 0;
}