#include "basPlugin.h"

class basNegativePlugin : public basPlugin
{
public:
	basNegativePlugin(unsigned int);

	virtual int ProcessImage( BBitmap *, BBitmap ** );
	virtual int ProcessRefs( BMessage *msg )
	{
		return -1;
	}

	virtual void Configure()
	{
	}

	virtual const char *PluginName() { return "Negative Plugin"; };
	virtual const int PluginVersion() { return ((0 << 24) | (1 << 16) | ( 0 << 8 ) | 0 ); };
	virtual const int PluginType() { return PLUGIN_TYPE_PROCESS_IMAGE; }
};

basNegativePlugin::basNegativePlugin(unsigned int params)
	: basPlugin(params)
{
}

int basNegativePlugin::ProcessImage( BBitmap *in, BBitmap **out )
{
	*out = new BBitmap( in->Bounds(), B_RGBA32);	

	if (*out == NULL) 
	{
		return -1;
	}

	long drows, dcols, srows, scols;
	unsigned long *spixptr, *spix1;
	unsigned char *dpixptr, *dpix;
	
	unsigned long slb, dlb;
	long i, j;
	unsigned char *a;

	srows = in->Bounds().IntegerHeight()+1;
	scols = in->Bounds().IntegerWidth()+1;
	drows = (*out)->Bounds().IntegerHeight();
	dcols = (*out)->Bounds().IntegerWidth();
		
	// Get values from image
	dpixptr = (unsigned char *)(*out)->Bits();
	dpix = (unsigned char *)dpixptr;
	spixptr = (unsigned long *)in->Bits();
	slb = in->BytesPerRow()/ 4;
	dlb = (*out)->BytesPerRow();
	
	//negate image
	for(i=0; i <= drows;i++)
	{
		spix1 = spixptr + i*slb;
		for (j =0; j <= dcols; j++)
		{		
			a = (unsigned char *)(spix1 + j);			
			dpix[0] = (unsigned char) (255.0 - (float)a[0]);
			dpix[1] = (unsigned char) (255.0 - (float)a[1]);
			dpix[2] = (unsigned char) (255.0 - (float)a[2]);
			dpix += 4;
		}		
		dpix = (dpixptr += dlb);
	}	

	return 0;
}

extern "C" basPlugin* ReturnPlugin(unsigned int params)
{
	return new basNegativePlugin(params);
}

int main()
{
	return 0;
}